--	cluster_counter.vhd
-- 	jamieson olsen <jamieson@fnal.gov>

-- 	counts up the number of valid clusters and delays them by seven
-- 	clock cycles so that the C_CNT output is valid when the COUNT_VALID
--	output of the cluster matcher modules goes high.  See engineeing note 
--	2002-02-07B for details...

-- 	Delays are explicitly stated as registers, although the Xilinx tools will 
--	convert the FFs to CLB shift registers.

-- 	*** revision history ***

--	07 Feb 2002:	created, functionally simulated OK.
--
--	12 Feb 2002:	scale output back to two bits.  00 = no clusters, 01 = 1 cluster, 
--					10 = 2 clusters, 11 = three or more clusters.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity cluster_counter is
	port(
		reset:		in	std_logic;						-- active low, async.
		clock:		in	std_logic;						-- 53MHz
		val_clust:	in	std_logic_vector(7 downto 0);	-- cluster valid bits
		c_cnt:		out	std_logic_vector(1 downto 0)	-- passed on to L1 formatter
	);	
end cluster_counter;

architecture cluster_counter_arch of cluster_counter is

	-- declare registers...

		-- signal sector_reg:		std_logic_vector(6 downto 0);  -- removed 7MAY04

	-- declare signals...

		--signal lut_a:		std_logic_vector(2 downto 0);
		--signal lut_b:		std_logic_vector(2 downto 0);
		--signal lut_c:		std_logic_vector(1 downto 0);
		--signal sum_c:		std_logic_vector(3 downto 0);
		signal lut:		std_logic_vector(3 downto 0);

--		signal dly_reg0:	std_logic_vector(1 downto 0);
		signal dly_reg1:	std_logic_vector(1 downto 0);
		signal dly_reg2:	std_logic_vector(1 downto 0);
		signal dly_reg3:	std_logic_vector(1 downto 0);
		signal dly_reg4:	std_logic_vector(1 downto 0);
		signal dly_reg5:	std_logic_vector(1 downto 0);
		signal dly_reg6:	std_logic_vector(1 downto 0);
	
begin

	-- combinatorial stuff...

		c_cnt(1 downto 0)	<=	dly_reg6(1 downto 0);

-- change to pipeline using case -Jan-11-2005 S. X. Wu
		--lut_a(2 downto 0)	<=	"000" when (val_clust(3 downto 0)="0000") else
		--						"011" when (val_clust(3 downto 0)="0001") else
		--						"001" when (val_clust(3 downto 0)="0010") else
		--						"010" when (val_clust(3 downto 0)="0011") else
		--						"001" when (val_clust(3 downto 0)="0100") else
		--						"010" when (val_clust(3 downto 0)="0101") else
		--						"010" when (val_clust(3 downto 0)="0110") else
		--						"011" when (val_clust(3 downto 0)="0111") else
		--						"001" when (val_clust(3 downto 0)="1000") else
		--						"010" when (val_clust(3 downto 0)="1001") else
		--						"010" when (val_clust(3 downto 0)="1010") else
		--						"011" when (val_clust(3 downto 0)="1011") else
		--						"010" when (val_clust(3 downto 0)="1100") else
		--						"011" when (val_clust(3 downto 0)="1101") else
		--						"011" when (val_clust(3 downto 0)="1110") else
		--						"100";

		--lut_b(2 downto 0)	<=	"000" when (val_clust(7 downto 4)="0000") else
		--						"011" when (val_clust(7 downto 4)="0001") else
		--						"001" when (val_clust(7 downto 4)="0010") else
		--						"010" when (val_clust(7 downto 4)="0011") else
		--						"001" when (val_clust(7 downto 4)="0100") else
		--						"010" when (val_clust(7 downto 4)="0101") else
		--						"010" when (val_clust(7 downto 4)="0110") else
		--						"011" when (val_clust(7 downto 4)="0111") else
		--						"001" when (val_clust(7 downto 4)="1000") else
		--						"010" when (val_clust(7 downto 4)="1001") else
		--						"010" when (val_clust(7 downto 4)="1010") else
		--						"011" when (val_clust(7 downto 4)="1011") else
		--						"010" when (val_clust(7 downto 4)="1100") else
		--						"011" when (val_clust(7 downto 4)="1101") else
		--						"011" when (val_clust(7 downto 4)="1110") else
		--						"100";

		--sum_c(3 downto 0)	<=	('0' & lut_a(2 downto 0)) + ('0' & lut_b(2 downto 0));

		--lut_c(1 downto 0)	<=	"00" when (sum_c(3 downto 0)="0000") else
		--						"01" when (sum_c(3 downto 0)="0001") else
		--						"10" when (sum_c(3 downto 0)="0010") else
		--						"11";


	-- sequential stuff...

--		shift_sector: process(clock, reset)		change to sync reset Aug-06-04
		shift_sector: process(clock)
		 begin
			if rising_edge(clock) then
			 if (reset='0') then
--				dly_reg0(1 downto 0)	<= "00";
				lut	<= "0000";
				dly_reg1(1 downto 0)	<= "00";
				dly_reg2(1 downto 0)	<= "00";
				dly_reg3(1 downto 0)	<= "00";
				dly_reg4(1 downto 0)	<= "00";
				dly_reg5(1 downto 0)	<= "00";
				dly_reg6(1 downto 0)	<= "00";
			 else
--				dly_reg0(1 downto 0)	<= lut_c(1 downto 0);
            case val_clust(3 downto 0) is
				  when "0000" => lut(1 downto 0) <= "00";
				  when "0001" | "0010" | "0100" | "1000" => lut(1 downto 0) <= "01";
				  when "0111" | "1011" | "1101" | "1110" | "1111" => lut(1 downto 0) <= "11";
				  when others => lut(1 downto 0) <= "10";
				end case;
            case val_clust(7 downto 4) is
				  when "0000" => lut(3 downto 2) <= "00";
				  when "0001" | "0010" | "0100" | "1000" => lut(3 downto 2) <= "01";
				  when "0111" | "1011" | "1101" | "1110" | "1111" => lut(3 downto 2) <= "11";
				  when others => lut(3 downto 2) <= "10";
				end case;
--				dly_reg1(1 downto 0)	<= dly_reg0(1 downto 0);
            case lut is
				  when "0000" => dly_reg1 <= "00";
				  when "0001" | "0100" => dly_reg1 <= "01";
				  when "0101" | "1000" | "0010" => dly_reg1 <= "10";
				  when others => dly_reg1 <= "11";
				end case;
				dly_reg2(1 downto 0)	<= dly_reg1(1 downto 0);
				dly_reg3(1 downto 0)	<= dly_reg2(1 downto 0);
				dly_reg4(1 downto 0)	<= dly_reg3(1 downto 0);
				dly_reg5(1 downto 0)	<= dly_reg4(1 downto 0);
				dly_reg6(1 downto 0)	<= dly_reg5(1 downto 0);
			 end if;
		  end if;
		end process;

end cluster_counter_arch;
