-- comp_a.vhd
-- jamieson olsen <jamieson@fnal.gov>
-- 28 Jan 2002

-- revised 31 Jan 2002 : add another bit to the width.
-- revised 14 Aug 2002 : expand the input busses by 1 bit.
-- revised 21 Oct 2002 : add reset

-- the msb of each of the input words is an active high valid bit.

-- A(4..0)  	B(4..0)		Q(4..0)
-- -------		-------		----------------------------------
-- 1aaaa		1bbbb		if (aaaa<=bbbb) pass 01aaaa, else 11bbbb *
-- 1aaaa		0----		pass 01aaaa
-- 0----		1bbbb		pass 11bbbb
-- 0----		0----		pass 000000

-- * if aaaa and bbbb are equal, always choose aaaa.

library ieee;
use ieee.std_logic_1164.all;

entity comp_a is
	port(
		clock:		in	std_logic;
		reset:		in	std_logic;
		a: 			in 	std_logic_vector(4 downto 0);	-- valid + |difference|
		b:			in	std_logic_vector(4 downto 0);	-- valid + |difference|
		q:			out	std_logic_vector(5 downto 0)	-- sel + valid + |difference|
	);
end comp_a;

architecture comp_a_arch of comp_a is

	-- define registers...

		signal q_reg: 	std_logic_vector(5 downto 0);

	-- define signals

		signal sel_a:		std_logic;
		signal sel_b:		std_logic;
		signal smaller_a:	std_logic;
		signal mux_out:		std_logic_vector(5 downto 0);
	
 begin

	-- combinatorial stuff...

		-- compare values...

			smaller_a	<=	'1' when (a(3 downto 0) < b(3 downto 0)) else
							'1' when (a(3 downto 0) = b(3 downto 0)) else
							'0';
						
			sel_a		<=	'1' when (a(4)='1' and b(4)='1' and smaller_a='1') else
							'1' when (a(4)='1' and b(4)='0'                  ) else
							'0';
	
			sel_b		<=	'1' when (a(4)='1' and b(4)='1' and smaller_a='0') else
							'1' when (a(4)='0' and b(4)='1'                  ) else
							'0';

		-- mux operation (sel_a, sel_b)

			-- 10 = pass 0+A
			-- 01 = pass 1+B
			-- 00 = neither are valid, pass zeros.
			-- 11 = can't happen, but send zeros anyway
	
			mux_out(5 downto 0)	<= 	('1'&b(4 downto 0)) when (sel_a='0' and sel_b='1') else
									('0'&a(4 downto 0)) when (sel_a='1' and sel_b='0') else
									"000000";
	
			q(5 downto 0)		<= q_reg(5 downto 0);


	-- sequential stuff...

--		process(clock, reset) 	 	change to sync reset Aug-06-04
		process(clock) 
		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				q_reg(5 downto 0)	<= "000000";
			 else
				q_reg(5 downto 0)	<= mux_out(5 downto 0);
			 end if;
		  end if;
		end process;

end comp_a_arch;

