-- diff.vhd
-- jamieson olsen <jamieson@fnal.gov>
-- 28 Jan 2002

-- revised 31 Jan 2002: make the output 4 bits wide.

-- revised 12 Feb 2002: the lookup tables in this design are 
--						a bottleneck... try to simplify.

-- revised 21 Oct 2002:	add reset

-- revised 14 Aug 2002:	expand the output bus by one bit.

-- ABS_OUT(3..0) is equal to |cft-cps|.  if this difference is equal to or 
-- greater than 15 this value will be set to 1111.

-- ABS_OUT(4)	 is a valid bit.  It is a logical AND of the CFT and CPS bits.

-- cft(5) is an active high valid bit
-- cps(5) is an active high valid bit

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity diff is
	port(
		clock:		in	std_logic;
		reset:		in	std_logic;
		cft: 		in 	std_logic_vector(5 downto 0);	-- valid + projected track phi
		cps:		in	std_logic_vector(5 downto 0);	-- valid + cluster centroid
		abs_out:	out	std_logic_vector(4 downto 0)	-- valid + |cft-cps|
	);
end diff;

architecture diff_arch of diff is

	-- define registers...

		signal abs_reg: 	std_logic_vector(3 downto 0);
		signal val_reg:		std_logic;
	
	-- define signals

		signal temp_diff:	std_logic_vector(5 downto 0);
		signal temp_abs:	std_logic_vector(3 downto 0);

	-- declare components...
		
 begin

	-- combinatorial stuff...


		abs_out(4 downto 0)		<= (val_reg & abs_reg(3 downto 0));

		-- the difference is computed as a 2's complement where
		-- +1 =  1, +2 =  2, +3 =  3 ....
		-- -1 = 63, -2 = 62, -3 = 61 ....

		temp_diff(5 downto 0)	<= ('0' & cft(4 downto 0)) - ('0' & cps(4 downto 0));

		temp_abs(3 downto 0)	<= 	temp_diff(3 downto 0) when (temp_diff(5 downto 0) < "01111") else
									"0001" when (temp_diff(5 downto 0)="111111") else	-- cft-cps = -1
									"0010" when (temp_diff(5 downto 0)="111110") else	-- cft-cps = -2
									"0011" when (temp_diff(5 downto 0)="111101") else	-- cft-cps = -3
									"0100" when (temp_diff(5 downto 0)="111100") else	-- cft-cps = -4
									"0101" when (temp_diff(5 downto 0)="111011") else	-- cft-cps = -5
									"0110" when (temp_diff(5 downto 0)="111010") else	-- cft-cps = -6
									"0110" when (temp_diff(5 downto 0)="111001") else	-- cft-cps = -7
									"0110" when (temp_diff(5 downto 0)="111000") else	-- cft-cps = -8
									"0110" when (temp_diff(5 downto 0)="110111") else	-- cft-cps = -9
									"0110" when (temp_diff(5 downto 0)="110110") else	-- cft-cps = -10
									"0110" when (temp_diff(5 downto 0)="110101") else	-- cft-cps = -11
									"0110" when (temp_diff(5 downto 0)="110100") else	-- cft-cps = -12
									"0110" when (temp_diff(5 downto 0)="110011") else	-- cft-cps = -13
									"0110" when (temp_diff(5 downto 0)="110010") else	-- cft-cps = -14
									"1111";												-- |cft-cps| >= 15



	-- sequential stuff...

--		process(clock, reset) 	 	change to sync reset Aug-06-04
		process(clock) 
		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				val_reg <= '0';
				abs_reg(3 downto 0) <= "0000";
			 else
				val_reg 			<= cft(5) and cps(5);
				abs_reg(3 downto 0)	<= temp_abs(3 downto 0);
			 end if;
		  end if;
		end process;
														
end diff_arch;

