-- muon_fe.vhd
-- front end module for HIGH, MED, and LOW Pt modules
-- in the L1 Muon Track sorter module for DFEA.
-- ENG NOTE: 2002-01-14B

-- jamieson olsen <jamieson@fnal.gov>
-- 14 Jan 2002: created, simulated.
-- 08 Aug 2002:	added START input.  START should be asserted one clock cycle 
--				BEFORE the first track
-- 21 Oct 2002:	add reset

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity muon_fe is
	port(
		din: 	in 	std_logic_vector(10 downto 0);	-- VCEeePppppp
		start:	in	std_logic;
		clock:	in	std_logic;
		reset:	in	std_logic;
		sel:	in	std_logic;		-- active high
		dout: 	out std_logic_vector(10 downto 0)
    );
end muon_fe;


architecture muon_fe_arch of muon_fe is

	-- define registers...
	
		signal din_reg:		std_logic_vector(10 downto 0);
		signal wptr_reg:	std_logic_vector( 3 downto 0);	-- write pointer
		signal rptr_reg:	std_logic_vector( 3 downto 0);	-- read pointer

	-- define signals

		signal start_reg:		std_logic;

	-- declare components...

		component ram11x16d 
		 	port (
				wclk:		in	std_logic;						-- sync write clk
				we:			in	std_logic;						-- active high
				addra:		in	std_logic_vector( 3 downto 0);	-- write port address
				din:		in	std_logic_vector(10 downto 0);
				addrb:		in	std_logic_vector( 3 downto 0);	-- read port address
				dout:		out	std_logic_vector(10 downto 0)	-- read data
			);
		end component;
		
 begin

	-- combinatorial stuff...
-- input registers removed to reduce latency	Aug-09-04
				din_reg(10 downto 0) <= din(10 downto 0);

	-- sequential stuff...
	
--		pointers: process(clock, reset) change to sync reset Aug-06-04
	  pointers: process(clock)
	  begin
		 if rising_edge(clock) then
			if (reset='0') then
-- input registers removed to reduce latency	Aug-09-04
--				din_reg(10 downto 0) <= "00000000000";
				wptr_reg(3 downto 0) <= "0000";
				rptr_reg(3 downto 0) <= "0000";
				start_reg <= '0';

			else
-- input registers removed to reduce latency	Aug-09-04
--				din_reg(10 downto 0) <= din(10 downto 0);

				if (start='1') then
					wptr_reg(3 downto 0)	<= "0000";
				elsif (wptr_reg(3 downto 0)="0101") then
					wptr_reg(3 downto 0)	<= "0000";
				else
					wptr_reg(3 downto 0) 	<= wptr_reg(3 downto 0) + 1;
				end if;
				start_reg <= start;	
				if (start_reg='1') then
					rptr_reg(3 downto 0)	<= "0000";
				elsif (sel='1') then
					rptr_reg(3 downto 0) 	<= rptr_reg(3 downto 0) + 1;
				else
					rptr_reg(3 downto 0) 	<= rptr_reg(3 downto 0);
				end if;

			 end if;
		  end if;
		end process;

	-- instantiate components...

		track_ram: ram11x16d 
		 	port map(
				wclk				=> clock,
				we					=> '1',
				addra(3 downto 0)	=> wptr_reg(3 downto 0),
				din(10 downto 0)	=> din_reg(10 downto 0),
				addrb(3 downto 0)	=> rptr_reg(3 downto 0),
				dout(10 downto 0)	=> dout(10 downto 0)
			);
		
end muon_fe_arch;

