-- sum_pt.vhd
-- jamieson olsen <jamieson@fnal.gov>

--	reference my engineering note 2002-02-08B!

-- *** revision history *** 
--
-- 08 Feb 2002:	  	created and functionally simulated.
-- 14 Aug 2002: 	change Pt bin definition: 00=max, 11=low

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity sum_pt is
	port(
		reset:			in 	std_logic;
		clock:			in 	std_logic;
		start:			in	std_logic;		

		max_ext_pt:		in	std_logic_vector(2 downto 0);
		high_ext_pt:	in	std_logic_vector(2 downto 0);
		med_ext_pt:		in	std_logic_vector(2 downto 0);
		low_ext_pt:		in	std_logic_vector(2 downto 0);

		max_valid:		in	std_logic;
		high_valid:		in	std_logic;
		med_valid:		in	std_logic;
		low_valid:		in	std_logic;

		total_pt:		out	std_logic_vector(7 downto 0)
	);
end sum_pt;

architecture sum_pt_reg of sum_pt is

	-- define signals...

		signal max_sum:		std_logic_vector(7 downto 0);
		signal high_sum:	std_logic_vector(7 downto 0);
		signal med_sum:		std_logic_vector(7 downto 0);
		signal low_sum:		std_logic_vector(7 downto 0);

		signal max_pt:		std_logic_vector(1 downto 0);
		signal high_pt:		std_logic_vector(1 downto 0);
		signal med_pt:		std_logic_vector(1 downto 0);
		signal low_pt:		std_logic_vector(1 downto 0);

	-- define registers...

		signal max_reg:		std_logic_vector(7 downto 0);
		signal high_reg:	std_logic_vector(7 downto 0);
		signal med_reg:		std_logic_vector(7 downto 0);
		signal low_reg:		std_logic_vector(7 downto 0);
		signal total_reg:	std_logic_vector(7 downto 0);

		signal start_reg:	std_logic_vector(1 downto 0);

	-- declare components...

		component pt_adder
			port(
				reset:		in 	std_logic;
				clock:		in 	std_logic;
				pt:			in	std_logic_vector(1 downto 0);
				ext_pt:		in	std_logic_vector(2 downto 0);
				valid:		in	std_logic;
				start:		in	std_logic;
				sum_pt:		out	std_logic_vector(7 downto 0)
			);
		end component;

 begin

	-- combinatorial stuff...

		max_pt(1 downto 0)	<= "00";
		high_pt(1 downto 0)	<= "01";
		med_pt(1 downto 0)	<= "10";
		low_pt(1 downto 0)	<= "11";

		total_pt(7 downto 0)	<=	total_reg(7 downto 0);

	-- sequential stuff...

--	process(clock, reset)	change to sync reset Aug-06-04
	process(clock)
	 	 begin
	
			if rising_edge(clock) then
			 if (reset='0') then 
	
				max_reg(7 downto 0)		<= 	"00000000";
				high_reg(7 downto 0)	<= 	"00000000";
				med_reg(7 downto 0)		<= 	"00000000";
				low_reg(7 downto 0)		<= 	"00000000";
				start_reg(1 downto 0)	<=	"00";
				total_reg(7 downto 0)	<= 	"00000000";
			 else
				start_reg(0)	<= 	start;
				start_reg(1)	<=	start_reg(0);
	
				if (start_reg(0)='1') then							-- load
					max_reg(7 downto 0)		<= max_sum(7 downto 0);
					high_reg(7 downto 0)	<= high_sum(7 downto 0);
					med_reg(7 downto 0)		<= med_sum(7 downto 0);
					low_reg(7 downto 0)		<= low_sum(7 downto 0);
				else												-- shift
					max_reg(7 downto 0)		<= "00000000";
					high_reg(7 downto 0)	<= max_reg(7 downto 0);
					med_reg(7 downto 0)		<= high_reg(7 downto 0);
					low_reg(7 downto 0)		<= med_reg(7 downto 0);
				end if;
	
				if (start_reg(1)='1') then						-- load
					total_reg(7 downto 0)	<=	low_reg(7 downto 0);	
				else												-- add
					total_reg(7 downto 0)	<=	low_reg(7 downto 0) + total_reg(7 downto 0);
				end if;
	
			 end if;
		  end if;
		end process;

	-- instantiate components...

		max_adder: pt_adder
			port map(
				reset				=> 	reset,
				clock				=>	clock,
				pt(1 downto 0)		=>	max_pt(1 downto 0),
				ext_pt				=>	max_ext_pt(2 downto 0),
				valid				=>	max_valid,
				start				=>	start,
				sum_pt(7 downto 0)	=>	max_sum(7 downto 0)
			);

		high_adder: pt_adder
			port map(
				reset				=> 	reset,
				clock				=>	clock,
				pt(1 downto 0)		=>	high_pt(1 downto 0),
				ext_pt				=>	high_ext_pt(2 downto 0),
				valid				=>	high_valid,
				start				=>	start,
				sum_pt(7 downto 0)	=>	high_sum(7 downto 0)
			);

		med_adder: pt_adder
			port map(
				reset				=> 	reset,
				clock				=>	clock,
				pt(1 downto 0)		=>	med_pt(1 downto 0),
				ext_pt				=>	med_ext_pt(2 downto 0),
				valid				=>	med_valid,
				start				=>	start,
				sum_pt(7 downto 0)	=>	med_sum(7 downto 0)
			);

		low_adder: pt_adder
			port map(
				reset				=> 	reset,
				clock				=>	clock,
				pt(1 downto 0)		=>	low_pt(1 downto 0),
				ext_pt				=>	low_ext_pt(2 downto 0),
				valid				=>	low_valid,
				start				=>	start,
				sum_pt(7 downto 0)	=>	low_sum(7 downto 0)
			);

end sum_pt_reg;
