-- L2pipeline.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- basically a 80-bit wide variable depth shift register.  constructed out of 10 dual port rams.
-- data is written in on port A, and read out on port B.
-- read/write pointers integrated into this module; dout is equal to din, but delayed by DEPTH crossings
-- *** this pipeline always runs, and cannot be turned off. ***  see engineering note 2002-09-17a for details.

-- *** revision history ***
--
-- 16 Sept 2002: created, functionally simulated.
-- 26 Sept 2002: shorten from 96 bits to 80 bits wide.
--  2 Aug  2003: found *BUG* where mux_sel_reg was being clocked by rising AND falling edges of clock,
--  			 this caused momentary glitching when output mux transitoned between high rams and low
--				 rams.  the first output word (which contains tick and turn count) would be from the
--				 wrong bank of rams, or 32 events off!

library ieee,mylib;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use mylib.dfea_pack.all;
--library virtex;

entity L2_pipeline is
	port(
		reset:				in	std_logic;	  					-- active low, async.
		clock:				in	std_logic;	  					-- 53MHz
		bor:				in	std_logic;						-- mark first word of seven
		din:				in	std_logic_vector(79 downto 0);	-- raw L1 data to be stored
--		depth:				in	std_logic_vector( 5 downto 0);	-- pipeline depth, in 132ns crossings.
-- this avoids wrap around problem Nov-2-2004
		depth:				in	std_logic_vector( 8 downto 0);	-- pipeline depth, in 19ns crossings.

		dout:				out	std_logic_vector(79 downto 0)	-- delayed raw L1 data out
	);
end L2_pipeline;

architecture L2pipeline_arch of L2_pipeline is

	-- declare registers...

--		signal mux_sel_reg:		std_logic;
		signal waddr_reg:	std_logic_vector(9 downto 0);
		signal raddr_reg:	std_logic_vector(9 downto 0);

	-- declare signals...

		-- signal read_addr:	std_logic_vector(8 downto 0);
--		signal l_ena:		std_logic;
--		signal h_ena:		std_logic;
--		signal l_enb:		std_logic;
--		signal h_enb:		std_logic;
		-- signal logic:		std_logic_vector(1 downto 0);

--		signal lram0_dob:	std_logic_vector(15 downto 0);
--		signal lram1_dob:	std_logic_vector(15 downto 0);
--		signal lram2_dob:	std_logic_vector(15 downto 0);
--		signal lram3_dob:	std_logic_vector(15 downto 0);
--		signal lram4_dob:	std_logic_vector(15 downto 0);
		-- signal lram5_dob:	std_logic_vector(15 downto 0);

--		signal hram0_dob:	std_logic_vector(15 downto 0);
--		signal hram1_dob:	std_logic_vector(15 downto 0);
--		signal hram2_dob:	std_logic_vector(15 downto 0);
--		signal hram3_dob:	std_logic_vector(15 downto 0);
--		signal hram4_dob:	std_logic_vector(15 downto 0);
		--signal hram5_dob:	std_logic_vector(15 downto 0);

	-- declare components...

begin

	-- combinatorial stuff...

--		l_ena <= not waddr_reg(8);
--		h_ena <= waddr_reg(8);
--		l_enb <= not raddr_reg(8);
--		h_enb <= raddr_reg(8);

--		dout(79 downto 0) <= (lram4_dob(15 downto 0) & lram3_dob(15 downto 0) &
--							  lram2_dob(15 downto 0) & lram1_dob(15 downto 0) & lram0_dob(15 downto 0)) when mux_sel_reg='0' else
--							 (hram4_dob(15 downto 0) & hram3_dob(15 downto 0) &
--							  hram2_dob(15 downto 0) & hram1_dob(15 downto 0) & hram0_dob(15 downto 0));
  
	-- sequential stuff...
		waddr_reg(9) <= '0';
		raddr_reg(9) <= '0';
--		process(clock, reset)  reset removed Aug-06-04
		process(clock)
		 begin
			if rising_edge(clock) then
--			 if (reset='0') then
--				waddr_reg(8 downto 0) <= "000000000";
--				raddr_reg(8 downto 0) <= "000000000";
--				mux_sel_reg			  <= '0';
--			 else
--				if (bor='1') then
--					waddr_reg(8 downto 3) <= waddr_reg(8 downto 3) + 1;
--					waddr_reg(2 downto 0) <= "000";
-- this simplifies the logic S.X.Wu Oct-13-2004
--					raddr_reg(8 downto 3) <= waddr_reg(8 downto 3) - depth(5 downto 0) + 1;
--					raddr_reg(8 downto 3) <= waddr_reg(8 downto 3) - depth(5 downto 0);
-- raddr_reg(2 downbto 0) and w_addr(2 downto 0) are the same Oct-13-2004
--					raddr_reg(2 downto 0) <= "000";
--				else
--					waddr_reg(2 downto 0) <= waddr_reg(2 downto 0) + 1;
--					raddr_reg(2 downto 0) <= raddr_reg(2 downto 0) + 1;
--				end if;

--				mux_sel_reg <= raddr_reg(8);

--			 end if;
          raddr_reg(8 downto 0) <= raddr_reg(8 downto 0) + 1;
			 waddr_reg(8 downto 0) <= raddr_reg(8 downto 0) + depth;
		  end if;
		end process;
--raddr_reg(2 downto 0) <= waddr_reg(2 downto 0);



	-- instantiate components...
-- using ramb16 instead of ramb4	  Aug-06-04
i_ram0: RAMB16_S36_S36
		   port map(
			   dia => din(31 downto 0),
				dib => gnd32,
				dipa => gnd4,
				dipb => gnd4,
				ena => vcc,
				enb => vcc,
				wea => vcc,
				web => gnd,
				ssra => gnd,
				ssrb => gnd,
				clka => clock,
				clkb => clock,
				addra => waddr_reg(8 downto 0),
				addrb => raddr_reg(8 downto 0),
				dob => dout(31 downto 0));
i_ram1: RAMB16_S36_S36
		   port map(
			   dia => din(63 downto 32),
				dib => gnd32,
				dipa => gnd4,
				dipb => gnd4,
				ena => vcc,
				enb => vcc,
				wea => vcc,
				web => gnd,
				ssra => gnd,
				ssrb => gnd,
				clka => clock,
				clkb => clock,
				addra => waddr_reg(8 downto 0),
				addrb => raddr_reg(8 downto 0),
				dob => dout(63 downto 32));
i_ram2: RAMB16_S18_S18
		   port map(
			   dia => din(79 downto 64),
				dib => gnd16,
				dipa => gnd2,
				dipb => gnd2,
				ena => vcc,
				enb => vcc,
				wea => vcc,
				web => gnd,
				ssra => gnd,
				ssrb => gnd,
				clka => clock,
				clkb => clock,
				addra => waddr_reg,
				addrb => raddr_reg,
				dob => dout(79 downto 64));
end L2pipeline_arch;
