library IEEE,mylib;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.dfea_pack.all;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ctrl_decode is
    Port ( clock : in std_logic;
           eof:	  in std_logic;
           bus_cmd:	  in std_logic;
		     version:   in std_logic_vector(15 downto 0);					
		     bus_o:   in std_logic_vector(15 downto 0);					
		     ctrl_bus:   inout std_logic_vector(15 downto 0);					
-- capture input data, muon out and L1 out data
           cap_l1 : out std_logic;
-- capture L2CPS and L2CFT data
           cap_l2 : out std_logic;
		     ctrl_q : OUT std_logic_vector(3 downto 0);
           lock : out std_logic;
           reset : out std_logic;
           u10 : in std_logic;
           wr_mobo : out std_logic;
           rd_mobo : out std_logic;
           rd_eqs_vers : out std_logic;
           rd_status : out std_logic;
           rd_input : out std_logic;
           rd_muon : out std_logic;
           rd_CTOC : out std_logic;
		     diag_sel : OUT std_logic_vector(3 downto 0);
		     mem_depth : OUT std_logic_vector(8 downto 0);
           addr : out std_logic_vector(9 downto 0);
           w_data : out std_logic_vector(15 downto 0);
			  r_data : out std_logic_vector(15 downto 0)
			  );
end ctrl_decode;

architecture my_arch of ctrl_decode is
signal R_CONFU: std_logic_vector(9 downto 0);
signal R_in: std_logic_vector(3 downto 0);
signal R_mu: std_logic_vector(6 downto 0);
signal R_L1: std_logic_vector(5 downto 0);
signal R_CPS: std_logic_vector(4 downto 0);
signal R_CFT: std_logic_vector(3 downto 0);
signal R_mobo: std_logic_vector(6 downto 0);
signal bus_cmd_q: std_logic;
signal rd_version: std_logic;
signal wr_depth: std_logic;
signal rd_depth: std_logic;
signal depth: std_logic_vector(6 downto 0);
signal lock_cntr: std_logic_vector(5 downto 0);
signal ctrl_in: std_logic_vector(3 downto 0);
signal cmd_dl: std_logic_vector(6 downto 0);
signal bus_rwq: std_logic;
signal bus_oe_n: std_logic;
signal cap_l1_i: std_logic;
signal cap_l2_i: std_logic;
signal reset_i: std_logic;
--signal cmd: std_logic_vector(15 downto 0);
signal ctrl_bus_q: std_logic_vector(15 downto 0);
signal bus_t: std_logic_vector(15 downto 0);
attribute IOB: string;
attribute IOB of bus_t: signal is "true";
attribute IOB of ctrl_bus_q: signal is "true";
begin
R_CONFU <= R_CONFU8(9 downto 0) when u10 = '0' else R_CONFU10(9 downto 0);
R_in <= R_up_in(9 downto 6) when u10 = '0' else R_down_in(9 downto 6);
R_mu <= R_up_mu(9 downto 3) when u10 = '0' else R_down_mu(9 downto 3);
R_L1 <= R_up_L1(9 downto 4) when u10 = '0' else R_down_L1(9 downto 4);
R_CPS <= R_up_CPS(9 downto 5) when u10 = '0' else R_down_CPS(9 downto 5);
R_CFT <= R_up_CFT(9 downto 6) when u10 = '0' else R_down_CFT(9 downto 6);
R_mobo <= R_up_mobo(9 downto 3) when u10 = '0' else R_down_mobo(9 downto 3);
-- ctrl_bus signals
g_ctrl_bus: for i in bus_o'range generate
  ctrl_bus(i) <= bus_o(i) when bus_t(i) = '0' else 'Z';
end generate;
addr <= ctrl_bus_q(9 downto 0);
lock <= lock_cntr(5);
process(clock)
begin
  if(clock'event and clock = '1')then
    bus_cmd_q <= bus_cmd;
    if(cmd_dl(0) = '1' and bus_cmd_q = '0')then
	   lock_cntr <= (others => '0');
	 elsif(lock_cntr(5) = '0')then
	   lock_cntr <= lock_cntr + 1;
	 end if;
    if(cmd_dl = "0000000" or (cmd_dl(0) = '1' and bus_cmd_q = '0'))then
	   cmd_dl <= "0000100";
	 else
	   cmd_dl <= cmd_dl(5 downto 0) & cmd_dl(6);
	 end if;
	 ctrL_bus_q <= ctrl_bus;
    if(cmd_dl(0) = '1')then
-- command cycle is cycle(6)
		diag_sel <= ctrl_bus_q(11 downto 8);
	   cap_l1_i <= ctrl_bus_q(7);
	   cap_l2_i <= ctrl_bus_q(6);
	   reset_i <= ctrl_bus_q(4);
		ctrl_in <= ctrl_bus_q(3 downto 0);
	 end if;
	 if(eof = '1')then
		cap_l1 <= cap_l1_i;
		cap_l2 <= cap_l2_i;
		ctrl_q <= ctrl_in;
		reset <= not reset_i;
	 end if;
--    if(cmd_dl(0) = '1')then
-- address cycle follows command cycle
--	   cmd <= ctrl_bus;
--	 end if;
--    if(cmd_dl(1) = '1')then
-- data cycle follows address cycle if any
--	   w_data_i <= ctrl_bus;
--	 end if;
	 for i in bus_t'range loop
	   bus_t(i) <= bus_oe_n;
	 end loop;
	 bus_rwq <= cmd_dl(0) and bus_cmd_q;
	 bus_oe_n <= not(lock_cntr(5) and ((bus_rwq and not ctrl_bus_q(15)) or cmd_dl(2)));
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '0' and ctrl_bus_q(9 downto 6) = R_in)then
	   rd_input <= '1';
	 else
	   rd_input <= '0';
	 end if;
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '0' and ctrl_bus_q(9 downto 3) = R_mu)then
	   rd_muon <= '1';
	 else
	   rd_muon <= '0';
	 end if;
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '0' and 
	   (ctrl_bus_q(9 downto 4) = R_L1 or
		 ctrl_bus_q(9 downto 5) = R_CPS or
		 (ctrl_bus_q(9 downto 6) = R_CFT and ctrl_bus_q(5 downto 3) /= "111")))then
	   rd_CTOC <= '1';
	 else
	   rd_CTOC <= '0';
	 end if;
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '0' and ctrl_bus_q(9 downto 3) = R_mobo)then
	   rd_mobo <= not ctrl_bus_q(2) and not ctrl_bus_q(1);
	   rd_eqs_vers <= ctrl_bus_q(2) or ctrl_bus_q(1);
	 else
	   rd_mobo <= '0';
	   rd_eqs_vers <= '0';
	 end if;
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '1' and ctrl_bus_q(9 downto 3) = R_mobo)then
	   wr_mobo <= '1';
	 else
	   wr_mobo <= '0';
	 end if;
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '1' and ctrl_bus_q(9 downto 0) = R_L2pipe_depth(9 downto 0))then
	   wr_depth <= '1';
	 else
	   wr_depth <= '0';
	 end if;
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '0' and ctrl_bus_q(9 downto 0) = R_L2pipe_depth(9 downto 0))then
	   rd_depth <= '1';
	 else
	   rd_depth <= '0';
	 end if;
	 if(bus_rwq = '1' and ctrl_bus_q(15) = '0' and ctrl_bus_q(9 downto 0) = R_CONFU(9 downto 0))then
	   rd_version <= '1';
	 else
	   rd_version <= '0';
	 end if;
	 if(reset_i = '1')then
	   depth <= L2pipe_depth;
	 elsif(wr_depth = '1')then
	   depth <= ctrl_bus_q(6 downto 0);
	 end if;
    mem_depth <= (depth(5 downto 0) * "111") + 8;
  end if;
end process;
r_data <= "000000000" & depth when rd_depth = '1' else (others => 'Z');
r_data <= version when rd_version = '1' else (others => 'Z');
w_data <= ctrl_bus_q;
rd_status <= cmd_dl(3);
end my_arch;
