-- sprior16b.vhd
-- 16-bit synchronous one-hot "peel-a-way" priority logic
-- msb is bit 15 and it has the highest priority.

-- jamieson olsen <jamieson@fnal.gov>

-- *** revision history ***

-- 18 march 2002: created, functionally simulated.

library ieee;
use ieee.std_logic_1164.all;

entity sprior16b is
 port (
    din:        in  std_logic_vector(15 downto 0);
    clock:      in  std_logic;
    load:       in  std_logic;                       -- sync load active high
    reset:      in  std_logic;                       -- async reset active low
    dout:       out std_logic_vector(15 downto 0)
    );
end sprior16b;


architecture sprior16b_arch of sprior16b is

    signal inreg:   std_logic_vector(15 downto 0);
    signal outreg:  std_logic_vector(15 downto 0);
    signal pri_out: std_logic_vector(15 downto 0);

 begin

    -- combinatorial stuff...

            dout(15 downto 0) <= outreg(15 downto 0);

        -- 16-bit combinatorial one hot priority encoder

          pri_out(15) <= inreg(15);

          pri_out(14) <= inreg(14)    and not inreg(15);

          pri_out(13) <= inreg(13)    and not inreg(14)
                                      and not inreg(15);

          pri_out(12) <= inreg(12)    and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out(11) <= inreg(11)    and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out(10) <= inreg(10)    and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out( 9) <= inreg( 9)    and not inreg(10)
                                      and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out( 8) <= inreg( 8)    and not inreg( 9)
                                      and not inreg(10)
                                      and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out( 7) <= inreg( 7)    and not inreg( 8)
                                      and not inreg( 9)
                                      and not inreg(10)
                                      and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out( 6) <= inreg( 6)    and not inreg( 7)
                                      and not inreg( 8)
                                      and not inreg( 9)
                                      and not inreg(10)
                                      and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out( 5) <= inreg( 5)    and not inreg( 6)
                                      and not inreg( 7)
                                      and not inreg( 8)
                                      and not inreg( 9)
                                      and not inreg(10)
                                      and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out( 4) <= inreg( 4)    and not inreg( 5)
                                      and not inreg( 6)
                                      and not inreg( 7)
                                      and not inreg( 8)
                                      and not inreg( 9)
                                      and not inreg(10)
                                      and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out( 3) <= inreg( 3)    and not inreg( 4)
                                      and not inreg( 5)
                                      and not inreg( 6)
                                      and not inreg( 7)
                                      and not inreg( 8)
                                      and not inreg( 9)
                                      and not inreg(10)
                                      and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out( 2) <= inreg( 2)    and not inreg( 3)
                                      and not inreg( 4)
                                      and not inreg( 5)
                                      and not inreg( 6)
                                      and not inreg( 7)
                                      and not inreg( 8)
                                      and not inreg( 9)
                                      and not inreg(10)
                                      and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out( 1) <= inreg( 1)    and not inreg( 2)
                                      and not inreg( 3)
                                      and not inreg( 4)
                                      and not inreg( 5)
                                      and not inreg( 6)
                                      and not inreg( 7)
                                      and not inreg( 8)
                                      and not inreg( 9)
                                      and not inreg(10)
                                      and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

          pri_out( 0) <= inreg( 0)    and not inreg( 1)
                                      and not inreg( 2)
                                      and not inreg( 3)
                                      and not inreg( 4)
                                      and not inreg( 5)
                                      and not inreg( 6)
                                      and not inreg( 7)
                                      and not inreg( 8)
                                      and not inreg( 9)
                                      and not inreg(10)
                                      and not inreg(11)
                                      and not inreg(12)
                                      and not inreg(13)
                                      and not inreg(14)
                                      and not inreg(15);

    -- synchronous stuff...


    process(clock, reset)
     begin
        if (reset='0') then
            outreg(15 downto 0) <= X"0000";
        elsif rising_edge(clock) then
            outreg(15 downto 0) <= pri_out(15 downto 0);
        else
            null;
        end if;
    end process;


    -- sync process to maintain the input register.  in this design the input
    -- register does four things:

    -- 1. async clear with reset
    -- 2. sync load 
    -- 3. sync reset
    -- 4. hold value

    -- the one hot output of the priority logic is fed back to the input
    -- flops.  if an output of the priority logic is TRUE, then that correponding
    -- bit in the input register is cleared.  this enables the priority logic
    -- to find the NEXT HIGHEST priority bit.  visualize the highest priority
    -- bits being "peeled away"...



--	    process(clock, reset)	 change to sync reset Aug-08-04
	process(clock)
     begin

       if rising_edge(clock) then
         if (reset='0') then
           inreg(15 downto 0) <= X"0000";
			elsif (load='1') then  -- load
           inreg <= din;
         else   -- clear
           inreg <= inreg and not pri_out;
         end if;
       end if;
    end process;


end sprior16b_arch;
