-- 16 bit synchronous one hot priority logic

--	load is sync and active high
--	reset is async and active low

-- redesign (29 june 1999) use output of priority logic to clear the input
-- flops.  note: this destroys the contents of the input register.

-- revised 14 july 1999 for max, high, and med threshold bins

-- (21 july 1999) yet another revision of the 16 bit version of this file.
--  input register has a three way mux that can: load, clear, and hold.
--  the register is cleared by the output of the select logic.  this should
--  greatly improve speed.  note: input register is destroyed during normal
--  readout.

-- revised (15 march 2000) the priority is assigned backwards!  bit 0 (block 0) 
-- should have a higher priority that bit 15 (block 15).  make the change to the
-- combinatorial "pri_out <=" section below..

-- jamieson olsen <jamieson@fnal.gov>
-- 22 june 1999


library ieee;
use ieee.std_logic_1164.all;

entity sprior16 is
 port (
	din: 		in std_logic_vector (15 downto 0);
	clock:		in std_logic;
	load:		in std_logic;
	reset:		in std_logic;
	dout: 		out std_logic_vector (15 downto 0)
    );
end sprior16;


architecture sprior16_arch of sprior16 is

	signal inreg: 	std_logic_vector(15 downto 0);
	signal outreg: 	std_logic_vector(15 downto 0);
	signal pri_out:	std_logic_vector(15 downto 0);

 begin

	-- combinatorial equations

	dout(15 downto 0) <= outreg(15 downto 0);
	

	-- this is the 16 bit combinatorial one hot priority encoder
	-- should be one, maybe two levels of logic.

								
	pri_out(0) <= inreg( 0);
								
	pri_out(1) <= inreg( 1) 	and not inreg( 0);
								
	pri_out(2) <= inreg( 2) 	and not inreg( 0)
								and not inreg( 1);
								
	pri_out(3) <= inreg( 3) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2);
								
	pri_out( 4) <= inreg( 4) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3);
								
	pri_out( 5) <= inreg( 5) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4);
								
	pri_out( 6) <= inreg( 6) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4)
								and not inreg( 5);
								
	pri_out( 7) <= inreg( 7) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4)
								and not inreg( 5)
								and not inreg( 6);
								
	pri_out( 8) <= inreg( 8) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4)
								and not inreg( 5)
								and not inreg( 6)
								and not inreg( 7);
								
	pri_out( 9) <= inreg( 9) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4)
								and not inreg( 5)
								and not inreg( 6)
								and not inreg( 7)
								and not inreg( 8);
								
	pri_out(10) <= inreg(10) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4)
								and not inreg( 5)
								and not inreg( 6)
								and not inreg( 7)
								and not inreg( 8)
								and not inreg( 9);
								
	pri_out(11) <= inreg(11) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4)
								and not inreg( 5)
								and not inreg( 6)
								and not inreg( 7)
								and not inreg( 8)
								and not inreg( 9)
								and not inreg(10);
								
	pri_out(12) <= inreg(12) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4)
								and not inreg( 5)
								and not inreg( 6)
								and not inreg( 7)
								and not inreg( 8)
								and not inreg( 9)
								and not inreg(10)
								and not inreg(11);
								
	pri_out(13) <= inreg(13) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4)
								and not inreg( 5)
								and not inreg( 6)
								and not inreg( 7)
								and not inreg( 8)
								and not inreg( 9)
								and not inreg(10)
								and not inreg(11)
								and not inreg(12);
								
	pri_out(14) <= inreg(14) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4)
								and not inreg( 5)
								and not inreg( 6)
								and not inreg( 7)
								and not inreg( 8)
								and not inreg( 9)
								and not inreg(10)
								and not inreg(11)
								and not inreg(12)
								and not inreg(13);
								
	pri_out(15) <= inreg(15) 	and not inreg( 0)
								and not inreg( 1)
								and not inreg( 2)
								and not inreg( 3)
								and not inreg( 4)
								and not inreg( 5)
								and not inreg( 6)
								and not inreg( 7)
								and not inreg( 8)
								and not inreg( 9)
								and not inreg(10)
								and not inreg(11)
								and not inreg(12)
								and not inreg(13)
								and not inreg(14);



	-- sync process to maintain the output register


	process(clock, reset)
	 begin
		if (reset='0') then
--			outreg(15 downto 0) <= "0000000000000000";
		elsif rising_edge(clock) then
--			outreg(15 downto 0) <= pri_out(15 downto 0);
		else
			null;
		end if;
	end process;

outreg <= pri_out;


	-- sync process to maintain the input register

	process(clock, reset)
	 begin
		if (reset='0') then
			inreg(15 downto 0) <= "0000000000000000";
		elsif rising_edge(clock) then

			if (load='1') then	-- load
				inreg(15) <= din(15);
			elsif (load='0' and pri_out(15)='1') then	-- clear
				inreg(15) <= '0';
			else	-- hold
				inreg(15) <= inreg(15);
			end if;


			if (load='1') then	-- load
				inreg(14) <= din(14);
			elsif (load='0' and pri_out(14)='1') then	-- clear
				inreg(14) <= '0';
			else	-- hold
				inreg(14) <= inreg(14);
			end if;


			if (load='1') then	-- load
				inreg(13) <= din(13);
			elsif (load='0' and pri_out(13)='1') then	-- clear
				inreg(13) <= '0';
			else	-- hold
				inreg(13) <= inreg(13);
			end if;

			if (load='1') then	-- load
				inreg(12) <= din(12);
			elsif (load='0' and pri_out(12)='1') then	-- clear
				inreg(12) <= '0';
			else	-- hold
				inreg(12) <= inreg(12);
			end if;

			if (load='1') then	-- load
				inreg(11) <= din(11);
			elsif (load='0' and pri_out(11)='1') then	-- clear
				inreg(11) <= '0';
			else	-- hold
				inreg(11) <= inreg(11);
			end if;

			if (load='1') then	-- load
				inreg(10) <= din(10);
			elsif (load='0' and pri_out(10)='1') then	-- clear
				inreg(10) <= '0';
			else	-- hold
				inreg(10) <= inreg(10);
			end if;

			if (load='1') then	-- load
				inreg(9) <= din(9);
			elsif (load='0' and pri_out(9)='1') then	-- clear
				inreg(9) <= '0';
			else	-- hold
				inreg(9) <= inreg(9);
			end if;

			if (load='1') then	-- load
				inreg(8) <= din(8);
			elsif (load='0' and pri_out(8)='1') then	-- clear
				inreg(8) <= '0';
			else	-- hold
				inreg(8) <= inreg(8);
			end if;

			if (load='1') then	-- load
				inreg(7) <= din(7);
			elsif (load='0' and pri_out(7)='1') then	-- clear
				inreg(7) <= '0';
			else	-- hold
				inreg(7) <= inreg(7);
			end if;

			if (load='1') then	-- load
				inreg(6) <= din(6);
			elsif (load='0' and pri_out(6)='1') then	-- clear
				inreg(6) <= '0';
			else	-- hold
				inreg(6) <= inreg(6);
			end if;

			if (load='1') then	-- load
				inreg(5) <= din(5);
			elsif (load='0' and pri_out(5)='1') then	-- clear
				inreg(5) <= '0';
			else	-- hold
				inreg(5) <= inreg(5);
			end if;

			if (load='1') then	-- load
				inreg(4) <= din(4);
			elsif (load='0' and pri_out(4)='1') then	-- clear
				inreg(4) <= '0';
			else	-- hold
				inreg(4) <= inreg(4);
			end if;

			if (load='1') then	-- load
				inreg(3) <= din(3);
			elsif (load='0' and pri_out(3)='1') then	-- clear
				inreg(3) <= '0';
			else	-- hold
				inreg(3) <= inreg(3);
			end if;

			if (load='1') then	-- load
				inreg(2) <= din(2);
			elsif (load='0' and pri_out(2)='1') then	-- clear
				inreg(2) <= '0';
			else	-- hold
				inreg(2) <= inreg(2);
			end if;

			if (load='1') then	-- load
				inreg(1) <= din(1);
			elsif (load='0' and pri_out(1)='1') then	-- clear
				inreg(1) <= '0';
			else	-- hold
				inreg(1) <= inreg(1);
			end if;

			if (load='1') then	-- load
				inreg(0) <= din(0);
			elsif (load='0' and pri_out(0)='1') then	-- clear
				inreg(0) <= '0';
			else	-- hold
				inreg(0) <= inreg(0);
			end if;

		end if;
	end process;


end sprior16_arch;
