-- gate_mux704.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- *** revision history ***
--  28 Jun 1999:    created using Virtex TBUFs
--   9 Jan 2002:    modifed the order so that sel(0) will select d(21 downto 0) now.
--  27 Feb 2002:    Synplify cannot synthsize the tri-states correctly (!) It keeps wanting to
--                  build the MUX out of LUTs!  Rewrite until it gets it right.
--  28 Feb 2002:    Re-write using conventional gates.  This may get ugly!
--  31 Mar 2004:    XST having trouble with too many logic levels.  Revert back to using TBUFS.
--  01 Jul 2004:    changed to synchronous reset
--  Jul-01-04                tri-state control signal simplified S.X.Wu
library ieee;
use ieee.std_logic_1164.all;

entity mux352 is
    port (
        d:      in  std_logic_vector(351 downto 0);     -- matrix flops
        sel:    in  std_logic_vector(15 downto 0);      -- one hot select lines
        reset:  in  std_logic;                          -- active low
        clock:  in  std_logic;                          -- 53MHz
        q:      out std_logic_vector(21 downto 0)
    );
end mux352;

architecture mux352_arch of mux352 is

    -- define registers...

        signal mux_out_reg: std_logic_vector(21 downto 0);

    -- define signals...

        signal mux_out:     std_logic_vector(21 downto 0);

 begin

    -- combinatorial stuff...

        q(21 downto 0)       <= mux_out_reg(21 downto 0);
-- since sel is one-hot it could be simplified 1-jul-2004 S.X.Wu
i_gen: for i in sel'range generate
      mux_out <= d(i*22+21 downto i*22) when sel(i) = '1' else (others => 'Z');
end generate; 

    -- sequential stuff...
-- change to sychronous reset 1-jul-2004 S.X.Wu
process(clock)
begin
  if(clock'event and clock = '1')then
    if(reset = '0')then
	   mux_out_reg <= (others => '0');
	 else
	   mux_out_reg <= mux_out;
	 end if;
  end if;
end process;
end mux352_arch;
