--
-- Strawman design for DFEA2 illustrating exactly which
-- track cells should if possible be included in the generalized 
-- duplicate track removal vetoes.
--
-- For illustration only, this file assumes that all the
-- track results from all pT bins and the 3 sectors can be shared. 
-- Probably this needs to be split into 4 files, where only the
-- necessary inputs for each output type are used, like what was
-- done for DFEA V102.
-- For now, these files include as inputs all tracks from the two
-- neighboring sectors rather than just the immediate neighbors which
-- are actually used as vetoes.
--
-- I have used the following nomenclature (reason for this
-- is to keep fixed format length which makes things easier for
-- my coding, and trigsim coding). It also seemed necessary to keep
-- to the same indexing scheme as in the DFEA track designs.
-- I think this achieves something reasonable and unambiguous.
-- Feedback appreciated.
--
-- mcell (maxpt  output vector)
-- hcell (highpt output vector)
-- ncell (medpt  output vector)
-- lcell (lowpt  output vector)
--
-- The input vectors are referred to with two characters,
-- the leading character being the pT bin (m,h,n,l),
-- and the second character is
-- l (for N+1 sector), c for home sector (N), r (for N-1 sector).
-- So:  ml, mc, mr 
--      hl, hc, hr
--      nl, nc, nr
--      ll, lc, lr
--
-- Note. It is likely that not all of the logic conditions implicit
-- in the neighboring sector veto terms can actually be formed, but I'm
-- pretty sure some if it can be, and this should be useful.
-- So an implementation method which is robust against the absence of this
-- info would be great.
-- (ie. make sure inaccessible veto terms can not be set to true)
--
--                            Graham W. Wilson   Jan 6th 2006
-- 
library ieee;
use ieee.std_logic_1164.all;

entity mveto_general is
  port (
         mc:  in  std_logic_vector (351 downto 0);
         hc:  in  std_logic_vector (351 downto 0);
         nc:  in  std_logic_vector (351 downto 0);
         lc:  in  std_logic_vector (703 downto 0);
         ml:  in  std_logic_vector (351 downto 0);
         hl:  in  std_logic_vector (351 downto 0);
         nl:  in  std_logic_vector (351 downto 0);
         ll:  in  std_logic_vector (703 downto 0);
         mr:  in  std_logic_vector (351 downto 0);
         hr:  in  std_logic_vector (351 downto 0);
         nr:  in  std_logic_vector (351 downto 0);
         lr:  in  std_logic_vector (703 downto 0);
         version : out std_logic_vector(15 downto 0);     
         mcell: out std_logic_vector (351 downto 0);         
         hcell: out std_logic_vector (351 downto 0);         
         ncell: out std_logic_vector (351 downto 0);         
         lcell: out std_logic_vector (703 downto 0)
     );
end mveto_general;

architecture generalveto_arch of mveto_general is
 
  begin
    
  version <= x"0000";
  
  mcell(  0) <= mc(  0) and 
          not ( mc(  1) or mr( 43) );

  mcell(  1) <= mc(  1);

  mcell(  2) <= mc(  2) and 
          not ( mc(  1) or mc(  3) );

  mcell(  3) <= mc(  3);

  mcell(  4) <= mc(  4) and 
          not ( mc(  3) or mc(  5) );

  mcell(  5) <= mc(  5);

  mcell(  6) <= mc(  6) and 
          not ( mc(  5) or mc(  7) );

  mcell(  7) <= mc(  7);

  mcell(  8) <= mc(  8) and 
          not ( mc(  7) or mc(  9) );

  mcell(  9) <= mc(  9);

  mcell( 10) <= mc( 10) and 
          not ( mc(  9) or mc( 11) );

  mcell( 11) <= mc( 11);

  mcell( 12) <= mc( 12) and 
          not ( mc( 11) or mc( 13) );

  mcell( 13) <= mc( 13);

  mcell( 14) <= mc( 14) and 
          not ( mc( 13) or mc( 15) );

  mcell( 15) <= mc( 15);

  mcell( 16) <= mc( 16) and 
          not ( mc( 15) or mc( 17) );

  mcell( 17) <= mc( 17);

  mcell( 18) <= mc( 18) and 
          not ( mc( 17) or mc( 19) );

  mcell( 19) <= mc( 19);

  mcell( 20) <= mc( 20) and 
          not ( mc( 19) or mc( 21) );

  mcell( 21) <= mc( 21);

  mcell( 22) <= mc( 22) and 
          not ( mc( 21) or mc( 23) );

  mcell( 23) <= mc( 23);

  mcell( 24) <= mc( 24) and 
          not ( mc( 23) or mc( 25) );

  mcell( 25) <= mc( 25);

  mcell( 26) <= mc( 26) and 
          not ( mc( 25) or mc( 27) );

  mcell( 27) <= mc( 27);

  mcell( 28) <= mc( 28) and 
          not ( mc( 27) or mc( 29) );

  mcell( 29) <= mc( 29);

  mcell( 30) <= mc( 30) and 
          not ( mc( 29) or mc( 31) );

  mcell( 31) <= mc( 31);

  mcell( 32) <= mc( 32) and 
          not ( mc( 31) or mc( 33) );

  mcell( 33) <= mc( 33);

  mcell( 34) <= mc( 34) and 
          not ( mc( 33) or mc( 35) );

  mcell( 35) <= mc( 35);

  mcell( 36) <= mc( 36) and 
          not ( mc( 35) or mc( 37) );

  mcell( 37) <= mc( 37);

  mcell( 38) <= mc( 38) and 
          not ( mc( 37) or mc( 39) );

  mcell( 39) <= mc( 39);

  mcell( 40) <= mc( 40) and 
          not ( mc( 39) or mc( 41) );

  mcell( 41) <= mc( 41);

  mcell( 42) <= mc( 42) and 
          not ( mc( 41) or mc( 43) );

  mcell( 43) <= mc( 43);

  mcell( 44) <= mc( 44) and 
          not ( mc(  0) or mc(  1) or mr( 43) or mc( 45) or mr( 87) );

  mcell( 45) <= mc( 45) and 
          not ( mc(  0) or mc(  1) or mc(  2) );

  mcell( 46) <= mc( 46) and 
          not ( mc(  1) or mc(  2) or mc(  3) or mc( 45) or mc( 47) );

  mcell( 47) <= mc( 47) and 
          not ( mc(  2) or mc(  3) or mc(  4) );

  mcell( 48) <= mc( 48) and 
          not ( mc(  3) or mc(  4) or mc(  5) or mc( 47) or mc( 49) );

  mcell( 49) <= mc( 49) and 
          not ( mc(  4) or mc(  5) or mc(  6) );

  mcell( 50) <= mc( 50) and 
          not ( mc(  5) or mc(  6) or mc(  7) or mc( 49) or mc( 51) );

  mcell( 51) <= mc( 51) and 
          not ( mc(  6) or mc(  7) or mc(  8) );

  mcell( 52) <= mc( 52) and 
          not ( mc(  7) or mc(  8) or mc(  9) or mc( 51) or mc( 53) );

  mcell( 53) <= mc( 53) and 
          not ( mc(  8) or mc(  9) or mc( 10) );

  mcell( 54) <= mc( 54) and 
          not ( mc(  9) or mc( 10) or mc( 11) or mc( 53) or mc( 55) );

  mcell( 55) <= mc( 55) and 
          not ( mc( 10) or mc( 11) or mc( 12) );

  mcell( 56) <= mc( 56) and 
          not ( mc( 11) or mc( 12) or mc( 13) or mc( 55) or mc( 57) );

  mcell( 57) <= mc( 57) and 
          not ( mc( 12) or mc( 13) or mc( 14) );

  mcell( 58) <= mc( 58) and 
          not ( mc( 13) or mc( 14) or mc( 15) or mc( 57) or mc( 59) );

  mcell( 59) <= mc( 59) and 
          not ( mc( 14) or mc( 15) or mc( 16) );

  mcell( 60) <= mc( 60) and 
          not ( mc( 15) or mc( 16) or mc( 17) or mc( 59) or mc( 61) );

  mcell( 61) <= mc( 61) and 
          not ( mc( 16) or mc( 17) or mc( 18) );

  mcell( 62) <= mc( 62) and 
          not ( mc( 17) or mc( 18) or mc( 19) or mc( 61) or mc( 63) );

  mcell( 63) <= mc( 63) and 
          not ( mc( 18) or mc( 19) or mc( 20) );

  mcell( 64) <= mc( 64) and 
          not ( mc( 19) or mc( 20) or mc( 21) or mc( 63) or mc( 65) );

  mcell( 65) <= mc( 65) and 
          not ( mc( 20) or mc( 21) or mc( 22) );

  mcell( 66) <= mc( 66) and 
          not ( mc( 21) or mc( 22) or mc( 23) or mc( 65) or mc( 67) );

  mcell( 67) <= mc( 67) and 
          not ( mc( 22) or mc( 23) or mc( 24) );

  mcell( 68) <= mc( 68) and 
          not ( mc( 23) or mc( 24) or mc( 25) or mc( 67) or mc( 69) );

  mcell( 69) <= mc( 69) and 
          not ( mc( 24) or mc( 25) or mc( 26) );

  mcell( 70) <= mc( 70) and 
          not ( mc( 25) or mc( 26) or mc( 27) or mc( 69) or mc( 71) );

  mcell( 71) <= mc( 71) and 
          not ( mc( 26) or mc( 27) or mc( 28) );

  mcell( 72) <= mc( 72) and 
          not ( mc( 27) or mc( 28) or mc( 29) or mc( 71) or mc( 73) );

  mcell( 73) <= mc( 73) and 
          not ( mc( 28) or mc( 29) or mc( 30) );

  mcell( 74) <= mc( 74) and 
          not ( mc( 29) or mc( 30) or mc( 31) or mc( 73) or mc( 75) );

  mcell( 75) <= mc( 75) and 
          not ( mc( 30) or mc( 31) or mc( 32) );

  mcell( 76) <= mc( 76) and 
          not ( mc( 31) or mc( 32) or mc( 33) or mc( 75) or mc( 77) );

  mcell( 77) <= mc( 77) and 
          not ( mc( 32) or mc( 33) or mc( 34) );

  mcell( 78) <= mc( 78) and 
          not ( mc( 33) or mc( 34) or mc( 35) or mc( 77) or mc( 79) );

  mcell( 79) <= mc( 79) and 
          not ( mc( 34) or mc( 35) or mc( 36) );

  mcell( 80) <= mc( 80) and 
          not ( mc( 35) or mc( 36) or mc( 37) or mc( 79) or mc( 81) );

  mcell( 81) <= mc( 81) and 
          not ( mc( 36) or mc( 37) or mc( 38) );

  mcell( 82) <= mc( 82) and 
          not ( mc( 37) or mc( 38) or mc( 39) or mc( 81) or mc( 83) );

  mcell( 83) <= mc( 83) and 
          not ( mc( 38) or mc( 39) or mc( 40) );

  mcell( 84) <= mc( 84) and 
          not ( mc( 39) or mc( 40) or mc( 41) or mc( 83) or mc( 85) );

  mcell( 85) <= mc( 85) and 
          not ( mc( 40) or mc( 41) or mc( 42) );

  mcell( 86) <= mc( 86) and 
          not ( mc( 41) or mc( 42) or mc( 43) or mc( 85) or mc( 87) );

  mcell( 87) <= mc( 87) and 
          not ( ml(  0) or mc( 42) or mc( 43) );

  mcell( 88) <= mc( 88) and 
          not ( mc(  0) or mc(  1) or mr( 43) or mc( 89) or mr(131) );

  mcell( 89) <= mc( 89) and 
          not ( mc(  0) or mc(  1) or mc(  2) );

  mcell( 90) <= mc( 90) and 
          not ( mc(  1) or mc(  2) or mc(  3) or mc( 89) or mc( 91) );

  mcell( 91) <= mc( 91) and 
          not ( mc(  2) or mc(  3) or mc(  4) );

  mcell( 92) <= mc( 92) and 
          not ( mc(  3) or mc(  4) or mc(  5) or mc( 91) or mc( 93) );

  mcell( 93) <= mc( 93) and 
          not ( mc(  4) or mc(  5) or mc(  6) );

  mcell( 94) <= mc( 94) and 
          not ( mc(  5) or mc(  6) or mc(  7) or mc( 93) or mc( 95) );

  mcell( 95) <= mc( 95) and 
          not ( mc(  6) or mc(  7) or mc(  8) );

  mcell( 96) <= mc( 96) and 
          not ( mc(  7) or mc(  8) or mc(  9) or mc( 95) or mc( 97) );

  mcell( 97) <= mc( 97) and 
          not ( mc(  8) or mc(  9) or mc( 10) );

  mcell( 98) <= mc( 98) and 
          not ( mc(  9) or mc( 10) or mc( 11) or mc( 97) or mc( 99) );

  mcell( 99) <= mc( 99) and 
          not ( mc( 10) or mc( 11) or mc( 12) );

  mcell(100) <= mc(100) and 
          not ( mc( 11) or mc( 12) or mc( 13) or mc( 99) or mc(101) );

  mcell(101) <= mc(101) and 
          not ( mc( 12) or mc( 13) or mc( 14) );

  mcell(102) <= mc(102) and 
          not ( mc( 13) or mc( 14) or mc( 15) or mc(101) or mc(103) );

  mcell(103) <= mc(103) and 
          not ( mc( 14) or mc( 15) or mc( 16) );

  mcell(104) <= mc(104) and 
          not ( mc( 15) or mc( 16) or mc( 17) or mc(103) or mc(105) );

  mcell(105) <= mc(105) and 
          not ( mc( 16) or mc( 17) or mc( 18) );

  mcell(106) <= mc(106) and 
          not ( mc( 17) or mc( 18) or mc( 19) or mc(105) or mc(107) );

  mcell(107) <= mc(107) and 
          not ( mc( 18) or mc( 19) or mc( 20) );

  mcell(108) <= mc(108) and 
          not ( mc( 19) or mc( 20) or mc( 21) or mc(107) or mc(109) );

  mcell(109) <= mc(109) and 
          not ( mc( 20) or mc( 21) or mc( 22) );

  mcell(110) <= mc(110) and 
          not ( mc( 21) or mc( 22) or mc( 23) or mc(109) or mc(111) );

  mcell(111) <= mc(111) and 
          not ( mc( 22) or mc( 23) or mc( 24) );

  mcell(112) <= mc(112) and 
          not ( mc( 23) or mc( 24) or mc( 25) or mc(111) or mc(113) );

  mcell(113) <= mc(113) and 
          not ( mc( 24) or mc( 25) or mc( 26) );

  mcell(114) <= mc(114) and 
          not ( mc( 25) or mc( 26) or mc( 27) or mc(113) or mc(115) );

  mcell(115) <= mc(115) and 
          not ( mc( 26) or mc( 27) or mc( 28) );

  mcell(116) <= mc(116) and 
          not ( mc( 27) or mc( 28) or mc( 29) or mc(115) or mc(117) );

  mcell(117) <= mc(117) and 
          not ( mc( 28) or mc( 29) or mc( 30) );

  mcell(118) <= mc(118) and 
          not ( mc( 29) or mc( 30) or mc( 31) or mc(117) or mc(119) );

  mcell(119) <= mc(119) and 
          not ( mc( 30) or mc( 31) or mc( 32) );

  mcell(120) <= mc(120) and 
          not ( mc( 31) or mc( 32) or mc( 33) or mc(119) or mc(121) );

  mcell(121) <= mc(121) and 
          not ( mc( 32) or mc( 33) or mc( 34) );

  mcell(122) <= mc(122) and 
          not ( mc( 33) or mc( 34) or mc( 35) or mc(121) or mc(123) );

  mcell(123) <= mc(123) and 
          not ( mc( 34) or mc( 35) or mc( 36) );

  mcell(124) <= mc(124) and 
          not ( mc( 35) or mc( 36) or mc( 37) or mc(123) or mc(125) );

  mcell(125) <= mc(125) and 
          not ( mc( 36) or mc( 37) or mc( 38) );

  mcell(126) <= mc(126) and 
          not ( mc( 37) or mc( 38) or mc( 39) or mc(125) or mc(127) );

  mcell(127) <= mc(127) and 
          not ( mc( 38) or mc( 39) or mc( 40) );

  mcell(128) <= mc(128) and 
          not ( mc( 39) or mc( 40) or mc( 41) or mc(127) or mc(129) );

  mcell(129) <= mc(129) and 
          not ( mc( 40) or mc( 41) or mc( 42) );

  mcell(130) <= mc(130) and 
          not ( mc( 41) or mc( 42) or mc( 43) or mc(129) or mc(131) );

  mcell(131) <= mc(131) and 
          not ( ml(  0) or mc( 42) or mc( 43) );

  mcell(132) <= mc(132) and 
          not ( mc( 44) or mc( 45) or mr( 87) or mc(133) or mr(175) );

  mcell(133) <= mc(133) and 
          not ( mc( 44) or mc( 45) or mc( 46) );

  mcell(134) <= mc(134) and 
          not ( mc( 45) or mc( 46) or mc( 47) or mc(133) or mc(135) );

  mcell(135) <= mc(135) and 
          not ( mc( 46) or mc( 47) or mc( 48) );

  mcell(136) <= mc(136) and 
          not ( mc( 47) or mc( 48) or mc( 49) or mc(135) or mc(137) );

  mcell(137) <= mc(137) and 
          not ( mc( 48) or mc( 49) or mc( 50) );

  mcell(138) <= mc(138) and 
          not ( mc( 49) or mc( 50) or mc( 51) or mc(137) or mc(139) );

  mcell(139) <= mc(139) and 
          not ( mc( 50) or mc( 51) or mc( 52) );

  mcell(140) <= mc(140) and 
          not ( mc( 51) or mc( 52) or mc( 53) or mc(139) or mc(141) );

  mcell(141) <= mc(141) and 
          not ( mc( 52) or mc( 53) or mc( 54) );

  mcell(142) <= mc(142) and 
          not ( mc( 53) or mc( 54) or mc( 55) or mc(141) or mc(143) );

  mcell(143) <= mc(143) and 
          not ( mc( 54) or mc( 55) or mc( 56) );

  mcell(144) <= mc(144) and 
          not ( mc( 55) or mc( 56) or mc( 57) or mc(143) or mc(145) );

  mcell(145) <= mc(145) and 
          not ( mc( 56) or mc( 57) or mc( 58) );

  mcell(146) <= mc(146) and 
          not ( mc( 57) or mc( 58) or mc( 59) or mc(145) or mc(147) );

  mcell(147) <= mc(147) and 
          not ( mc( 58) or mc( 59) or mc( 60) );

  mcell(148) <= mc(148) and 
          not ( mc( 59) or mc( 60) or mc( 61) or mc(147) or mc(149) );

  mcell(149) <= mc(149) and 
          not ( mc( 60) or mc( 61) or mc( 62) );

  mcell(150) <= mc(150) and 
          not ( mc( 61) or mc( 62) or mc( 63) or mc(149) or mc(151) );

  mcell(151) <= mc(151) and 
          not ( mc( 62) or mc( 63) or mc( 64) );

  mcell(152) <= mc(152) and 
          not ( mc( 63) or mc( 64) or mc( 65) or mc(151) or mc(153) );

  mcell(153) <= mc(153) and 
          not ( mc( 64) or mc( 65) or mc( 66) );

  mcell(154) <= mc(154) and 
          not ( mc( 65) or mc( 66) or mc( 67) or mc(153) or mc(155) );

  mcell(155) <= mc(155) and 
          not ( mc( 66) or mc( 67) or mc( 68) );

  mcell(156) <= mc(156) and 
          not ( mc( 67) or mc( 68) or mc( 69) or mc(155) or mc(157) );

  mcell(157) <= mc(157) and 
          not ( mc( 68) or mc( 69) or mc( 70) );

  mcell(158) <= mc(158) and 
          not ( mc( 69) or mc( 70) or mc( 71) or mc(157) or mc(159) );

  mcell(159) <= mc(159) and 
          not ( mc( 70) or mc( 71) or mc( 72) );

  mcell(160) <= mc(160) and 
          not ( mc( 71) or mc( 72) or mc( 73) or mc(159) or mc(161) );

  mcell(161) <= mc(161) and 
          not ( mc( 72) or mc( 73) or mc( 74) );

  mcell(162) <= mc(162) and 
          not ( mc( 73) or mc( 74) or mc( 75) or mc(161) or mc(163) );

  mcell(163) <= mc(163) and 
          not ( mc( 74) or mc( 75) or mc( 76) );

  mcell(164) <= mc(164) and 
          not ( mc( 75) or mc( 76) or mc( 77) or mc(163) or mc(165) );

  mcell(165) <= mc(165) and 
          not ( mc( 76) or mc( 77) or mc( 78) );

  mcell(166) <= mc(166) and 
          not ( mc( 77) or mc( 78) or mc( 79) or mc(165) or mc(167) );

  mcell(167) <= mc(167) and 
          not ( mc( 78) or mc( 79) or mc( 80) );

  mcell(168) <= mc(168) and 
          not ( mc( 79) or mc( 80) or mc( 81) or mc(167) or mc(169) );

  mcell(169) <= mc(169) and 
          not ( mc( 80) or mc( 81) or mc( 82) );

  mcell(170) <= mc(170) and 
          not ( mc( 81) or mc( 82) or mc( 83) or mc(169) or mc(171) );

  mcell(171) <= mc(171) and 
          not ( mc( 82) or mc( 83) or mc( 84) );

  mcell(172) <= mc(172) and 
          not ( mc( 83) or mc( 84) or mc( 85) or mc(171) or mc(173) );

  mcell(173) <= mc(173) and 
          not ( mc( 84) or mc( 85) or mc( 86) );

  mcell(174) <= mc(174) and 
          not ( mc( 85) or mc( 86) or mc( 87) or mc(173) or mc(175) );

  mcell(175) <= mc(175) and 
          not ( ml( 44) or mc( 86) or mc( 87) );

  mcell(176) <= mc(176) and 
          not ( mc( 88) or mc( 89) or mr(131) or mc(177) or mr(219) );

  mcell(177) <= mc(177) and 
          not ( mc( 88) or mc( 89) or mc( 90) );

  mcell(178) <= mc(178) and 
          not ( mc( 89) or mc( 90) or mc( 91) or mc(177) or mc(179) );

  mcell(179) <= mc(179) and 
          not ( mc( 90) or mc( 91) or mc( 92) );

  mcell(180) <= mc(180) and 
          not ( mc( 91) or mc( 92) or mc( 93) or mc(179) or mc(181) );

  mcell(181) <= mc(181) and 
          not ( mc( 92) or mc( 93) or mc( 94) );

  mcell(182) <= mc(182) and 
          not ( mc( 93) or mc( 94) or mc( 95) or mc(181) or mc(183) );

  mcell(183) <= mc(183) and 
          not ( mc( 94) or mc( 95) or mc( 96) );

  mcell(184) <= mc(184) and 
          not ( mc( 95) or mc( 96) or mc( 97) or mc(183) or mc(185) );

  mcell(185) <= mc(185) and 
          not ( mc( 96) or mc( 97) or mc( 98) );

  mcell(186) <= mc(186) and 
          not ( mc( 97) or mc( 98) or mc( 99) or mc(185) or mc(187) );

  mcell(187) <= mc(187) and 
          not ( mc( 98) or mc( 99) or mc(100) );

  mcell(188) <= mc(188) and 
          not ( mc( 99) or mc(100) or mc(101) or mc(187) or mc(189) );

  mcell(189) <= mc(189) and 
          not ( mc(100) or mc(101) or mc(102) );

  mcell(190) <= mc(190) and 
          not ( mc(101) or mc(102) or mc(103) or mc(189) or mc(191) );

  mcell(191) <= mc(191) and 
          not ( mc(102) or mc(103) or mc(104) );

  mcell(192) <= mc(192) and 
          not ( mc(103) or mc(104) or mc(105) or mc(191) or mc(193) );

  mcell(193) <= mc(193) and 
          not ( mc(104) or mc(105) or mc(106) );

  mcell(194) <= mc(194) and 
          not ( mc(105) or mc(106) or mc(107) or mc(193) or mc(195) );

  mcell(195) <= mc(195) and 
          not ( mc(106) or mc(107) or mc(108) );

  mcell(196) <= mc(196) and 
          not ( mc(107) or mc(108) or mc(109) or mc(195) or mc(197) );

  mcell(197) <= mc(197) and 
          not ( mc(108) or mc(109) or mc(110) );

  mcell(198) <= mc(198) and 
          not ( mc(109) or mc(110) or mc(111) or mc(197) or mc(199) );

  mcell(199) <= mc(199) and 
          not ( mc(110) or mc(111) or mc(112) );

  mcell(200) <= mc(200) and 
          not ( mc(111) or mc(112) or mc(113) or mc(199) or mc(201) );

  mcell(201) <= mc(201) and 
          not ( mc(112) or mc(113) or mc(114) );

  mcell(202) <= mc(202) and 
          not ( mc(113) or mc(114) or mc(115) or mc(201) or mc(203) );

  mcell(203) <= mc(203) and 
          not ( mc(114) or mc(115) or mc(116) );

  mcell(204) <= mc(204) and 
          not ( mc(115) or mc(116) or mc(117) or mc(203) or mc(205) );

  mcell(205) <= mc(205) and 
          not ( mc(116) or mc(117) or mc(118) );

  mcell(206) <= mc(206) and 
          not ( mc(117) or mc(118) or mc(119) or mc(205) or mc(207) );

  mcell(207) <= mc(207) and 
          not ( mc(118) or mc(119) or mc(120) );

  mcell(208) <= mc(208) and 
          not ( mc(119) or mc(120) or mc(121) or mc(207) or mc(209) );

  mcell(209) <= mc(209) and 
          not ( mc(120) or mc(121) or mc(122) );

  mcell(210) <= mc(210) and 
          not ( mc(121) or mc(122) or mc(123) or mc(209) or mc(211) );

  mcell(211) <= mc(211) and 
          not ( mc(122) or mc(123) or mc(124) );

  mcell(212) <= mc(212) and 
          not ( mc(123) or mc(124) or mc(125) or mc(211) or mc(213) );

  mcell(213) <= mc(213) and 
          not ( mc(124) or mc(125) or mc(126) );

  mcell(214) <= mc(214) and 
          not ( mc(125) or mc(126) or mc(127) or mc(213) or mc(215) );

  mcell(215) <= mc(215) and 
          not ( mc(126) or mc(127) or mc(128) );

  mcell(216) <= mc(216) and 
          not ( mc(127) or mc(128) or mc(129) or mc(215) or mc(217) );

  mcell(217) <= mc(217) and 
          not ( mc(128) or mc(129) or mc(130) );

  mcell(218) <= mc(218) and 
          not ( mc(129) or mc(130) or mc(131) or mc(217) or mc(219) );

  mcell(219) <= mc(219) and 
          not ( ml( 88) or mc(130) or mc(131) );

  mcell(220) <= mc(220) and 
          not ( mc(132) or mc(133) or mr(175) or mc(221) or mr(263) );

  mcell(221) <= mc(221) and 
          not ( mc(132) or mc(133) or mc(134) );

  mcell(222) <= mc(222) and 
          not ( mc(133) or mc(134) or mc(135) or mc(221) or mc(223) );

  mcell(223) <= mc(223) and 
          not ( mc(134) or mc(135) or mc(136) );

  mcell(224) <= mc(224) and 
          not ( mc(135) or mc(136) or mc(137) or mc(223) or mc(225) );

  mcell(225) <= mc(225) and 
          not ( mc(136) or mc(137) or mc(138) );

  mcell(226) <= mc(226) and 
          not ( mc(137) or mc(138) or mc(139) or mc(225) or mc(227) );

  mcell(227) <= mc(227) and 
          not ( mc(138) or mc(139) or mc(140) );

  mcell(228) <= mc(228) and 
          not ( mc(139) or mc(140) or mc(141) or mc(227) or mc(229) );

  mcell(229) <= mc(229) and 
          not ( mc(140) or mc(141) or mc(142) );

  mcell(230) <= mc(230) and 
          not ( mc(141) or mc(142) or mc(143) or mc(229) or mc(231) );

  mcell(231) <= mc(231) and 
          not ( mc(142) or mc(143) or mc(144) );

  mcell(232) <= mc(232) and 
          not ( mc(143) or mc(144) or mc(145) or mc(231) or mc(233) );

  mcell(233) <= mc(233) and 
          not ( mc(144) or mc(145) or mc(146) );

  mcell(234) <= mc(234) and 
          not ( mc(145) or mc(146) or mc(147) or mc(233) or mc(235) );

  mcell(235) <= mc(235) and 
          not ( mc(146) or mc(147) or mc(148) );

  mcell(236) <= mc(236) and 
          not ( mc(147) or mc(148) or mc(149) or mc(235) or mc(237) );

  mcell(237) <= mc(237) and 
          not ( mc(148) or mc(149) or mc(150) );

  mcell(238) <= mc(238) and 
          not ( mc(149) or mc(150) or mc(151) or mc(237) or mc(239) );

  mcell(239) <= mc(239) and 
          not ( mc(150) or mc(151) or mc(152) );

  mcell(240) <= mc(240) and 
          not ( mc(151) or mc(152) or mc(153) or mc(239) or mc(241) );

  mcell(241) <= mc(241) and 
          not ( mc(152) or mc(153) or mc(154) );

  mcell(242) <= mc(242) and 
          not ( mc(153) or mc(154) or mc(155) or mc(241) or mc(243) );

  mcell(243) <= mc(243) and 
          not ( mc(154) or mc(155) or mc(156) );

  mcell(244) <= mc(244) and 
          not ( mc(155) or mc(156) or mc(157) or mc(243) or mc(245) );

  mcell(245) <= mc(245) and 
          not ( mc(156) or mc(157) or mc(158) );

  mcell(246) <= mc(246) and 
          not ( mc(157) or mc(158) or mc(159) or mc(245) or mc(247) );

  mcell(247) <= mc(247) and 
          not ( mc(158) or mc(159) or mc(160) );

  mcell(248) <= mc(248) and 
          not ( mc(159) or mc(160) or mc(161) or mc(247) or mc(249) );

  mcell(249) <= mc(249) and 
          not ( mc(160) or mc(161) or mc(162) );

  mcell(250) <= mc(250) and 
          not ( mc(161) or mc(162) or mc(163) or mc(249) or mc(251) );

  mcell(251) <= mc(251) and 
          not ( mc(162) or mc(163) or mc(164) );

  mcell(252) <= mc(252) and 
          not ( mc(163) or mc(164) or mc(165) or mc(251) or mc(253) );

  mcell(253) <= mc(253) and 
          not ( mc(164) or mc(165) or mc(166) );

  mcell(254) <= mc(254) and 
          not ( mc(165) or mc(166) or mc(167) or mc(253) or mc(255) );

  mcell(255) <= mc(255) and 
          not ( mc(166) or mc(167) or mc(168) );

  mcell(256) <= mc(256) and 
          not ( mc(167) or mc(168) or mc(169) or mc(255) or mc(257) );

  mcell(257) <= mc(257) and 
          not ( mc(168) or mc(169) or mc(170) );

  mcell(258) <= mc(258) and 
          not ( mc(169) or mc(170) or mc(171) or mc(257) or mc(259) );

  mcell(259) <= mc(259) and 
          not ( mc(170) or mc(171) or mc(172) );

  mcell(260) <= mc(260) and 
          not ( mc(171) or mc(172) or mc(173) or mc(259) or mc(261) );

  mcell(261) <= mc(261) and 
          not ( mc(172) or mc(173) or mc(174) );

  mcell(262) <= mc(262) and 
          not ( mc(173) or mc(174) or mc(175) or mc(261) or mc(263) );

  mcell(263) <= mc(263) and 
          not ( ml(132) or mc(174) or mc(175) );

  mcell(264) <= mc(264) and 
          not ( mc(176) or mc(177) or mr(219) or mc(265) or mr(307) );

  mcell(265) <= mc(265) and 
          not ( mc(176) or mc(177) or mc(178) );

  mcell(266) <= mc(266) and 
          not ( mc(177) or mc(178) or mc(179) or mc(265) or mc(267) );

  mcell(267) <= mc(267) and 
          not ( mc(178) or mc(179) or mc(180) );

  mcell(268) <= mc(268) and 
          not ( mc(179) or mc(180) or mc(181) or mc(267) or mc(269) );

  mcell(269) <= mc(269) and 
          not ( mc(180) or mc(181) or mc(182) );

  mcell(270) <= mc(270) and 
          not ( mc(181) or mc(182) or mc(183) or mc(269) or mc(271) );

  mcell(271) <= mc(271) and 
          not ( mc(182) or mc(183) or mc(184) );

  mcell(272) <= mc(272) and 
          not ( mc(183) or mc(184) or mc(185) or mc(271) or mc(273) );

  mcell(273) <= mc(273) and 
          not ( mc(184) or mc(185) or mc(186) );

  mcell(274) <= mc(274) and 
          not ( mc(185) or mc(186) or mc(187) or mc(273) or mc(275) );

  mcell(275) <= mc(275) and 
          not ( mc(186) or mc(187) or mc(188) );

  mcell(276) <= mc(276) and 
          not ( mc(187) or mc(188) or mc(189) or mc(275) or mc(277) );

  mcell(277) <= mc(277) and 
          not ( mc(188) or mc(189) or mc(190) );

  mcell(278) <= mc(278) and 
          not ( mc(189) or mc(190) or mc(191) or mc(277) or mc(279) );

  mcell(279) <= mc(279) and 
          not ( mc(190) or mc(191) or mc(192) );

  mcell(280) <= mc(280) and 
          not ( mc(191) or mc(192) or mc(193) or mc(279) or mc(281) );

  mcell(281) <= mc(281) and 
          not ( mc(192) or mc(193) or mc(194) );

  mcell(282) <= mc(282) and 
          not ( mc(193) or mc(194) or mc(195) or mc(281) or mc(283) );

  mcell(283) <= mc(283) and 
          not ( mc(194) or mc(195) or mc(196) );

  mcell(284) <= mc(284) and 
          not ( mc(195) or mc(196) or mc(197) or mc(283) or mc(285) );

  mcell(285) <= mc(285) and 
          not ( mc(196) or mc(197) or mc(198) );

  mcell(286) <= mc(286) and 
          not ( mc(197) or mc(198) or mc(199) or mc(285) or mc(287) );

  mcell(287) <= mc(287) and 
          not ( mc(198) or mc(199) or mc(200) );

  mcell(288) <= mc(288) and 
          not ( mc(199) or mc(200) or mc(201) or mc(287) or mc(289) );

  mcell(289) <= mc(289) and 
          not ( mc(200) or mc(201) or mc(202) );

  mcell(290) <= mc(290) and 
          not ( mc(201) or mc(202) or mc(203) or mc(289) or mc(291) );

  mcell(291) <= mc(291) and 
          not ( mc(202) or mc(203) or mc(204) );

  mcell(292) <= mc(292) and 
          not ( mc(203) or mc(204) or mc(205) or mc(291) or mc(293) );

  mcell(293) <= mc(293) and 
          not ( mc(204) or mc(205) or mc(206) );

  mcell(294) <= mc(294) and 
          not ( mc(205) or mc(206) or mc(207) or mc(293) or mc(295) );

  mcell(295) <= mc(295) and 
          not ( mc(206) or mc(207) or mc(208) );

  mcell(296) <= mc(296) and 
          not ( mc(207) or mc(208) or mc(209) or mc(295) or mc(297) );

  mcell(297) <= mc(297) and 
          not ( mc(208) or mc(209) or mc(210) );

  mcell(298) <= mc(298) and 
          not ( mc(209) or mc(210) or mc(211) or mc(297) or mc(299) );

  mcell(299) <= mc(299) and 
          not ( mc(210) or mc(211) or mc(212) );

  mcell(300) <= mc(300) and 
          not ( mc(211) or mc(212) or mc(213) or mc(299) or mc(301) );

  mcell(301) <= mc(301) and 
          not ( mc(212) or mc(213) or mc(214) );

  mcell(302) <= mc(302) and 
          not ( mc(213) or mc(214) or mc(215) or mc(301) or mc(303) );

  mcell(303) <= mc(303) and 
          not ( mc(214) or mc(215) or mc(216) );

  mcell(304) <= mc(304) and 
          not ( mc(215) or mc(216) or mc(217) or mc(303) or mc(305) );

  mcell(305) <= mc(305) and 
          not ( mc(216) or mc(217) or mc(218) );

  mcell(306) <= mc(306) and 
          not ( mc(217) or mc(218) or mc(219) or mc(305) or mc(307) );

  mcell(307) <= mc(307) and 
          not ( ml(176) or mc(218) or mc(219) );

  mcell(308) <= mc(308) and 
          not ( mc(220) or mc(221) or mr(263) or mc(309) or mr(351) );

  mcell(309) <= mc(309) and 
          not ( mc(220) or mc(221) or mc(222) );

  mcell(310) <= mc(310) and 
          not ( mc(221) or mc(222) or mc(223) or mc(309) or mc(311) );

  mcell(311) <= mc(311) and 
          not ( mc(222) or mc(223) or mc(224) );

  mcell(312) <= mc(312) and 
          not ( mc(223) or mc(224) or mc(225) or mc(311) or mc(313) );

  mcell(313) <= mc(313) and 
          not ( mc(224) or mc(225) or mc(226) );

  mcell(314) <= mc(314) and 
          not ( mc(225) or mc(226) or mc(227) or mc(313) or mc(315) );

  mcell(315) <= mc(315) and 
          not ( mc(226) or mc(227) or mc(228) );

  mcell(316) <= mc(316) and 
          not ( mc(227) or mc(228) or mc(229) or mc(315) or mc(317) );

  mcell(317) <= mc(317) and 
          not ( mc(228) or mc(229) or mc(230) );

  mcell(318) <= mc(318) and 
          not ( mc(229) or mc(230) or mc(231) or mc(317) or mc(319) );

  mcell(319) <= mc(319) and 
          not ( mc(230) or mc(231) or mc(232) );

  mcell(320) <= mc(320) and 
          not ( mc(231) or mc(232) or mc(233) or mc(319) or mc(321) );

  mcell(321) <= mc(321) and 
          not ( mc(232) or mc(233) or mc(234) );

  mcell(322) <= mc(322) and 
          not ( mc(233) or mc(234) or mc(235) or mc(321) or mc(323) );

  mcell(323) <= mc(323) and 
          not ( mc(234) or mc(235) or mc(236) );

  mcell(324) <= mc(324) and 
          not ( mc(235) or mc(236) or mc(237) or mc(323) or mc(325) );

  mcell(325) <= mc(325) and 
          not ( mc(236) or mc(237) or mc(238) );

  mcell(326) <= mc(326) and 
          not ( mc(237) or mc(238) or mc(239) or mc(325) or mc(327) );

  mcell(327) <= mc(327) and 
          not ( mc(238) or mc(239) or mc(240) );

  mcell(328) <= mc(328) and 
          not ( mc(239) or mc(240) or mc(241) or mc(327) or mc(329) );

  mcell(329) <= mc(329) and 
          not ( mc(240) or mc(241) or mc(242) );

  mcell(330) <= mc(330) and 
          not ( mc(241) or mc(242) or mc(243) or mc(329) or mc(331) );

  mcell(331) <= mc(331) and 
          not ( mc(242) or mc(243) or mc(244) );

  mcell(332) <= mc(332) and 
          not ( mc(243) or mc(244) or mc(245) or mc(331) or mc(333) );

  mcell(333) <= mc(333) and 
          not ( mc(244) or mc(245) or mc(246) );

  mcell(334) <= mc(334) and 
          not ( mc(245) or mc(246) or mc(247) or mc(333) or mc(335) );

  mcell(335) <= mc(335) and 
          not ( mc(246) or mc(247) or mc(248) );

  mcell(336) <= mc(336) and 
          not ( mc(247) or mc(248) or mc(249) or mc(335) or mc(337) );

  mcell(337) <= mc(337) and 
          not ( mc(248) or mc(249) or mc(250) );

  mcell(338) <= mc(338) and 
          not ( mc(249) or mc(250) or mc(251) or mc(337) or mc(339) );

  mcell(339) <= mc(339) and 
          not ( mc(250) or mc(251) or mc(252) );

  mcell(340) <= mc(340) and 
          not ( mc(251) or mc(252) or mc(253) or mc(339) or mc(341) );

  mcell(341) <= mc(341) and 
          not ( mc(252) or mc(253) or mc(254) );

  mcell(342) <= mc(342) and 
          not ( mc(253) or mc(254) or mc(255) or mc(341) or mc(343) );

  mcell(343) <= mc(343) and 
          not ( mc(254) or mc(255) or mc(256) );

  mcell(344) <= mc(344) and 
          not ( mc(255) or mc(256) or mc(257) or mc(343) or mc(345) );

  mcell(345) <= mc(345) and 
          not ( mc(256) or mc(257) or mc(258) );

  mcell(346) <= mc(346) and 
          not ( mc(257) or mc(258) or mc(259) or mc(345) or mc(347) );

  mcell(347) <= mc(347) and 
          not ( mc(258) or mc(259) or mc(260) );

  mcell(348) <= mc(348) and 
          not ( mc(259) or mc(260) or mc(261) or mc(347) or mc(349) );

  mcell(349) <= mc(349) and 
          not ( mc(260) or mc(261) or mc(262) );

  mcell(350) <= mc(350) and 
          not ( mc(261) or mc(262) or mc(263) or mc(349) or mc(351) );

  mcell(351) <= mc(351) and 
          not ( ml(220) or mc(262) or mc(263) );

  hcell(  0) <= hc(  0) and 
          not ( mc(264) or mc(265) or mr(307) or hc(  1) or hr( 43) );

  hcell(  1) <= hc(  1) and 
          not ( mc(264) or mc(265) or mc(266) );

  hcell(  2) <= hc(  2) and 
          not ( mc(265) or mc(266) or mc(267) or hc(  1) or hc(  3) );

  hcell(  3) <= hc(  3) and 
          not ( mc(266) or mc(267) or mc(268) );

  hcell(  4) <= hc(  4) and 
          not ( mc(267) or mc(268) or mc(269) or hc(  3) or hc(  5) );

  hcell(  5) <= hc(  5) and 
          not ( mc(268) or mc(269) or mc(270) );

  hcell(  6) <= hc(  6) and 
          not ( mc(269) or mc(270) or mc(271) or hc(  5) or hc(  7) );

  hcell(  7) <= hc(  7) and 
          not ( mc(270) or mc(271) or mc(272) );

  hcell(  8) <= hc(  8) and 
          not ( mc(271) or mc(272) or mc(273) or hc(  7) or hc(  9) );

  hcell(  9) <= hc(  9) and 
          not ( mc(272) or mc(273) or mc(274) );

  hcell( 10) <= hc( 10) and 
          not ( mc(273) or mc(274) or mc(275) or hc(  9) or hc( 11) );

  hcell( 11) <= hc( 11) and 
          not ( mc(274) or mc(275) or mc(276) );

  hcell( 12) <= hc( 12) and 
          not ( mc(275) or mc(276) or mc(277) or hc( 11) or hc( 13) );

  hcell( 13) <= hc( 13) and 
          not ( mc(276) or mc(277) or mc(278) );

  hcell( 14) <= hc( 14) and 
          not ( mc(277) or mc(278) or mc(279) or hc( 13) or hc( 15) );

  hcell( 15) <= hc( 15) and 
          not ( mc(278) or mc(279) or mc(280) );

  hcell( 16) <= hc( 16) and 
          not ( mc(279) or mc(280) or mc(281) or hc( 15) or hc( 17) );

  hcell( 17) <= hc( 17) and 
          not ( mc(280) or mc(281) or mc(282) );

  hcell( 18) <= hc( 18) and 
          not ( mc(281) or mc(282) or mc(283) or hc( 17) or hc( 19) );

  hcell( 19) <= hc( 19) and 
          not ( mc(282) or mc(283) or mc(284) );

  hcell( 20) <= hc( 20) and 
          not ( mc(283) or mc(284) or mc(285) or hc( 19) or hc( 21) );

  hcell( 21) <= hc( 21) and 
          not ( mc(284) or mc(285) or mc(286) );

  hcell( 22) <= hc( 22) and 
          not ( mc(285) or mc(286) or mc(287) or hc( 21) or hc( 23) );

  hcell( 23) <= hc( 23) and 
          not ( mc(286) or mc(287) or mc(288) );

  hcell( 24) <= hc( 24) and 
          not ( mc(287) or mc(288) or mc(289) or hc( 23) or hc( 25) );

  hcell( 25) <= hc( 25) and 
          not ( mc(288) or mc(289) or mc(290) );

  hcell( 26) <= hc( 26) and 
          not ( mc(289) or mc(290) or mc(291) or hc( 25) or hc( 27) );

  hcell( 27) <= hc( 27) and 
          not ( mc(290) or mc(291) or mc(292) );

  hcell( 28) <= hc( 28) and 
          not ( mc(291) or mc(292) or mc(293) or hc( 27) or hc( 29) );

  hcell( 29) <= hc( 29) and 
          not ( mc(292) or mc(293) or mc(294) );

  hcell( 30) <= hc( 30) and 
          not ( mc(293) or mc(294) or mc(295) or hc( 29) or hc( 31) );

  hcell( 31) <= hc( 31) and 
          not ( mc(294) or mc(295) or mc(296) );

  hcell( 32) <= hc( 32) and 
          not ( mc(295) or mc(296) or mc(297) or hc( 31) or hc( 33) );

  hcell( 33) <= hc( 33) and 
          not ( mc(296) or mc(297) or mc(298) );

  hcell( 34) <= hc( 34) and 
          not ( mc(297) or mc(298) or mc(299) or hc( 33) or hc( 35) );

  hcell( 35) <= hc( 35) and 
          not ( mc(298) or mc(299) or mc(300) );

  hcell( 36) <= hc( 36) and 
          not ( mc(299) or mc(300) or mc(301) or hc( 35) or hc( 37) );

  hcell( 37) <= hc( 37) and 
          not ( mc(300) or mc(301) or mc(302) );

  hcell( 38) <= hc( 38) and 
          not ( mc(301) or mc(302) or mc(303) or hc( 37) or hc( 39) );

  hcell( 39) <= hc( 39) and 
          not ( mc(302) or mc(303) or mc(304) );

  hcell( 40) <= hc( 40) and 
          not ( mc(303) or mc(304) or mc(305) or hc( 39) or hc( 41) );

  hcell( 41) <= hc( 41) and 
          not ( mc(304) or mc(305) or mc(306) );

  hcell( 42) <= hc( 42) and 
          not ( mc(305) or mc(306) or mc(307) or hc( 41) or hc( 43) );

  hcell( 43) <= hc( 43) and 
          not ( ml(264) or mc(306) or mc(307) );

  hcell( 44) <= hc( 44) and 
          not ( mc(308) or mc(309) or mr(351) or hc( 45) or hr( 87) );

  hcell( 45) <= hc( 45) and 
          not ( mc(308) or mc(309) or mc(310) );

  hcell( 46) <= hc( 46) and 
          not ( mc(309) or mc(310) or mc(311) or hc( 45) or hc( 47) );

  hcell( 47) <= hc( 47) and 
          not ( mc(310) or mc(311) or mc(312) );

  hcell( 48) <= hc( 48) and 
          not ( mc(311) or mc(312) or mc(313) or hc( 47) or hc( 49) );

  hcell( 49) <= hc( 49) and 
          not ( mc(312) or mc(313) or mc(314) );

  hcell( 50) <= hc( 50) and 
          not ( mc(313) or mc(314) or mc(315) or hc( 49) or hc( 51) );

  hcell( 51) <= hc( 51) and 
          not ( mc(314) or mc(315) or mc(316) );

  hcell( 52) <= hc( 52) and 
          not ( mc(315) or mc(316) or mc(317) or hc( 51) or hc( 53) );

  hcell( 53) <= hc( 53) and 
          not ( mc(316) or mc(317) or mc(318) );

  hcell( 54) <= hc( 54) and 
          not ( mc(317) or mc(318) or mc(319) or hc( 53) or hc( 55) );

  hcell( 55) <= hc( 55) and 
          not ( mc(318) or mc(319) or mc(320) );

  hcell( 56) <= hc( 56) and 
          not ( mc(319) or mc(320) or mc(321) or hc( 55) or hc( 57) );

  hcell( 57) <= hc( 57) and 
          not ( mc(320) or mc(321) or mc(322) );

  hcell( 58) <= hc( 58) and 
          not ( mc(321) or mc(322) or mc(323) or hc( 57) or hc( 59) );

  hcell( 59) <= hc( 59) and 
          not ( mc(322) or mc(323) or mc(324) );

  hcell( 60) <= hc( 60) and 
          not ( mc(323) or mc(324) or mc(325) or hc( 59) or hc( 61) );

  hcell( 61) <= hc( 61) and 
          not ( mc(324) or mc(325) or mc(326) );

  hcell( 62) <= hc( 62) and 
          not ( mc(325) or mc(326) or mc(327) or hc( 61) or hc( 63) );

  hcell( 63) <= hc( 63) and 
          not ( mc(326) or mc(327) or mc(328) );

  hcell( 64) <= hc( 64) and 
          not ( mc(327) or mc(328) or mc(329) or hc( 63) or hc( 65) );

  hcell( 65) <= hc( 65) and 
          not ( mc(328) or mc(329) or mc(330) );

  hcell( 66) <= hc( 66) and 
          not ( mc(329) or mc(330) or mc(331) or hc( 65) or hc( 67) );

  hcell( 67) <= hc( 67) and 
          not ( mc(330) or mc(331) or mc(332) );

  hcell( 68) <= hc( 68) and 
          not ( mc(331) or mc(332) or mc(333) or hc( 67) or hc( 69) );

  hcell( 69) <= hc( 69) and 
          not ( mc(332) or mc(333) or mc(334) );

  hcell( 70) <= hc( 70) and 
          not ( mc(333) or mc(334) or mc(335) or hc( 69) or hc( 71) );

  hcell( 71) <= hc( 71) and 
          not ( mc(334) or mc(335) or mc(336) );

  hcell( 72) <= hc( 72) and 
          not ( mc(335) or mc(336) or mc(337) or hc( 71) or hc( 73) );

  hcell( 73) <= hc( 73) and 
          not ( mc(336) or mc(337) or mc(338) );

  hcell( 74) <= hc( 74) and 
          not ( mc(337) or mc(338) or mc(339) or hc( 73) or hc( 75) );

  hcell( 75) <= hc( 75) and 
          not ( mc(338) or mc(339) or mc(340) );

  hcell( 76) <= hc( 76) and 
          not ( mc(339) or mc(340) or mc(341) or hc( 75) or hc( 77) );

  hcell( 77) <= hc( 77) and 
          not ( mc(340) or mc(341) or mc(342) );

  hcell( 78) <= hc( 78) and 
          not ( mc(341) or mc(342) or mc(343) or hc( 77) or hc( 79) );

  hcell( 79) <= hc( 79) and 
          not ( mc(342) or mc(343) or mc(344) );

  hcell( 80) <= hc( 80) and 
          not ( mc(343) or mc(344) or mc(345) or hc( 79) or hc( 81) );

  hcell( 81) <= hc( 81) and 
          not ( mc(344) or mc(345) or mc(346) );

  hcell( 82) <= hc( 82) and 
          not ( mc(345) or mc(346) or mc(347) or hc( 81) or hc( 83) );

  hcell( 83) <= hc( 83) and 
          not ( mc(346) or mc(347) or mc(348) );

  hcell( 84) <= hc( 84) and 
          not ( mc(347) or mc(348) or mc(349) or hc( 83) or hc( 85) );

  hcell( 85) <= hc( 85) and 
          not ( mc(348) or mc(349) or mc(350) );

  hcell( 86) <= hc( 86) and 
          not ( mc(349) or mc(350) or mc(351) or hc( 85) or hc( 87) );

  hcell( 87) <= hc( 87) and 
          not ( ml(308) or mc(350) or mc(351) );

  hcell( 88) <= hc( 88) and 
          not ( hc(  0) or hc(  1) or hr( 43) or hc( 89) or hr(131) );

  hcell( 89) <= hc( 89) and 
          not ( hc(  0) or hc(  1) or hc(  2) );

  hcell( 90) <= hc( 90) and 
          not ( hc(  1) or hc(  2) or hc(  3) or hc( 89) or hc( 91) );

  hcell( 91) <= hc( 91) and 
          not ( hc(  2) or hc(  3) or hc(  4) );

  hcell( 92) <= hc( 92) and 
          not ( hc(  3) or hc(  4) or hc(  5) or hc( 91) or hc( 93) );

  hcell( 93) <= hc( 93) and 
          not ( hc(  4) or hc(  5) or hc(  6) );

  hcell( 94) <= hc( 94) and 
          not ( hc(  5) or hc(  6) or hc(  7) or hc( 93) or hc( 95) );

  hcell( 95) <= hc( 95) and 
          not ( hc(  6) or hc(  7) or hc(  8) );

  hcell( 96) <= hc( 96) and 
          not ( hc(  7) or hc(  8) or hc(  9) or hc( 95) or hc( 97) );

  hcell( 97) <= hc( 97) and 
          not ( hc(  8) or hc(  9) or hc( 10) );

  hcell( 98) <= hc( 98) and 
          not ( hc(  9) or hc( 10) or hc( 11) or hc( 97) or hc( 99) );

  hcell( 99) <= hc( 99) and 
          not ( hc( 10) or hc( 11) or hc( 12) );

  hcell(100) <= hc(100) and 
          not ( hc( 11) or hc( 12) or hc( 13) or hc( 99) or hc(101) );

  hcell(101) <= hc(101) and 
          not ( hc( 12) or hc( 13) or hc( 14) );

  hcell(102) <= hc(102) and 
          not ( hc( 13) or hc( 14) or hc( 15) or hc(101) or hc(103) );

  hcell(103) <= hc(103) and 
          not ( hc( 14) or hc( 15) or hc( 16) );

  hcell(104) <= hc(104) and 
          not ( hc( 15) or hc( 16) or hc( 17) or hc(103) or hc(105) );

  hcell(105) <= hc(105) and 
          not ( hc( 16) or hc( 17) or hc( 18) );

  hcell(106) <= hc(106) and 
          not ( hc( 17) or hc( 18) or hc( 19) or hc(105) or hc(107) );

  hcell(107) <= hc(107) and 
          not ( hc( 18) or hc( 19) or hc( 20) );

  hcell(108) <= hc(108) and 
          not ( hc( 19) or hc( 20) or hc( 21) or hc(107) or hc(109) );

  hcell(109) <= hc(109) and 
          not ( hc( 20) or hc( 21) or hc( 22) );

  hcell(110) <= hc(110) and 
          not ( hc( 21) or hc( 22) or hc( 23) or hc(109) or hc(111) );

  hcell(111) <= hc(111) and 
          not ( hc( 22) or hc( 23) or hc( 24) );

  hcell(112) <= hc(112) and 
          not ( hc( 23) or hc( 24) or hc( 25) or hc(111) or hc(113) );

  hcell(113) <= hc(113) and 
          not ( hc( 24) or hc( 25) or hc( 26) );

  hcell(114) <= hc(114) and 
          not ( hc( 25) or hc( 26) or hc( 27) or hc(113) or hc(115) );

  hcell(115) <= hc(115) and 
          not ( hc( 26) or hc( 27) or hc( 28) );

  hcell(116) <= hc(116) and 
          not ( hc( 27) or hc( 28) or hc( 29) or hc(115) or hc(117) );

  hcell(117) <= hc(117) and 
          not ( hc( 28) or hc( 29) or hc( 30) );

  hcell(118) <= hc(118) and 
          not ( hc( 29) or hc( 30) or hc( 31) or hc(117) or hc(119) );

  hcell(119) <= hc(119) and 
          not ( hc( 30) or hc( 31) or hc( 32) );

  hcell(120) <= hc(120) and 
          not ( hc( 31) or hc( 32) or hc( 33) or hc(119) or hc(121) );

  hcell(121) <= hc(121) and 
          not ( hc( 32) or hc( 33) or hc( 34) );

  hcell(122) <= hc(122) and 
          not ( hc( 33) or hc( 34) or hc( 35) or hc(121) or hc(123) );

  hcell(123) <= hc(123) and 
          not ( hc( 34) or hc( 35) or hc( 36) );

  hcell(124) <= hc(124) and 
          not ( hc( 35) or hc( 36) or hc( 37) or hc(123) or hc(125) );

  hcell(125) <= hc(125) and 
          not ( hc( 36) or hc( 37) or hc( 38) );

  hcell(126) <= hc(126) and 
          not ( hc( 37) or hc( 38) or hc( 39) or hc(125) or hc(127) );

  hcell(127) <= hc(127) and 
          not ( hc( 38) or hc( 39) or hc( 40) );

  hcell(128) <= hc(128) and 
          not ( hc( 39) or hc( 40) or hc( 41) or hc(127) or hc(129) );

  hcell(129) <= hc(129) and 
          not ( hc( 40) or hc( 41) or hc( 42) );

  hcell(130) <= hc(130) and 
          not ( hc( 41) or hc( 42) or hc( 43) or hc(129) or hc(131) );

  hcell(131) <= hc(131) and 
          not ( hl(  0) or hc( 42) or hc( 43) );

  hcell(132) <= hc(132) and 
          not ( hc( 44) or hc( 45) or hr( 87) or hc(133) or hr(175) );

  hcell(133) <= hc(133) and 
          not ( hc( 44) or hc( 45) or hc( 46) );

  hcell(134) <= hc(134) and 
          not ( hc( 45) or hc( 46) or hc( 47) or hc(133) or hc(135) );

  hcell(135) <= hc(135) and 
          not ( hc( 46) or hc( 47) or hc( 48) );

  hcell(136) <= hc(136) and 
          not ( hc( 47) or hc( 48) or hc( 49) or hc(135) or hc(137) );

  hcell(137) <= hc(137) and 
          not ( hc( 48) or hc( 49) or hc( 50) );

  hcell(138) <= hc(138) and 
          not ( hc( 49) or hc( 50) or hc( 51) or hc(137) or hc(139) );

  hcell(139) <= hc(139) and 
          not ( hc( 50) or hc( 51) or hc( 52) );

  hcell(140) <= hc(140) and 
          not ( hc( 51) or hc( 52) or hc( 53) or hc(139) or hc(141) );

  hcell(141) <= hc(141) and 
          not ( hc( 52) or hc( 53) or hc( 54) );

  hcell(142) <= hc(142) and 
          not ( hc( 53) or hc( 54) or hc( 55) or hc(141) or hc(143) );

  hcell(143) <= hc(143) and 
          not ( hc( 54) or hc( 55) or hc( 56) );

  hcell(144) <= hc(144) and 
          not ( hc( 55) or hc( 56) or hc( 57) or hc(143) or hc(145) );

  hcell(145) <= hc(145) and 
          not ( hc( 56) or hc( 57) or hc( 58) );

  hcell(146) <= hc(146) and 
          not ( hc( 57) or hc( 58) or hc( 59) or hc(145) or hc(147) );

  hcell(147) <= hc(147) and 
          not ( hc( 58) or hc( 59) or hc( 60) );

  hcell(148) <= hc(148) and 
          not ( hc( 59) or hc( 60) or hc( 61) or hc(147) or hc(149) );

  hcell(149) <= hc(149) and 
          not ( hc( 60) or hc( 61) or hc( 62) );

  hcell(150) <= hc(150) and 
          not ( hc( 61) or hc( 62) or hc( 63) or hc(149) or hc(151) );

  hcell(151) <= hc(151) and 
          not ( hc( 62) or hc( 63) or hc( 64) );

  hcell(152) <= hc(152) and 
          not ( hc( 63) or hc( 64) or hc( 65) or hc(151) or hc(153) );

  hcell(153) <= hc(153) and 
          not ( hc( 64) or hc( 65) or hc( 66) );

  hcell(154) <= hc(154) and 
          not ( hc( 65) or hc( 66) or hc( 67) or hc(153) or hc(155) );

  hcell(155) <= hc(155) and 
          not ( hc( 66) or hc( 67) or hc( 68) );

  hcell(156) <= hc(156) and 
          not ( hc( 67) or hc( 68) or hc( 69) or hc(155) or hc(157) );

  hcell(157) <= hc(157) and 
          not ( hc( 68) or hc( 69) or hc( 70) );

  hcell(158) <= hc(158) and 
          not ( hc( 69) or hc( 70) or hc( 71) or hc(157) or hc(159) );

  hcell(159) <= hc(159) and 
          not ( hc( 70) or hc( 71) or hc( 72) );

  hcell(160) <= hc(160) and 
          not ( hc( 71) or hc( 72) or hc( 73) or hc(159) or hc(161) );

  hcell(161) <= hc(161) and 
          not ( hc( 72) or hc( 73) or hc( 74) );

  hcell(162) <= hc(162) and 
          not ( hc( 73) or hc( 74) or hc( 75) or hc(161) or hc(163) );

  hcell(163) <= hc(163) and 
          not ( hc( 74) or hc( 75) or hc( 76) );

  hcell(164) <= hc(164) and 
          not ( hc( 75) or hc( 76) or hc( 77) or hc(163) or hc(165) );

  hcell(165) <= hc(165) and 
          not ( hc( 76) or hc( 77) or hc( 78) );

  hcell(166) <= hc(166) and 
          not ( hc( 77) or hc( 78) or hc( 79) or hc(165) or hc(167) );

  hcell(167) <= hc(167) and 
          not ( hc( 78) or hc( 79) or hc( 80) );

  hcell(168) <= hc(168) and 
          not ( hc( 79) or hc( 80) or hc( 81) or hc(167) or hc(169) );

  hcell(169) <= hc(169) and 
          not ( hc( 80) or hc( 81) or hc( 82) );

  hcell(170) <= hc(170) and 
          not ( hc( 81) or hc( 82) or hc( 83) or hc(169) or hc(171) );

  hcell(171) <= hc(171) and 
          not ( hc( 82) or hc( 83) or hc( 84) );

  hcell(172) <= hc(172) and 
          not ( hc( 83) or hc( 84) or hc( 85) or hc(171) or hc(173) );

  hcell(173) <= hc(173) and 
          not ( hc( 84) or hc( 85) or hc( 86) );

  hcell(174) <= hc(174) and 
          not ( hc( 85) or hc( 86) or hc( 87) or hc(173) or hc(175) );

  hcell(175) <= hc(175) and 
          not ( hl( 44) or hc( 86) or hc( 87) );

  hcell(176) <= hc(176) and 
          not ( hc( 88) or hc( 89) or hr(131) or hc(177) or hr(219) );

  hcell(177) <= hc(177) and 
          not ( hc( 88) or hc( 89) or hc( 90) );

  hcell(178) <= hc(178) and 
          not ( hc( 89) or hc( 90) or hc( 91) or hc(177) or hc(179) );

  hcell(179) <= hc(179) and 
          not ( hc( 90) or hc( 91) or hc( 92) );

  hcell(180) <= hc(180) and 
          not ( hc( 91) or hc( 92) or hc( 93) or hc(179) or hc(181) );

  hcell(181) <= hc(181) and 
          not ( hc( 92) or hc( 93) or hc( 94) );

  hcell(182) <= hc(182) and 
          not ( hc( 93) or hc( 94) or hc( 95) or hc(181) or hc(183) );

  hcell(183) <= hc(183) and 
          not ( hc( 94) or hc( 95) or hc( 96) );

  hcell(184) <= hc(184) and 
          not ( hc( 95) or hc( 96) or hc( 97) or hc(183) or hc(185) );

  hcell(185) <= hc(185) and 
          not ( hc( 96) or hc( 97) or hc( 98) );

  hcell(186) <= hc(186) and 
          not ( hc( 97) or hc( 98) or hc( 99) or hc(185) or hc(187) );

  hcell(187) <= hc(187) and 
          not ( hc( 98) or hc( 99) or hc(100) );

  hcell(188) <= hc(188) and 
          not ( hc( 99) or hc(100) or hc(101) or hc(187) or hc(189) );

  hcell(189) <= hc(189) and 
          not ( hc(100) or hc(101) or hc(102) );

  hcell(190) <= hc(190) and 
          not ( hc(101) or hc(102) or hc(103) or hc(189) or hc(191) );

  hcell(191) <= hc(191) and 
          not ( hc(102) or hc(103) or hc(104) );

  hcell(192) <= hc(192) and 
          not ( hc(103) or hc(104) or hc(105) or hc(191) or hc(193) );

  hcell(193) <= hc(193) and 
          not ( hc(104) or hc(105) or hc(106) );

  hcell(194) <= hc(194) and 
          not ( hc(105) or hc(106) or hc(107) or hc(193) or hc(195) );

  hcell(195) <= hc(195) and 
          not ( hc(106) or hc(107) or hc(108) );

  hcell(196) <= hc(196) and 
          not ( hc(107) or hc(108) or hc(109) or hc(195) or hc(197) );

  hcell(197) <= hc(197) and 
          not ( hc(108) or hc(109) or hc(110) );

  hcell(198) <= hc(198) and 
          not ( hc(109) or hc(110) or hc(111) or hc(197) or hc(199) );

  hcell(199) <= hc(199) and 
          not ( hc(110) or hc(111) or hc(112) );

  hcell(200) <= hc(200) and 
          not ( hc(111) or hc(112) or hc(113) or hc(199) or hc(201) );

  hcell(201) <= hc(201) and 
          not ( hc(112) or hc(113) or hc(114) );

  hcell(202) <= hc(202) and 
          not ( hc(113) or hc(114) or hc(115) or hc(201) or hc(203) );

  hcell(203) <= hc(203) and 
          not ( hc(114) or hc(115) or hc(116) );

  hcell(204) <= hc(204) and 
          not ( hc(115) or hc(116) or hc(117) or hc(203) or hc(205) );

  hcell(205) <= hc(205) and 
          not ( hc(116) or hc(117) or hc(118) );

  hcell(206) <= hc(206) and 
          not ( hc(117) or hc(118) or hc(119) or hc(205) or hc(207) );

  hcell(207) <= hc(207) and 
          not ( hc(118) or hc(119) or hc(120) );

  hcell(208) <= hc(208) and 
          not ( hc(119) or hc(120) or hc(121) or hc(207) or hc(209) );

  hcell(209) <= hc(209) and 
          not ( hc(120) or hc(121) or hc(122) );

  hcell(210) <= hc(210) and 
          not ( hc(121) or hc(122) or hc(123) or hc(209) or hc(211) );

  hcell(211) <= hc(211) and 
          not ( hc(122) or hc(123) or hc(124) );

  hcell(212) <= hc(212) and 
          not ( hc(123) or hc(124) or hc(125) or hc(211) or hc(213) );

  hcell(213) <= hc(213) and 
          not ( hc(124) or hc(125) or hc(126) );

  hcell(214) <= hc(214) and 
          not ( hc(125) or hc(126) or hc(127) or hc(213) or hc(215) );

  hcell(215) <= hc(215) and 
          not ( hc(126) or hc(127) or hc(128) );

  hcell(216) <= hc(216) and 
          not ( hc(127) or hc(128) or hc(129) or hc(215) or hc(217) );

  hcell(217) <= hc(217) and 
          not ( hc(128) or hc(129) or hc(130) );

  hcell(218) <= hc(218) and 
          not ( hc(129) or hc(130) or hc(131) or hc(217) or hc(219) );

  hcell(219) <= hc(219) and 
          not ( hl( 88) or hc(130) or hc(131) );

  hcell(220) <= hc(220) and 
          not ( hc(132) or hc(133) or hr(175) or hc(221) or hr(263) );

  hcell(221) <= hc(221) and 
          not ( hc(132) or hc(133) or hc(134) );

  hcell(222) <= hc(222) and 
          not ( hc(133) or hc(134) or hc(135) or hc(221) or hc(223) );

  hcell(223) <= hc(223) and 
          not ( hc(134) or hc(135) or hc(136) );

  hcell(224) <= hc(224) and 
          not ( hc(135) or hc(136) or hc(137) or hc(223) or hc(225) );

  hcell(225) <= hc(225) and 
          not ( hc(136) or hc(137) or hc(138) );

  hcell(226) <= hc(226) and 
          not ( hc(137) or hc(138) or hc(139) or hc(225) or hc(227) );

  hcell(227) <= hc(227) and 
          not ( hc(138) or hc(139) or hc(140) );

  hcell(228) <= hc(228) and 
          not ( hc(139) or hc(140) or hc(141) or hc(227) or hc(229) );

  hcell(229) <= hc(229) and 
          not ( hc(140) or hc(141) or hc(142) );

  hcell(230) <= hc(230) and 
          not ( hc(141) or hc(142) or hc(143) or hc(229) or hc(231) );

  hcell(231) <= hc(231) and 
          not ( hc(142) or hc(143) or hc(144) );

  hcell(232) <= hc(232) and 
          not ( hc(143) or hc(144) or hc(145) or hc(231) or hc(233) );

  hcell(233) <= hc(233) and 
          not ( hc(144) or hc(145) or hc(146) );

  hcell(234) <= hc(234) and 
          not ( hc(145) or hc(146) or hc(147) or hc(233) or hc(235) );

  hcell(235) <= hc(235) and 
          not ( hc(146) or hc(147) or hc(148) );

  hcell(236) <= hc(236) and 
          not ( hc(147) or hc(148) or hc(149) or hc(235) or hc(237) );

  hcell(237) <= hc(237) and 
          not ( hc(148) or hc(149) or hc(150) );

  hcell(238) <= hc(238) and 
          not ( hc(149) or hc(150) or hc(151) or hc(237) or hc(239) );

  hcell(239) <= hc(239) and 
          not ( hc(150) or hc(151) or hc(152) );

  hcell(240) <= hc(240) and 
          not ( hc(151) or hc(152) or hc(153) or hc(239) or hc(241) );

  hcell(241) <= hc(241) and 
          not ( hc(152) or hc(153) or hc(154) );

  hcell(242) <= hc(242) and 
          not ( hc(153) or hc(154) or hc(155) or hc(241) or hc(243) );

  hcell(243) <= hc(243) and 
          not ( hc(154) or hc(155) or hc(156) );

  hcell(244) <= hc(244) and 
          not ( hc(155) or hc(156) or hc(157) or hc(243) or hc(245) );

  hcell(245) <= hc(245) and 
          not ( hc(156) or hc(157) or hc(158) );

  hcell(246) <= hc(246) and 
          not ( hc(157) or hc(158) or hc(159) or hc(245) or hc(247) );

  hcell(247) <= hc(247) and 
          not ( hc(158) or hc(159) or hc(160) );

  hcell(248) <= hc(248) and 
          not ( hc(159) or hc(160) or hc(161) or hc(247) or hc(249) );

  hcell(249) <= hc(249) and 
          not ( hc(160) or hc(161) or hc(162) );

  hcell(250) <= hc(250) and 
          not ( hc(161) or hc(162) or hc(163) or hc(249) or hc(251) );

  hcell(251) <= hc(251) and 
          not ( hc(162) or hc(163) or hc(164) );

  hcell(252) <= hc(252) and 
          not ( hc(163) or hc(164) or hc(165) or hc(251) or hc(253) );

  hcell(253) <= hc(253) and 
          not ( hc(164) or hc(165) or hc(166) );

  hcell(254) <= hc(254) and 
          not ( hc(165) or hc(166) or hc(167) or hc(253) or hc(255) );

  hcell(255) <= hc(255) and 
          not ( hc(166) or hc(167) or hc(168) );

  hcell(256) <= hc(256) and 
          not ( hc(167) or hc(168) or hc(169) or hc(255) or hc(257) );

  hcell(257) <= hc(257) and 
          not ( hc(168) or hc(169) or hc(170) );

  hcell(258) <= hc(258) and 
          not ( hc(169) or hc(170) or hc(171) or hc(257) or hc(259) );

  hcell(259) <= hc(259) and 
          not ( hc(170) or hc(171) or hc(172) );

  hcell(260) <= hc(260) and 
          not ( hc(171) or hc(172) or hc(173) or hc(259) or hc(261) );

  hcell(261) <= hc(261) and 
          not ( hc(172) or hc(173) or hc(174) );

  hcell(262) <= hc(262) and 
          not ( hc(173) or hc(174) or hc(175) or hc(261) or hc(263) );

  hcell(263) <= hc(263) and 
          not ( hl(132) or hc(174) or hc(175) );

  hcell(264) <= hc(264) and 
          not ( hc(176) or hc(177) or hr(219) or hc(265) or hr(307) );

  hcell(265) <= hc(265) and 
          not ( hc(176) or hc(177) or hc(178) );

  hcell(266) <= hc(266) and 
          not ( hc(177) or hc(178) or hc(179) or hc(265) or hc(267) );

  hcell(267) <= hc(267) and 
          not ( hc(178) or hc(179) or hc(180) );

  hcell(268) <= hc(268) and 
          not ( hc(179) or hc(180) or hc(181) or hc(267) or hc(269) );

  hcell(269) <= hc(269) and 
          not ( hc(180) or hc(181) or hc(182) );

  hcell(270) <= hc(270) and 
          not ( hc(181) or hc(182) or hc(183) or hc(269) or hc(271) );

  hcell(271) <= hc(271) and 
          not ( hc(182) or hc(183) or hc(184) );

  hcell(272) <= hc(272) and 
          not ( hc(183) or hc(184) or hc(185) or hc(271) or hc(273) );

  hcell(273) <= hc(273) and 
          not ( hc(184) or hc(185) or hc(186) );

  hcell(274) <= hc(274) and 
          not ( hc(185) or hc(186) or hc(187) or hc(273) or hc(275) );

  hcell(275) <= hc(275) and 
          not ( hc(186) or hc(187) or hc(188) );

  hcell(276) <= hc(276) and 
          not ( hc(187) or hc(188) or hc(189) or hc(275) or hc(277) );

  hcell(277) <= hc(277) and 
          not ( hc(188) or hc(189) or hc(190) );

  hcell(278) <= hc(278) and 
          not ( hc(189) or hc(190) or hc(191) or hc(277) or hc(279) );

  hcell(279) <= hc(279) and 
          not ( hc(190) or hc(191) or hc(192) );

  hcell(280) <= hc(280) and 
          not ( hc(191) or hc(192) or hc(193) or hc(279) or hc(281) );

  hcell(281) <= hc(281) and 
          not ( hc(192) or hc(193) or hc(194) );

  hcell(282) <= hc(282) and 
          not ( hc(193) or hc(194) or hc(195) or hc(281) or hc(283) );

  hcell(283) <= hc(283) and 
          not ( hc(194) or hc(195) or hc(196) );

  hcell(284) <= hc(284) and 
          not ( hc(195) or hc(196) or hc(197) or hc(283) or hc(285) );

  hcell(285) <= hc(285) and 
          not ( hc(196) or hc(197) or hc(198) );

  hcell(286) <= hc(286) and 
          not ( hc(197) or hc(198) or hc(199) or hc(285) or hc(287) );

  hcell(287) <= hc(287) and 
          not ( hc(198) or hc(199) or hc(200) );

  hcell(288) <= hc(288) and 
          not ( hc(199) or hc(200) or hc(201) or hc(287) or hc(289) );

  hcell(289) <= hc(289) and 
          not ( hc(200) or hc(201) or hc(202) );

  hcell(290) <= hc(290) and 
          not ( hc(201) or hc(202) or hc(203) or hc(289) or hc(291) );

  hcell(291) <= hc(291) and 
          not ( hc(202) or hc(203) or hc(204) );

  hcell(292) <= hc(292) and 
          not ( hc(203) or hc(204) or hc(205) or hc(291) or hc(293) );

  hcell(293) <= hc(293) and 
          not ( hc(204) or hc(205) or hc(206) );

  hcell(294) <= hc(294) and 
          not ( hc(205) or hc(206) or hc(207) or hc(293) or hc(295) );

  hcell(295) <= hc(295) and 
          not ( hc(206) or hc(207) or hc(208) );

  hcell(296) <= hc(296) and 
          not ( hc(207) or hc(208) or hc(209) or hc(295) or hc(297) );

  hcell(297) <= hc(297) and 
          not ( hc(208) or hc(209) or hc(210) );

  hcell(298) <= hc(298) and 
          not ( hc(209) or hc(210) or hc(211) or hc(297) or hc(299) );

  hcell(299) <= hc(299) and 
          not ( hc(210) or hc(211) or hc(212) );

  hcell(300) <= hc(300) and 
          not ( hc(211) or hc(212) or hc(213) or hc(299) or hc(301) );

  hcell(301) <= hc(301) and 
          not ( hc(212) or hc(213) or hc(214) );

  hcell(302) <= hc(302) and 
          not ( hc(213) or hc(214) or hc(215) or hc(301) or hc(303) );

  hcell(303) <= hc(303) and 
          not ( hc(214) or hc(215) or hc(216) );

  hcell(304) <= hc(304) and 
          not ( hc(215) or hc(216) or hc(217) or hc(303) or hc(305) );

  hcell(305) <= hc(305) and 
          not ( hc(216) or hc(217) or hc(218) );

  hcell(306) <= hc(306) and 
          not ( hc(217) or hc(218) or hc(219) or hc(305) or hc(307) );

  hcell(307) <= hc(307) and 
          not ( hl(176) or hc(218) or hc(219) );

  hcell(308) <= hc(308) and 
          not ( hc(220) or hc(221) or hr(263) or hc(309) or hr(351) );

  hcell(309) <= hc(309) and 
          not ( hc(220) or hc(221) or hc(222) );

  hcell(310) <= hc(310) and 
          not ( hc(221) or hc(222) or hc(223) or hc(309) or hc(311) );

  hcell(311) <= hc(311) and 
          not ( hc(222) or hc(223) or hc(224) );

  hcell(312) <= hc(312) and 
          not ( hc(223) or hc(224) or hc(225) or hc(311) or hc(313) );

  hcell(313) <= hc(313) and 
          not ( hc(224) or hc(225) or hc(226) );

  hcell(314) <= hc(314) and 
          not ( hc(225) or hc(226) or hc(227) or hc(313) or hc(315) );

  hcell(315) <= hc(315) and 
          not ( hc(226) or hc(227) or hc(228) );

  hcell(316) <= hc(316) and 
          not ( hc(227) or hc(228) or hc(229) or hc(315) or hc(317) );

  hcell(317) <= hc(317) and 
          not ( hc(228) or hc(229) or hc(230) );

  hcell(318) <= hc(318) and 
          not ( hc(229) or hc(230) or hc(231) or hc(317) or hc(319) );

  hcell(319) <= hc(319) and 
          not ( hc(230) or hc(231) or hc(232) );

  hcell(320) <= hc(320) and 
          not ( hc(231) or hc(232) or hc(233) or hc(319) or hc(321) );

  hcell(321) <= hc(321) and 
          not ( hc(232) or hc(233) or hc(234) );

  hcell(322) <= hc(322) and 
          not ( hc(233) or hc(234) or hc(235) or hc(321) or hc(323) );

  hcell(323) <= hc(323) and 
          not ( hc(234) or hc(235) or hc(236) );

  hcell(324) <= hc(324) and 
          not ( hc(235) or hc(236) or hc(237) or hc(323) or hc(325) );

  hcell(325) <= hc(325) and 
          not ( hc(236) or hc(237) or hc(238) );

  hcell(326) <= hc(326) and 
          not ( hc(237) or hc(238) or hc(239) or hc(325) or hc(327) );

  hcell(327) <= hc(327) and 
          not ( hc(238) or hc(239) or hc(240) );

  hcell(328) <= hc(328) and 
          not ( hc(239) or hc(240) or hc(241) or hc(327) or hc(329) );

  hcell(329) <= hc(329) and 
          not ( hc(240) or hc(241) or hc(242) );

  hcell(330) <= hc(330) and 
          not ( hc(241) or hc(242) or hc(243) or hc(329) or hc(331) );

  hcell(331) <= hc(331) and 
          not ( hc(242) or hc(243) or hc(244) );

  hcell(332) <= hc(332) and 
          not ( hc(243) or hc(244) or hc(245) or hc(331) or hc(333) );

  hcell(333) <= hc(333) and 
          not ( hc(244) or hc(245) or hc(246) );

  hcell(334) <= hc(334) and 
          not ( hc(245) or hc(246) or hc(247) or hc(333) or hc(335) );

  hcell(335) <= hc(335) and 
          not ( hc(246) or hc(247) or hc(248) );

  hcell(336) <= hc(336) and 
          not ( hc(247) or hc(248) or hc(249) or hc(335) or hc(337) );

  hcell(337) <= hc(337) and 
          not ( hc(248) or hc(249) or hc(250) );

  hcell(338) <= hc(338) and 
          not ( hc(249) or hc(250) or hc(251) or hc(337) or hc(339) );

  hcell(339) <= hc(339) and 
          not ( hc(250) or hc(251) or hc(252) );

  hcell(340) <= hc(340) and 
          not ( hc(251) or hc(252) or hc(253) or hc(339) or hc(341) );

  hcell(341) <= hc(341) and 
          not ( hc(252) or hc(253) or hc(254) );

  hcell(342) <= hc(342) and 
          not ( hc(253) or hc(254) or hc(255) or hc(341) or hc(343) );

  hcell(343) <= hc(343) and 
          not ( hc(254) or hc(255) or hc(256) );

  hcell(344) <= hc(344) and 
          not ( hc(255) or hc(256) or hc(257) or hc(343) or hc(345) );

  hcell(345) <= hc(345) and 
          not ( hc(256) or hc(257) or hc(258) );

  hcell(346) <= hc(346) and 
          not ( hc(257) or hc(258) or hc(259) or hc(345) or hc(347) );

  hcell(347) <= hc(347) and 
          not ( hc(258) or hc(259) or hc(260) );

  hcell(348) <= hc(348) and 
          not ( hc(259) or hc(260) or hc(261) or hc(347) or hc(349) );

  hcell(349) <= hc(349) and 
          not ( hc(260) or hc(261) or hc(262) );

  hcell(350) <= hc(350) and 
          not ( hc(261) or hc(262) or hc(263) or hc(349) or hc(351) );

  hcell(351) <= hc(351) and 
          not ( hl(220) or hc(262) or hc(263) );

  ncell(  0) <= nc(  0) and 
          not ( hc(264) or hc(265) or hr(307) or nc(  1) or nr( 43) );

  ncell(  1) <= nc(  1) and 
          not ( hc(264) or hc(265) or hc(266) );

  ncell(  2) <= nc(  2) and 
          not ( hc(265) or hc(266) or hc(267) or nc(  1) or nc(  3) );

  ncell(  3) <= nc(  3) and 
          not ( hc(266) or hc(267) or hc(268) );

  ncell(  4) <= nc(  4) and 
          not ( hc(267) or hc(268) or hc(269) or nc(  3) or nc(  5) );

  ncell(  5) <= nc(  5) and 
          not ( hc(268) or hc(269) or hc(270) );

  ncell(  6) <= nc(  6) and 
          not ( hc(269) or hc(270) or hc(271) or nc(  5) or nc(  7) );

  ncell(  7) <= nc(  7) and 
          not ( hc(270) or hc(271) or hc(272) );

  ncell(  8) <= nc(  8) and 
          not ( hc(271) or hc(272) or hc(273) or nc(  7) or nc(  9) );

  ncell(  9) <= nc(  9) and 
          not ( hc(272) or hc(273) or hc(274) );

  ncell( 10) <= nc( 10) and 
          not ( hc(273) or hc(274) or hc(275) or nc(  9) or nc( 11) );

  ncell( 11) <= nc( 11) and 
          not ( hc(274) or hc(275) or hc(276) );

  ncell( 12) <= nc( 12) and 
          not ( hc(275) or hc(276) or hc(277) or nc( 11) or nc( 13) );

  ncell( 13) <= nc( 13) and 
          not ( hc(276) or hc(277) or hc(278) );

  ncell( 14) <= nc( 14) and 
          not ( hc(277) or hc(278) or hc(279) or nc( 13) or nc( 15) );

  ncell( 15) <= nc( 15) and 
          not ( hc(278) or hc(279) or hc(280) );

  ncell( 16) <= nc( 16) and 
          not ( hc(279) or hc(280) or hc(281) or nc( 15) or nc( 17) );

  ncell( 17) <= nc( 17) and 
          not ( hc(280) or hc(281) or hc(282) );

  ncell( 18) <= nc( 18) and 
          not ( hc(281) or hc(282) or hc(283) or nc( 17) or nc( 19) );

  ncell( 19) <= nc( 19) and 
          not ( hc(282) or hc(283) or hc(284) );

  ncell( 20) <= nc( 20) and 
          not ( hc(283) or hc(284) or hc(285) or nc( 19) or nc( 21) );

  ncell( 21) <= nc( 21) and 
          not ( hc(284) or hc(285) or hc(286) );

  ncell( 22) <= nc( 22) and 
          not ( hc(285) or hc(286) or hc(287) or nc( 21) or nc( 23) );

  ncell( 23) <= nc( 23) and 
          not ( hc(286) or hc(287) or hc(288) );

  ncell( 24) <= nc( 24) and 
          not ( hc(287) or hc(288) or hc(289) or nc( 23) or nc( 25) );

  ncell( 25) <= nc( 25) and 
          not ( hc(288) or hc(289) or hc(290) );

  ncell( 26) <= nc( 26) and 
          not ( hc(289) or hc(290) or hc(291) or nc( 25) or nc( 27) );

  ncell( 27) <= nc( 27) and 
          not ( hc(290) or hc(291) or hc(292) );

  ncell( 28) <= nc( 28) and 
          not ( hc(291) or hc(292) or hc(293) or nc( 27) or nc( 29) );

  ncell( 29) <= nc( 29) and 
          not ( hc(292) or hc(293) or hc(294) );

  ncell( 30) <= nc( 30) and 
          not ( hc(293) or hc(294) or hc(295) or nc( 29) or nc( 31) );

  ncell( 31) <= nc( 31) and 
          not ( hc(294) or hc(295) or hc(296) );

  ncell( 32) <= nc( 32) and 
          not ( hc(295) or hc(296) or hc(297) or nc( 31) or nc( 33) );

  ncell( 33) <= nc( 33) and 
          not ( hc(296) or hc(297) or hc(298) );

  ncell( 34) <= nc( 34) and 
          not ( hc(297) or hc(298) or hc(299) or nc( 33) or nc( 35) );

  ncell( 35) <= nc( 35) and 
          not ( hc(298) or hc(299) or hc(300) );

  ncell( 36) <= nc( 36) and 
          not ( hc(299) or hc(300) or hc(301) or nc( 35) or nc( 37) );

  ncell( 37) <= nc( 37) and 
          not ( hc(300) or hc(301) or hc(302) );

  ncell( 38) <= nc( 38) and 
          not ( hc(301) or hc(302) or hc(303) or nc( 37) or nc( 39) );

  ncell( 39) <= nc( 39) and 
          not ( hc(302) or hc(303) or hc(304) );

  ncell( 40) <= nc( 40) and 
          not ( hc(303) or hc(304) or hc(305) or nc( 39) or nc( 41) );

  ncell( 41) <= nc( 41) and 
          not ( hc(304) or hc(305) or hc(306) );

  ncell( 42) <= nc( 42) and 
          not ( hc(305) or hc(306) or hc(307) or nc( 41) or nc( 43) );

  ncell( 43) <= nc( 43) and 
          not ( hl(264) or hc(306) or hc(307) );

  ncell( 44) <= nc( 44) and 
          not ( hc(308) or hc(309) or hr(351) or nc( 45) or nr( 87) );

  ncell( 45) <= nc( 45) and 
          not ( hc(308) or hc(309) or hc(310) );

  ncell( 46) <= nc( 46) and 
          not ( hc(309) or hc(310) or hc(311) or nc( 45) or nc( 47) );

  ncell( 47) <= nc( 47) and 
          not ( hc(310) or hc(311) or hc(312) );

  ncell( 48) <= nc( 48) and 
          not ( hc(311) or hc(312) or hc(313) or nc( 47) or nc( 49) );

  ncell( 49) <= nc( 49) and 
          not ( hc(312) or hc(313) or hc(314) );

  ncell( 50) <= nc( 50) and 
          not ( hc(313) or hc(314) or hc(315) or nc( 49) or nc( 51) );

  ncell( 51) <= nc( 51) and 
          not ( hc(314) or hc(315) or hc(316) );

  ncell( 52) <= nc( 52) and 
          not ( hc(315) or hc(316) or hc(317) or nc( 51) or nc( 53) );

  ncell( 53) <= nc( 53) and 
          not ( hc(316) or hc(317) or hc(318) );

  ncell( 54) <= nc( 54) and 
          not ( hc(317) or hc(318) or hc(319) or nc( 53) or nc( 55) );

  ncell( 55) <= nc( 55) and 
          not ( hc(318) or hc(319) or hc(320) );

  ncell( 56) <= nc( 56) and 
          not ( hc(319) or hc(320) or hc(321) or nc( 55) or nc( 57) );

  ncell( 57) <= nc( 57) and 
          not ( hc(320) or hc(321) or hc(322) );

  ncell( 58) <= nc( 58) and 
          not ( hc(321) or hc(322) or hc(323) or nc( 57) or nc( 59) );

  ncell( 59) <= nc( 59) and 
          not ( hc(322) or hc(323) or hc(324) );

  ncell( 60) <= nc( 60) and 
          not ( hc(323) or hc(324) or hc(325) or nc( 59) or nc( 61) );

  ncell( 61) <= nc( 61) and 
          not ( hc(324) or hc(325) or hc(326) );

  ncell( 62) <= nc( 62) and 
          not ( hc(325) or hc(326) or hc(327) or nc( 61) or nc( 63) );

  ncell( 63) <= nc( 63) and 
          not ( hc(326) or hc(327) or hc(328) );

  ncell( 64) <= nc( 64) and 
          not ( hc(327) or hc(328) or hc(329) or nc( 63) or nc( 65) );

  ncell( 65) <= nc( 65) and 
          not ( hc(328) or hc(329) or hc(330) );

  ncell( 66) <= nc( 66) and 
          not ( hc(329) or hc(330) or hc(331) or nc( 65) or nc( 67) );

  ncell( 67) <= nc( 67) and 
          not ( hc(330) or hc(331) or hc(332) );

  ncell( 68) <= nc( 68) and 
          not ( hc(331) or hc(332) or hc(333) or nc( 67) or nc( 69) );

  ncell( 69) <= nc( 69) and 
          not ( hc(332) or hc(333) or hc(334) );

  ncell( 70) <= nc( 70) and 
          not ( hc(333) or hc(334) or hc(335) or nc( 69) or nc( 71) );

  ncell( 71) <= nc( 71) and 
          not ( hc(334) or hc(335) or hc(336) );

  ncell( 72) <= nc( 72) and 
          not ( hc(335) or hc(336) or hc(337) or nc( 71) or nc( 73) );

  ncell( 73) <= nc( 73) and 
          not ( hc(336) or hc(337) or hc(338) );

  ncell( 74) <= nc( 74) and 
          not ( hc(337) or hc(338) or hc(339) or nc( 73) or nc( 75) );

  ncell( 75) <= nc( 75) and 
          not ( hc(338) or hc(339) or hc(340) );

  ncell( 76) <= nc( 76) and 
          not ( hc(339) or hc(340) or hc(341) or nc( 75) or nc( 77) );

  ncell( 77) <= nc( 77) and 
          not ( hc(340) or hc(341) or hc(342) );

  ncell( 78) <= nc( 78) and 
          not ( hc(341) or hc(342) or hc(343) or nc( 77) or nc( 79) );

  ncell( 79) <= nc( 79) and 
          not ( hc(342) or hc(343) or hc(344) );

  ncell( 80) <= nc( 80) and 
          not ( hc(343) or hc(344) or hc(345) or nc( 79) or nc( 81) );

  ncell( 81) <= nc( 81) and 
          not ( hc(344) or hc(345) or hc(346) );

  ncell( 82) <= nc( 82) and 
          not ( hc(345) or hc(346) or hc(347) or nc( 81) or nc( 83) );

  ncell( 83) <= nc( 83) and 
          not ( hc(346) or hc(347) or hc(348) );

  ncell( 84) <= nc( 84) and 
          not ( hc(347) or hc(348) or hc(349) or nc( 83) or nc( 85) );

  ncell( 85) <= nc( 85) and 
          not ( hc(348) or hc(349) or hc(350) );

  ncell( 86) <= nc( 86) and 
          not ( hc(349) or hc(350) or hc(351) or nc( 85) or nc( 87) );

  ncell( 87) <= nc( 87) and 
          not ( hl(308) or hc(350) or hc(351) );

  ncell( 88) <= nc( 88) and 
          not ( nc(  0) or nc(  1) or nr( 43) or nc( 89) or nr(131) );

  ncell( 89) <= nc( 89) and 
          not ( nc(  0) or nc(  1) or nc(  2) );

  ncell( 90) <= nc( 90) and 
          not ( nc(  1) or nc(  2) or nc(  3) or nc( 89) or nc( 91) );

  ncell( 91) <= nc( 91) and 
          not ( nc(  2) or nc(  3) or nc(  4) );

  ncell( 92) <= nc( 92) and 
          not ( nc(  3) or nc(  4) or nc(  5) or nc( 91) or nc( 93) );

  ncell( 93) <= nc( 93) and 
          not ( nc(  4) or nc(  5) or nc(  6) );

  ncell( 94) <= nc( 94) and 
          not ( nc(  5) or nc(  6) or nc(  7) or nc( 93) or nc( 95) );

  ncell( 95) <= nc( 95) and 
          not ( nc(  6) or nc(  7) or nc(  8) );

  ncell( 96) <= nc( 96) and 
          not ( nc(  7) or nc(  8) or nc(  9) or nc( 95) or nc( 97) );

  ncell( 97) <= nc( 97) and 
          not ( nc(  8) or nc(  9) or nc( 10) );

  ncell( 98) <= nc( 98) and 
          not ( nc(  9) or nc( 10) or nc( 11) or nc( 97) or nc( 99) );

  ncell( 99) <= nc( 99) and 
          not ( nc( 10) or nc( 11) or nc( 12) );

  ncell(100) <= nc(100) and 
          not ( nc( 11) or nc( 12) or nc( 13) or nc( 99) or nc(101) );

  ncell(101) <= nc(101) and 
          not ( nc( 12) or nc( 13) or nc( 14) );

  ncell(102) <= nc(102) and 
          not ( nc( 13) or nc( 14) or nc( 15) or nc(101) or nc(103) );

  ncell(103) <= nc(103) and 
          not ( nc( 14) or nc( 15) or nc( 16) );

  ncell(104) <= nc(104) and 
          not ( nc( 15) or nc( 16) or nc( 17) or nc(103) or nc(105) );

  ncell(105) <= nc(105) and 
          not ( nc( 16) or nc( 17) or nc( 18) );

  ncell(106) <= nc(106) and 
          not ( nc( 17) or nc( 18) or nc( 19) or nc(105) or nc(107) );

  ncell(107) <= nc(107) and 
          not ( nc( 18) or nc( 19) or nc( 20) );

  ncell(108) <= nc(108) and 
          not ( nc( 19) or nc( 20) or nc( 21) or nc(107) or nc(109) );

  ncell(109) <= nc(109) and 
          not ( nc( 20) or nc( 21) or nc( 22) );

  ncell(110) <= nc(110) and 
          not ( nc( 21) or nc( 22) or nc( 23) or nc(109) or nc(111) );

  ncell(111) <= nc(111) and 
          not ( nc( 22) or nc( 23) or nc( 24) );

  ncell(112) <= nc(112) and 
          not ( nc( 23) or nc( 24) or nc( 25) or nc(111) or nc(113) );

  ncell(113) <= nc(113) and 
          not ( nc( 24) or nc( 25) or nc( 26) );

  ncell(114) <= nc(114) and 
          not ( nc( 25) or nc( 26) or nc( 27) or nc(113) or nc(115) );

  ncell(115) <= nc(115) and 
          not ( nc( 26) or nc( 27) or nc( 28) );

  ncell(116) <= nc(116) and 
          not ( nc( 27) or nc( 28) or nc( 29) or nc(115) or nc(117) );

  ncell(117) <= nc(117) and 
          not ( nc( 28) or nc( 29) or nc( 30) );

  ncell(118) <= nc(118) and 
          not ( nc( 29) or nc( 30) or nc( 31) or nc(117) or nc(119) );

  ncell(119) <= nc(119) and 
          not ( nc( 30) or nc( 31) or nc( 32) );

  ncell(120) <= nc(120) and 
          not ( nc( 31) or nc( 32) or nc( 33) or nc(119) or nc(121) );

  ncell(121) <= nc(121) and 
          not ( nc( 32) or nc( 33) or nc( 34) );

  ncell(122) <= nc(122) and 
          not ( nc( 33) or nc( 34) or nc( 35) or nc(121) or nc(123) );

  ncell(123) <= nc(123) and 
          not ( nc( 34) or nc( 35) or nc( 36) );

  ncell(124) <= nc(124) and 
          not ( nc( 35) or nc( 36) or nc( 37) or nc(123) or nc(125) );

  ncell(125) <= nc(125) and 
          not ( nc( 36) or nc( 37) or nc( 38) );

  ncell(126) <= nc(126) and 
          not ( nc( 37) or nc( 38) or nc( 39) or nc(125) or nc(127) );

  ncell(127) <= nc(127) and 
          not ( nc( 38) or nc( 39) or nc( 40) );

  ncell(128) <= nc(128) and 
          not ( nc( 39) or nc( 40) or nc( 41) or nc(127) or nc(129) );

  ncell(129) <= nc(129) and 
          not ( nc( 40) or nc( 41) or nc( 42) );

  ncell(130) <= nc(130) and 
          not ( nc( 41) or nc( 42) or nc( 43) or nc(129) or nc(131) );

  ncell(131) <= nc(131) and 
          not ( nl(  0) or nc( 42) or nc( 43) );

  ncell(132) <= nc(132) and 
          not ( nc( 44) or nc( 45) or nr( 87) or nc(133) or nr(175) );

  ncell(133) <= nc(133) and 
          not ( nc( 44) or nc( 45) or nc( 46) );

  ncell(134) <= nc(134) and 
          not ( nc( 45) or nc( 46) or nc( 47) or nc(133) or nc(135) );

  ncell(135) <= nc(135) and 
          not ( nc( 46) or nc( 47) or nc( 48) );

  ncell(136) <= nc(136) and 
          not ( nc( 47) or nc( 48) or nc( 49) or nc(135) or nc(137) );

  ncell(137) <= nc(137) and 
          not ( nc( 48) or nc( 49) or nc( 50) );

  ncell(138) <= nc(138) and 
          not ( nc( 49) or nc( 50) or nc( 51) or nc(137) or nc(139) );

  ncell(139) <= nc(139) and 
          not ( nc( 50) or nc( 51) or nc( 52) );

  ncell(140) <= nc(140) and 
          not ( nc( 51) or nc( 52) or nc( 53) or nc(139) or nc(141) );

  ncell(141) <= nc(141) and 
          not ( nc( 52) or nc( 53) or nc( 54) );

  ncell(142) <= nc(142) and 
          not ( nc( 53) or nc( 54) or nc( 55) or nc(141) or nc(143) );

  ncell(143) <= nc(143) and 
          not ( nc( 54) or nc( 55) or nc( 56) );

  ncell(144) <= nc(144) and 
          not ( nc( 55) or nc( 56) or nc( 57) or nc(143) or nc(145) );

  ncell(145) <= nc(145) and 
          not ( nc( 56) or nc( 57) or nc( 58) );

  ncell(146) <= nc(146) and 
          not ( nc( 57) or nc( 58) or nc( 59) or nc(145) or nc(147) );

  ncell(147) <= nc(147) and 
          not ( nc( 58) or nc( 59) or nc( 60) );

  ncell(148) <= nc(148) and 
          not ( nc( 59) or nc( 60) or nc( 61) or nc(147) or nc(149) );

  ncell(149) <= nc(149) and 
          not ( nc( 60) or nc( 61) or nc( 62) );

  ncell(150) <= nc(150) and 
          not ( nc( 61) or nc( 62) or nc( 63) or nc(149) or nc(151) );

  ncell(151) <= nc(151) and 
          not ( nc( 62) or nc( 63) or nc( 64) );

  ncell(152) <= nc(152) and 
          not ( nc( 63) or nc( 64) or nc( 65) or nc(151) or nc(153) );

  ncell(153) <= nc(153) and 
          not ( nc( 64) or nc( 65) or nc( 66) );

  ncell(154) <= nc(154) and 
          not ( nc( 65) or nc( 66) or nc( 67) or nc(153) or nc(155) );

  ncell(155) <= nc(155) and 
          not ( nc( 66) or nc( 67) or nc( 68) );

  ncell(156) <= nc(156) and 
          not ( nc( 67) or nc( 68) or nc( 69) or nc(155) or nc(157) );

  ncell(157) <= nc(157) and 
          not ( nc( 68) or nc( 69) or nc( 70) );

  ncell(158) <= nc(158) and 
          not ( nc( 69) or nc( 70) or nc( 71) or nc(157) or nc(159) );

  ncell(159) <= nc(159) and 
          not ( nc( 70) or nc( 71) or nc( 72) );

  ncell(160) <= nc(160) and 
          not ( nc( 71) or nc( 72) or nc( 73) or nc(159) or nc(161) );

  ncell(161) <= nc(161) and 
          not ( nc( 72) or nc( 73) or nc( 74) );

  ncell(162) <= nc(162) and 
          not ( nc( 73) or nc( 74) or nc( 75) or nc(161) or nc(163) );

  ncell(163) <= nc(163) and 
          not ( nc( 74) or nc( 75) or nc( 76) );

  ncell(164) <= nc(164) and 
          not ( nc( 75) or nc( 76) or nc( 77) or nc(163) or nc(165) );

  ncell(165) <= nc(165) and 
          not ( nc( 76) or nc( 77) or nc( 78) );

  ncell(166) <= nc(166) and 
          not ( nc( 77) or nc( 78) or nc( 79) or nc(165) or nc(167) );

  ncell(167) <= nc(167) and 
          not ( nc( 78) or nc( 79) or nc( 80) );

  ncell(168) <= nc(168) and 
          not ( nc( 79) or nc( 80) or nc( 81) or nc(167) or nc(169) );

  ncell(169) <= nc(169) and 
          not ( nc( 80) or nc( 81) or nc( 82) );

  ncell(170) <= nc(170) and 
          not ( nc( 81) or nc( 82) or nc( 83) or nc(169) or nc(171) );

  ncell(171) <= nc(171) and 
          not ( nc( 82) or nc( 83) or nc( 84) );

  ncell(172) <= nc(172) and 
          not ( nc( 83) or nc( 84) or nc( 85) or nc(171) or nc(173) );

  ncell(173) <= nc(173) and 
          not ( nc( 84) or nc( 85) or nc( 86) );

  ncell(174) <= nc(174) and 
          not ( nc( 85) or nc( 86) or nc( 87) or nc(173) or nc(175) );

  ncell(175) <= nc(175) and 
          not ( nl( 44) or nc( 86) or nc( 87) );

  ncell(176) <= nc(176) and 
          not ( nc( 88) or nc( 89) or nr(131) or nc(177) or nr(219) );

  ncell(177) <= nc(177) and 
          not ( nc( 88) or nc( 89) or nc( 90) );

  ncell(178) <= nc(178) and 
          not ( nc( 89) or nc( 90) or nc( 91) or nc(177) or nc(179) );

  ncell(179) <= nc(179) and 
          not ( nc( 90) or nc( 91) or nc( 92) );

  ncell(180) <= nc(180) and 
          not ( nc( 91) or nc( 92) or nc( 93) or nc(179) or nc(181) );

  ncell(181) <= nc(181) and 
          not ( nc( 92) or nc( 93) or nc( 94) );

  ncell(182) <= nc(182) and 
          not ( nc( 93) or nc( 94) or nc( 95) or nc(181) or nc(183) );

  ncell(183) <= nc(183) and 
          not ( nc( 94) or nc( 95) or nc( 96) );

  ncell(184) <= nc(184) and 
          not ( nc( 95) or nc( 96) or nc( 97) or nc(183) or nc(185) );

  ncell(185) <= nc(185) and 
          not ( nc( 96) or nc( 97) or nc( 98) );

  ncell(186) <= nc(186) and 
          not ( nc( 97) or nc( 98) or nc( 99) or nc(185) or nc(187) );

  ncell(187) <= nc(187) and 
          not ( nc( 98) or nc( 99) or nc(100) );

  ncell(188) <= nc(188) and 
          not ( nc( 99) or nc(100) or nc(101) or nc(187) or nc(189) );

  ncell(189) <= nc(189) and 
          not ( nc(100) or nc(101) or nc(102) );

  ncell(190) <= nc(190) and 
          not ( nc(101) or nc(102) or nc(103) or nc(189) or nc(191) );

  ncell(191) <= nc(191) and 
          not ( nc(102) or nc(103) or nc(104) );

  ncell(192) <= nc(192) and 
          not ( nc(103) or nc(104) or nc(105) or nc(191) or nc(193) );

  ncell(193) <= nc(193) and 
          not ( nc(104) or nc(105) or nc(106) );

  ncell(194) <= nc(194) and 
          not ( nc(105) or nc(106) or nc(107) or nc(193) or nc(195) );

  ncell(195) <= nc(195) and 
          not ( nc(106) or nc(107) or nc(108) );

  ncell(196) <= nc(196) and 
          not ( nc(107) or nc(108) or nc(109) or nc(195) or nc(197) );

  ncell(197) <= nc(197) and 
          not ( nc(108) or nc(109) or nc(110) );

  ncell(198) <= nc(198) and 
          not ( nc(109) or nc(110) or nc(111) or nc(197) or nc(199) );

  ncell(199) <= nc(199) and 
          not ( nc(110) or nc(111) or nc(112) );

  ncell(200) <= nc(200) and 
          not ( nc(111) or nc(112) or nc(113) or nc(199) or nc(201) );

  ncell(201) <= nc(201) and 
          not ( nc(112) or nc(113) or nc(114) );

  ncell(202) <= nc(202) and 
          not ( nc(113) or nc(114) or nc(115) or nc(201) or nc(203) );

  ncell(203) <= nc(203) and 
          not ( nc(114) or nc(115) or nc(116) );

  ncell(204) <= nc(204) and 
          not ( nc(115) or nc(116) or nc(117) or nc(203) or nc(205) );

  ncell(205) <= nc(205) and 
          not ( nc(116) or nc(117) or nc(118) );

  ncell(206) <= nc(206) and 
          not ( nc(117) or nc(118) or nc(119) or nc(205) or nc(207) );

  ncell(207) <= nc(207) and 
          not ( nc(118) or nc(119) or nc(120) );

  ncell(208) <= nc(208) and 
          not ( nc(119) or nc(120) or nc(121) or nc(207) or nc(209) );

  ncell(209) <= nc(209) and 
          not ( nc(120) or nc(121) or nc(122) );

  ncell(210) <= nc(210) and 
          not ( nc(121) or nc(122) or nc(123) or nc(209) or nc(211) );

  ncell(211) <= nc(211) and 
          not ( nc(122) or nc(123) or nc(124) );

  ncell(212) <= nc(212) and 
          not ( nc(123) or nc(124) or nc(125) or nc(211) or nc(213) );

  ncell(213) <= nc(213) and 
          not ( nc(124) or nc(125) or nc(126) );

  ncell(214) <= nc(214) and 
          not ( nc(125) or nc(126) or nc(127) or nc(213) or nc(215) );

  ncell(215) <= nc(215) and 
          not ( nc(126) or nc(127) or nc(128) );

  ncell(216) <= nc(216) and 
          not ( nc(127) or nc(128) or nc(129) or nc(215) or nc(217) );

  ncell(217) <= nc(217) and 
          not ( nc(128) or nc(129) or nc(130) );

  ncell(218) <= nc(218) and 
          not ( nc(129) or nc(130) or nc(131) or nc(217) or nc(219) );

  ncell(219) <= nc(219) and 
          not ( nl( 88) or nc(130) or nc(131) );

  ncell(220) <= nc(220) and 
          not ( nc(132) or nc(133) or nr(175) or nc(221) or nr(263) );

  ncell(221) <= nc(221) and 
          not ( nc(132) or nc(133) or nc(134) );

  ncell(222) <= nc(222) and 
          not ( nc(133) or nc(134) or nc(135) or nc(221) or nc(223) );

  ncell(223) <= nc(223) and 
          not ( nc(134) or nc(135) or nc(136) );

  ncell(224) <= nc(224) and 
          not ( nc(135) or nc(136) or nc(137) or nc(223) or nc(225) );

  ncell(225) <= nc(225) and 
          not ( nc(136) or nc(137) or nc(138) );

  ncell(226) <= nc(226) and 
          not ( nc(137) or nc(138) or nc(139) or nc(225) or nc(227) );

  ncell(227) <= nc(227) and 
          not ( nc(138) or nc(139) or nc(140) );

  ncell(228) <= nc(228) and 
          not ( nc(139) or nc(140) or nc(141) or nc(227) or nc(229) );

  ncell(229) <= nc(229) and 
          not ( nc(140) or nc(141) or nc(142) );

  ncell(230) <= nc(230) and 
          not ( nc(141) or nc(142) or nc(143) or nc(229) or nc(231) );

  ncell(231) <= nc(231) and 
          not ( nc(142) or nc(143) or nc(144) );

  ncell(232) <= nc(232) and 
          not ( nc(143) or nc(144) or nc(145) or nc(231) or nc(233) );

  ncell(233) <= nc(233) and 
          not ( nc(144) or nc(145) or nc(146) );

  ncell(234) <= nc(234) and 
          not ( nc(145) or nc(146) or nc(147) or nc(233) or nc(235) );

  ncell(235) <= nc(235) and 
          not ( nc(146) or nc(147) or nc(148) );

  ncell(236) <= nc(236) and 
          not ( nc(147) or nc(148) or nc(149) or nc(235) or nc(237) );

  ncell(237) <= nc(237) and 
          not ( nc(148) or nc(149) or nc(150) );

  ncell(238) <= nc(238) and 
          not ( nc(149) or nc(150) or nc(151) or nc(237) or nc(239) );

  ncell(239) <= nc(239) and 
          not ( nc(150) or nc(151) or nc(152) );

  ncell(240) <= nc(240) and 
          not ( nc(151) or nc(152) or nc(153) or nc(239) or nc(241) );

  ncell(241) <= nc(241) and 
          not ( nc(152) or nc(153) or nc(154) );

  ncell(242) <= nc(242) and 
          not ( nc(153) or nc(154) or nc(155) or nc(241) or nc(243) );

  ncell(243) <= nc(243) and 
          not ( nc(154) or nc(155) or nc(156) );

  ncell(244) <= nc(244) and 
          not ( nc(155) or nc(156) or nc(157) or nc(243) or nc(245) );

  ncell(245) <= nc(245) and 
          not ( nc(156) or nc(157) or nc(158) );

  ncell(246) <= nc(246) and 
          not ( nc(157) or nc(158) or nc(159) or nc(245) or nc(247) );

  ncell(247) <= nc(247) and 
          not ( nc(158) or nc(159) or nc(160) );

  ncell(248) <= nc(248) and 
          not ( nc(159) or nc(160) or nc(161) or nc(247) or nc(249) );

  ncell(249) <= nc(249) and 
          not ( nc(160) or nc(161) or nc(162) );

  ncell(250) <= nc(250) and 
          not ( nc(161) or nc(162) or nc(163) or nc(249) or nc(251) );

  ncell(251) <= nc(251) and 
          not ( nc(162) or nc(163) or nc(164) );

  ncell(252) <= nc(252) and 
          not ( nc(163) or nc(164) or nc(165) or nc(251) or nc(253) );

  ncell(253) <= nc(253) and 
          not ( nc(164) or nc(165) or nc(166) );

  ncell(254) <= nc(254) and 
          not ( nc(165) or nc(166) or nc(167) or nc(253) or nc(255) );

  ncell(255) <= nc(255) and 
          not ( nc(166) or nc(167) or nc(168) );

  ncell(256) <= nc(256) and 
          not ( nc(167) or nc(168) or nc(169) or nc(255) or nc(257) );

  ncell(257) <= nc(257) and 
          not ( nc(168) or nc(169) or nc(170) );

  ncell(258) <= nc(258) and 
          not ( nc(169) or nc(170) or nc(171) or nc(257) or nc(259) );

  ncell(259) <= nc(259) and 
          not ( nc(170) or nc(171) or nc(172) );

  ncell(260) <= nc(260) and 
          not ( nc(171) or nc(172) or nc(173) or nc(259) or nc(261) );

  ncell(261) <= nc(261) and 
          not ( nc(172) or nc(173) or nc(174) );

  ncell(262) <= nc(262) and 
          not ( nc(173) or nc(174) or nc(175) or nc(261) or nc(263) );

  ncell(263) <= nc(263) and 
          not ( nl(132) or nc(174) or nc(175) );

  ncell(264) <= nc(264) and 
          not ( nc(176) or nc(177) or nr(219) or nc(265) or nr(307) );

  ncell(265) <= nc(265) and 
          not ( nc(176) or nc(177) or nc(178) );

  ncell(266) <= nc(266) and 
          not ( nc(177) or nc(178) or nc(179) or nc(265) or nc(267) );

  ncell(267) <= nc(267) and 
          not ( nc(178) or nc(179) or nc(180) );

  ncell(268) <= nc(268) and 
          not ( nc(179) or nc(180) or nc(181) or nc(267) or nc(269) );

  ncell(269) <= nc(269) and 
          not ( nc(180) or nc(181) or nc(182) );

  ncell(270) <= nc(270) and 
          not ( nc(181) or nc(182) or nc(183) or nc(269) or nc(271) );

  ncell(271) <= nc(271) and 
          not ( nc(182) or nc(183) or nc(184) );

  ncell(272) <= nc(272) and 
          not ( nc(183) or nc(184) or nc(185) or nc(271) or nc(273) );

  ncell(273) <= nc(273) and 
          not ( nc(184) or nc(185) or nc(186) );

  ncell(274) <= nc(274) and 
          not ( nc(185) or nc(186) or nc(187) or nc(273) or nc(275) );

  ncell(275) <= nc(275) and 
          not ( nc(186) or nc(187) or nc(188) );

  ncell(276) <= nc(276) and 
          not ( nc(187) or nc(188) or nc(189) or nc(275) or nc(277) );

  ncell(277) <= nc(277) and 
          not ( nc(188) or nc(189) or nc(190) );

  ncell(278) <= nc(278) and 
          not ( nc(189) or nc(190) or nc(191) or nc(277) or nc(279) );

  ncell(279) <= nc(279) and 
          not ( nc(190) or nc(191) or nc(192) );

  ncell(280) <= nc(280) and 
          not ( nc(191) or nc(192) or nc(193) or nc(279) or nc(281) );

  ncell(281) <= nc(281) and 
          not ( nc(192) or nc(193) or nc(194) );

  ncell(282) <= nc(282) and 
          not ( nc(193) or nc(194) or nc(195) or nc(281) or nc(283) );

  ncell(283) <= nc(283) and 
          not ( nc(194) or nc(195) or nc(196) );

  ncell(284) <= nc(284) and 
          not ( nc(195) or nc(196) or nc(197) or nc(283) or nc(285) );

  ncell(285) <= nc(285) and 
          not ( nc(196) or nc(197) or nc(198) );

  ncell(286) <= nc(286) and 
          not ( nc(197) or nc(198) or nc(199) or nc(285) or nc(287) );

  ncell(287) <= nc(287) and 
          not ( nc(198) or nc(199) or nc(200) );

  ncell(288) <= nc(288) and 
          not ( nc(199) or nc(200) or nc(201) or nc(287) or nc(289) );

  ncell(289) <= nc(289) and 
          not ( nc(200) or nc(201) or nc(202) );

  ncell(290) <= nc(290) and 
          not ( nc(201) or nc(202) or nc(203) or nc(289) or nc(291) );

  ncell(291) <= nc(291) and 
          not ( nc(202) or nc(203) or nc(204) );

  ncell(292) <= nc(292) and 
          not ( nc(203) or nc(204) or nc(205) or nc(291) or nc(293) );

  ncell(293) <= nc(293) and 
          not ( nc(204) or nc(205) or nc(206) );

  ncell(294) <= nc(294) and 
          not ( nc(205) or nc(206) or nc(207) or nc(293) or nc(295) );

  ncell(295) <= nc(295) and 
          not ( nc(206) or nc(207) or nc(208) );

  ncell(296) <= nc(296) and 
          not ( nc(207) or nc(208) or nc(209) or nc(295) or nc(297) );

  ncell(297) <= nc(297) and 
          not ( nc(208) or nc(209) or nc(210) );

  ncell(298) <= nc(298) and 
          not ( nc(209) or nc(210) or nc(211) or nc(297) or nc(299) );

  ncell(299) <= nc(299) and 
          not ( nc(210) or nc(211) or nc(212) );

  ncell(300) <= nc(300) and 
          not ( nc(211) or nc(212) or nc(213) or nc(299) or nc(301) );

  ncell(301) <= nc(301) and 
          not ( nc(212) or nc(213) or nc(214) );

  ncell(302) <= nc(302) and 
          not ( nc(213) or nc(214) or nc(215) or nc(301) or nc(303) );

  ncell(303) <= nc(303) and 
          not ( nc(214) or nc(215) or nc(216) );

  ncell(304) <= nc(304) and 
          not ( nc(215) or nc(216) or nc(217) or nc(303) or nc(305) );

  ncell(305) <= nc(305) and 
          not ( nc(216) or nc(217) or nc(218) );

  ncell(306) <= nc(306) and 
          not ( nc(217) or nc(218) or nc(219) or nc(305) or nc(307) );

  ncell(307) <= nc(307) and 
          not ( nl(176) or nc(218) or nc(219) );

  ncell(308) <= nc(308) and 
          not ( nc(220) or nc(221) or nr(263) or nc(309) or nr(351) );

  ncell(309) <= nc(309) and 
          not ( nc(220) or nc(221) or nc(222) );

  ncell(310) <= nc(310) and 
          not ( nc(221) or nc(222) or nc(223) or nc(309) or nc(311) );

  ncell(311) <= nc(311) and 
          not ( nc(222) or nc(223) or nc(224) );

  ncell(312) <= nc(312) and 
          not ( nc(223) or nc(224) or nc(225) or nc(311) or nc(313) );

  ncell(313) <= nc(313) and 
          not ( nc(224) or nc(225) or nc(226) );

  ncell(314) <= nc(314) and 
          not ( nc(225) or nc(226) or nc(227) or nc(313) or nc(315) );

  ncell(315) <= nc(315) and 
          not ( nc(226) or nc(227) or nc(228) );

  ncell(316) <= nc(316) and 
          not ( nc(227) or nc(228) or nc(229) or nc(315) or nc(317) );

  ncell(317) <= nc(317) and 
          not ( nc(228) or nc(229) or nc(230) );

  ncell(318) <= nc(318) and 
          not ( nc(229) or nc(230) or nc(231) or nc(317) or nc(319) );

  ncell(319) <= nc(319) and 
          not ( nc(230) or nc(231) or nc(232) );

  ncell(320) <= nc(320) and 
          not ( nc(231) or nc(232) or nc(233) or nc(319) or nc(321) );

  ncell(321) <= nc(321) and 
          not ( nc(232) or nc(233) or nc(234) );

  ncell(322) <= nc(322) and 
          not ( nc(233) or nc(234) or nc(235) or nc(321) or nc(323) );

  ncell(323) <= nc(323) and 
          not ( nc(234) or nc(235) or nc(236) );

  ncell(324) <= nc(324) and 
          not ( nc(235) or nc(236) or nc(237) or nc(323) or nc(325) );

  ncell(325) <= nc(325) and 
          not ( nc(236) or nc(237) or nc(238) );

  ncell(326) <= nc(326) and 
          not ( nc(237) or nc(238) or nc(239) or nc(325) or nc(327) );

  ncell(327) <= nc(327) and 
          not ( nc(238) or nc(239) or nc(240) );

  ncell(328) <= nc(328) and 
          not ( nc(239) or nc(240) or nc(241) or nc(327) or nc(329) );

  ncell(329) <= nc(329) and 
          not ( nc(240) or nc(241) or nc(242) );

  ncell(330) <= nc(330) and 
          not ( nc(241) or nc(242) or nc(243) or nc(329) or nc(331) );

  ncell(331) <= nc(331) and 
          not ( nc(242) or nc(243) or nc(244) );

  ncell(332) <= nc(332) and 
          not ( nc(243) or nc(244) or nc(245) or nc(331) or nc(333) );

  ncell(333) <= nc(333) and 
          not ( nc(244) or nc(245) or nc(246) );

  ncell(334) <= nc(334) and 
          not ( nc(245) or nc(246) or nc(247) or nc(333) or nc(335) );

  ncell(335) <= nc(335) and 
          not ( nc(246) or nc(247) or nc(248) );

  ncell(336) <= nc(336) and 
          not ( nc(247) or nc(248) or nc(249) or nc(335) or nc(337) );

  ncell(337) <= nc(337) and 
          not ( nc(248) or nc(249) or nc(250) );

  ncell(338) <= nc(338) and 
          not ( nc(249) or nc(250) or nc(251) or nc(337) or nc(339) );

  ncell(339) <= nc(339) and 
          not ( nc(250) or nc(251) or nc(252) );

  ncell(340) <= nc(340) and 
          not ( nc(251) or nc(252) or nc(253) or nc(339) or nc(341) );

  ncell(341) <= nc(341) and 
          not ( nc(252) or nc(253) or nc(254) );

  ncell(342) <= nc(342) and 
          not ( nc(253) or nc(254) or nc(255) or nc(341) or nc(343) );

  ncell(343) <= nc(343) and 
          not ( nc(254) or nc(255) or nc(256) );

  ncell(344) <= nc(344) and 
          not ( nc(255) or nc(256) or nc(257) or nc(343) or nc(345) );

  ncell(345) <= nc(345) and 
          not ( nc(256) or nc(257) or nc(258) );

  ncell(346) <= nc(346) and 
          not ( nc(257) or nc(258) or nc(259) or nc(345) or nc(347) );

  ncell(347) <= nc(347) and 
          not ( nc(258) or nc(259) or nc(260) );

  ncell(348) <= nc(348) and 
          not ( nc(259) or nc(260) or nc(261) or nc(347) or nc(349) );

  ncell(349) <= nc(349) and 
          not ( nc(260) or nc(261) or nc(262) );

  ncell(350) <= nc(350) and 
          not ( nc(261) or nc(262) or nc(263) or nc(349) or nc(351) );

  ncell(351) <= nc(351) and 
          not ( nl(220) or nc(262) or nc(263) );

  lcell(  0) <= lc(  0) and 
          not ( nc(264) or nc(265) or nr(307) or lc(  1) or lr( 43) );

  lcell(  1) <= lc(  1) and 
          not ( nc(264) or nc(265) or nc(266) );

  lcell(  2) <= lc(  2) and 
          not ( nc(265) or nc(266) or nc(267) or lc(  1) or lc(  3) );

  lcell(  3) <= lc(  3) and 
          not ( nc(266) or nc(267) or nc(268) );

  lcell(  4) <= lc(  4) and 
          not ( nc(267) or nc(268) or nc(269) or lc(  3) or lc(  5) );

  lcell(  5) <= lc(  5) and 
          not ( nc(268) or nc(269) or nc(270) );

  lcell(  6) <= lc(  6) and 
          not ( nc(269) or nc(270) or nc(271) or lc(  5) or lc(  7) );

  lcell(  7) <= lc(  7) and 
          not ( nc(270) or nc(271) or nc(272) );

  lcell(  8) <= lc(  8) and 
          not ( nc(271) or nc(272) or nc(273) or lc(  7) or lc(  9) );

  lcell(  9) <= lc(  9) and 
          not ( nc(272) or nc(273) or nc(274) );

  lcell( 10) <= lc( 10) and 
          not ( nc(273) or nc(274) or nc(275) or lc(  9) or lc( 11) );

  lcell( 11) <= lc( 11) and 
          not ( nc(274) or nc(275) or nc(276) );

  lcell( 12) <= lc( 12) and 
          not ( nc(275) or nc(276) or nc(277) or lc( 11) or lc( 13) );

  lcell( 13) <= lc( 13) and 
          not ( nc(276) or nc(277) or nc(278) );

  lcell( 14) <= lc( 14) and 
          not ( nc(277) or nc(278) or nc(279) or lc( 13) or lc( 15) );

  lcell( 15) <= lc( 15) and 
          not ( nc(278) or nc(279) or nc(280) );

  lcell( 16) <= lc( 16) and 
          not ( nc(279) or nc(280) or nc(281) or lc( 15) or lc( 17) );

  lcell( 17) <= lc( 17) and 
          not ( nc(280) or nc(281) or nc(282) );

  lcell( 18) <= lc( 18) and 
          not ( nc(281) or nc(282) or nc(283) or lc( 17) or lc( 19) );

  lcell( 19) <= lc( 19) and 
          not ( nc(282) or nc(283) or nc(284) );

  lcell( 20) <= lc( 20) and 
          not ( nc(283) or nc(284) or nc(285) or lc( 19) or lc( 21) );

  lcell( 21) <= lc( 21) and 
          not ( nc(284) or nc(285) or nc(286) );

  lcell( 22) <= lc( 22) and 
          not ( nc(285) or nc(286) or nc(287) or lc( 21) or lc( 23) );

  lcell( 23) <= lc( 23) and 
          not ( nc(286) or nc(287) or nc(288) );

  lcell( 24) <= lc( 24) and 
          not ( nc(287) or nc(288) or nc(289) or lc( 23) or lc( 25) );

  lcell( 25) <= lc( 25) and 
          not ( nc(288) or nc(289) or nc(290) );

  lcell( 26) <= lc( 26) and 
          not ( nc(289) or nc(290) or nc(291) or lc( 25) or lc( 27) );

  lcell( 27) <= lc( 27) and 
          not ( nc(290) or nc(291) or nc(292) );

  lcell( 28) <= lc( 28) and 
          not ( nc(291) or nc(292) or nc(293) or lc( 27) or lc( 29) );

  lcell( 29) <= lc( 29) and 
          not ( nc(292) or nc(293) or nc(294) );

  lcell( 30) <= lc( 30) and 
          not ( nc(293) or nc(294) or nc(295) or lc( 29) or lc( 31) );

  lcell( 31) <= lc( 31) and 
          not ( nc(294) or nc(295) or nc(296) );

  lcell( 32) <= lc( 32) and 
          not ( nc(295) or nc(296) or nc(297) or lc( 31) or lc( 33) );

  lcell( 33) <= lc( 33) and 
          not ( nc(296) or nc(297) or nc(298) );

  lcell( 34) <= lc( 34) and 
          not ( nc(297) or nc(298) or nc(299) or lc( 33) or lc( 35) );

  lcell( 35) <= lc( 35) and 
          not ( nc(298) or nc(299) or nc(300) );

  lcell( 36) <= lc( 36) and 
          not ( nc(299) or nc(300) or nc(301) or lc( 35) or lc( 37) );

  lcell( 37) <= lc( 37) and 
          not ( nc(300) or nc(301) or nc(302) );

  lcell( 38) <= lc( 38) and 
          not ( nc(301) or nc(302) or nc(303) or lc( 37) or lc( 39) );

  lcell( 39) <= lc( 39) and 
          not ( nc(302) or nc(303) or nc(304) );

  lcell( 40) <= lc( 40) and 
          not ( nc(303) or nc(304) or nc(305) or lc( 39) or lc( 41) );

  lcell( 41) <= lc( 41) and 
          not ( nc(304) or nc(305) or nc(306) );

  lcell( 42) <= lc( 42) and 
          not ( nc(305) or nc(306) or nc(307) or lc( 41) or lc( 43) );

  lcell( 43) <= lc( 43) and 
          not ( nl(264) or nc(306) or nc(307) );

  lcell( 44) <= lc( 44) and 
          not ( nc(308) or nc(309) or nr(351) or lc( 45) or lr( 87) );

  lcell( 45) <= lc( 45) and 
          not ( nc(308) or nc(309) or nc(310) );

  lcell( 46) <= lc( 46) and 
          not ( nc(309) or nc(310) or nc(311) or lc( 45) or lc( 47) );

  lcell( 47) <= lc( 47) and 
          not ( nc(310) or nc(311) or nc(312) );

  lcell( 48) <= lc( 48) and 
          not ( nc(311) or nc(312) or nc(313) or lc( 47) or lc( 49) );

  lcell( 49) <= lc( 49) and 
          not ( nc(312) or nc(313) or nc(314) );

  lcell( 50) <= lc( 50) and 
          not ( nc(313) or nc(314) or nc(315) or lc( 49) or lc( 51) );

  lcell( 51) <= lc( 51) and 
          not ( nc(314) or nc(315) or nc(316) );

  lcell( 52) <= lc( 52) and 
          not ( nc(315) or nc(316) or nc(317) or lc( 51) or lc( 53) );

  lcell( 53) <= lc( 53) and 
          not ( nc(316) or nc(317) or nc(318) );

  lcell( 54) <= lc( 54) and 
          not ( nc(317) or nc(318) or nc(319) or lc( 53) or lc( 55) );

  lcell( 55) <= lc( 55) and 
          not ( nc(318) or nc(319) or nc(320) );

  lcell( 56) <= lc( 56) and 
          not ( nc(319) or nc(320) or nc(321) or lc( 55) or lc( 57) );

  lcell( 57) <= lc( 57) and 
          not ( nc(320) or nc(321) or nc(322) );

  lcell( 58) <= lc( 58) and 
          not ( nc(321) or nc(322) or nc(323) or lc( 57) or lc( 59) );

  lcell( 59) <= lc( 59) and 
          not ( nc(322) or nc(323) or nc(324) );

  lcell( 60) <= lc( 60) and 
          not ( nc(323) or nc(324) or nc(325) or lc( 59) or lc( 61) );

  lcell( 61) <= lc( 61) and 
          not ( nc(324) or nc(325) or nc(326) );

  lcell( 62) <= lc( 62) and 
          not ( nc(325) or nc(326) or nc(327) or lc( 61) or lc( 63) );

  lcell( 63) <= lc( 63) and 
          not ( nc(326) or nc(327) or nc(328) );

  lcell( 64) <= lc( 64) and 
          not ( nc(327) or nc(328) or nc(329) or lc( 63) or lc( 65) );

  lcell( 65) <= lc( 65) and 
          not ( nc(328) or nc(329) or nc(330) );

  lcell( 66) <= lc( 66) and 
          not ( nc(329) or nc(330) or nc(331) or lc( 65) or lc( 67) );

  lcell( 67) <= lc( 67) and 
          not ( nc(330) or nc(331) or nc(332) );

  lcell( 68) <= lc( 68) and 
          not ( nc(331) or nc(332) or nc(333) or lc( 67) or lc( 69) );

  lcell( 69) <= lc( 69) and 
          not ( nc(332) or nc(333) or nc(334) );

  lcell( 70) <= lc( 70) and 
          not ( nc(333) or nc(334) or nc(335) or lc( 69) or lc( 71) );

  lcell( 71) <= lc( 71) and 
          not ( nc(334) or nc(335) or nc(336) );

  lcell( 72) <= lc( 72) and 
          not ( nc(335) or nc(336) or nc(337) or lc( 71) or lc( 73) );

  lcell( 73) <= lc( 73) and 
          not ( nc(336) or nc(337) or nc(338) );

  lcell( 74) <= lc( 74) and 
          not ( nc(337) or nc(338) or nc(339) or lc( 73) or lc( 75) );

  lcell( 75) <= lc( 75) and 
          not ( nc(338) or nc(339) or nc(340) );

  lcell( 76) <= lc( 76) and 
          not ( nc(339) or nc(340) or nc(341) or lc( 75) or lc( 77) );

  lcell( 77) <= lc( 77) and 
          not ( nc(340) or nc(341) or nc(342) );

  lcell( 78) <= lc( 78) and 
          not ( nc(341) or nc(342) or nc(343) or lc( 77) or lc( 79) );

  lcell( 79) <= lc( 79) and 
          not ( nc(342) or nc(343) or nc(344) );

  lcell( 80) <= lc( 80) and 
          not ( nc(343) or nc(344) or nc(345) or lc( 79) or lc( 81) );

  lcell( 81) <= lc( 81) and 
          not ( nc(344) or nc(345) or nc(346) );

  lcell( 82) <= lc( 82) and 
          not ( nc(345) or nc(346) or nc(347) or lc( 81) or lc( 83) );

  lcell( 83) <= lc( 83) and 
          not ( nc(346) or nc(347) or nc(348) );

  lcell( 84) <= lc( 84) and 
          not ( nc(347) or nc(348) or nc(349) or lc( 83) or lc( 85) );

  lcell( 85) <= lc( 85) and 
          not ( nc(348) or nc(349) or nc(350) );

  lcell( 86) <= lc( 86) and 
          not ( nc(349) or nc(350) or nc(351) or lc( 85) or lc( 87) );

  lcell( 87) <= lc( 87) and 
          not ( nl(308) or nc(350) or nc(351) );

  lcell( 88) <= lc( 88) and 
          not ( lc(  0) or lc(  1) or lr( 43) or lc( 89) or lr(131) );

  lcell( 89) <= lc( 89) and 
          not ( lc(  0) or lc(  1) or lc(  2) );

  lcell( 90) <= lc( 90) and 
          not ( lc(  1) or lc(  2) or lc(  3) or lc( 89) or lc( 91) );

  lcell( 91) <= lc( 91) and 
          not ( lc(  2) or lc(  3) or lc(  4) );

  lcell( 92) <= lc( 92) and 
          not ( lc(  3) or lc(  4) or lc(  5) or lc( 91) or lc( 93) );

  lcell( 93) <= lc( 93) and 
          not ( lc(  4) or lc(  5) or lc(  6) );

  lcell( 94) <= lc( 94) and 
          not ( lc(  5) or lc(  6) or lc(  7) or lc( 93) or lc( 95) );

  lcell( 95) <= lc( 95) and 
          not ( lc(  6) or lc(  7) or lc(  8) );

  lcell( 96) <= lc( 96) and 
          not ( lc(  7) or lc(  8) or lc(  9) or lc( 95) or lc( 97) );

  lcell( 97) <= lc( 97) and 
          not ( lc(  8) or lc(  9) or lc( 10) );

  lcell( 98) <= lc( 98) and 
          not ( lc(  9) or lc( 10) or lc( 11) or lc( 97) or lc( 99) );

  lcell( 99) <= lc( 99) and 
          not ( lc( 10) or lc( 11) or lc( 12) );

  lcell(100) <= lc(100) and 
          not ( lc( 11) or lc( 12) or lc( 13) or lc( 99) or lc(101) );

  lcell(101) <= lc(101) and 
          not ( lc( 12) or lc( 13) or lc( 14) );

  lcell(102) <= lc(102) and 
          not ( lc( 13) or lc( 14) or lc( 15) or lc(101) or lc(103) );

  lcell(103) <= lc(103) and 
          not ( lc( 14) or lc( 15) or lc( 16) );

  lcell(104) <= lc(104) and 
          not ( lc( 15) or lc( 16) or lc( 17) or lc(103) or lc(105) );

  lcell(105) <= lc(105) and 
          not ( lc( 16) or lc( 17) or lc( 18) );

  lcell(106) <= lc(106) and 
          not ( lc( 17) or lc( 18) or lc( 19) or lc(105) or lc(107) );

  lcell(107) <= lc(107) and 
          not ( lc( 18) or lc( 19) or lc( 20) );

  lcell(108) <= lc(108) and 
          not ( lc( 19) or lc( 20) or lc( 21) or lc(107) or lc(109) );

  lcell(109) <= lc(109) and 
          not ( lc( 20) or lc( 21) or lc( 22) );

  lcell(110) <= lc(110) and 
          not ( lc( 21) or lc( 22) or lc( 23) or lc(109) or lc(111) );

  lcell(111) <= lc(111) and 
          not ( lc( 22) or lc( 23) or lc( 24) );

  lcell(112) <= lc(112) and 
          not ( lc( 23) or lc( 24) or lc( 25) or lc(111) or lc(113) );

  lcell(113) <= lc(113) and 
          not ( lc( 24) or lc( 25) or lc( 26) );

  lcell(114) <= lc(114) and 
          not ( lc( 25) or lc( 26) or lc( 27) or lc(113) or lc(115) );

  lcell(115) <= lc(115) and 
          not ( lc( 26) or lc( 27) or lc( 28) );

  lcell(116) <= lc(116) and 
          not ( lc( 27) or lc( 28) or lc( 29) or lc(115) or lc(117) );

  lcell(117) <= lc(117) and 
          not ( lc( 28) or lc( 29) or lc( 30) );

  lcell(118) <= lc(118) and 
          not ( lc( 29) or lc( 30) or lc( 31) or lc(117) or lc(119) );

  lcell(119) <= lc(119) and 
          not ( lc( 30) or lc( 31) or lc( 32) );

  lcell(120) <= lc(120) and 
          not ( lc( 31) or lc( 32) or lc( 33) or lc(119) or lc(121) );

  lcell(121) <= lc(121) and 
          not ( lc( 32) or lc( 33) or lc( 34) );

  lcell(122) <= lc(122) and 
          not ( lc( 33) or lc( 34) or lc( 35) or lc(121) or lc(123) );

  lcell(123) <= lc(123) and 
          not ( lc( 34) or lc( 35) or lc( 36) );

  lcell(124) <= lc(124) and 
          not ( lc( 35) or lc( 36) or lc( 37) or lc(123) or lc(125) );

  lcell(125) <= lc(125) and 
          not ( lc( 36) or lc( 37) or lc( 38) );

  lcell(126) <= lc(126) and 
          not ( lc( 37) or lc( 38) or lc( 39) or lc(125) or lc(127) );

  lcell(127) <= lc(127) and 
          not ( lc( 38) or lc( 39) or lc( 40) );

  lcell(128) <= lc(128) and 
          not ( lc( 39) or lc( 40) or lc( 41) or lc(127) or lc(129) );

  lcell(129) <= lc(129) and 
          not ( lc( 40) or lc( 41) or lc( 42) );

  lcell(130) <= lc(130) and 
          not ( lc( 41) or lc( 42) or lc( 43) or lc(129) or lc(131) );

  lcell(131) <= lc(131) and 
          not ( ll(  0) or lc( 42) or lc( 43) );

  lcell(132) <= lc(132) and 
          not ( lc( 44) or lc( 45) or lr( 87) or lc(133) or lr(175) );

  lcell(133) <= lc(133) and 
          not ( lc( 44) or lc( 45) or lc( 46) );

  lcell(134) <= lc(134) and 
          not ( lc( 45) or lc( 46) or lc( 47) or lc(133) or lc(135) );

  lcell(135) <= lc(135) and 
          not ( lc( 46) or lc( 47) or lc( 48) );

  lcell(136) <= lc(136) and 
          not ( lc( 47) or lc( 48) or lc( 49) or lc(135) or lc(137) );

  lcell(137) <= lc(137) and 
          not ( lc( 48) or lc( 49) or lc( 50) );

  lcell(138) <= lc(138) and 
          not ( lc( 49) or lc( 50) or lc( 51) or lc(137) or lc(139) );

  lcell(139) <= lc(139) and 
          not ( lc( 50) or lc( 51) or lc( 52) );

  lcell(140) <= lc(140) and 
          not ( lc( 51) or lc( 52) or lc( 53) or lc(139) or lc(141) );

  lcell(141) <= lc(141) and 
          not ( lc( 52) or lc( 53) or lc( 54) );

  lcell(142) <= lc(142) and 
          not ( lc( 53) or lc( 54) or lc( 55) or lc(141) or lc(143) );

  lcell(143) <= lc(143) and 
          not ( lc( 54) or lc( 55) or lc( 56) );

  lcell(144) <= lc(144) and 
          not ( lc( 55) or lc( 56) or lc( 57) or lc(143) or lc(145) );

  lcell(145) <= lc(145) and 
          not ( lc( 56) or lc( 57) or lc( 58) );

  lcell(146) <= lc(146) and 
          not ( lc( 57) or lc( 58) or lc( 59) or lc(145) or lc(147) );

  lcell(147) <= lc(147) and 
          not ( lc( 58) or lc( 59) or lc( 60) );

  lcell(148) <= lc(148) and 
          not ( lc( 59) or lc( 60) or lc( 61) or lc(147) or lc(149) );

  lcell(149) <= lc(149) and 
          not ( lc( 60) or lc( 61) or lc( 62) );

  lcell(150) <= lc(150) and 
          not ( lc( 61) or lc( 62) or lc( 63) or lc(149) or lc(151) );

  lcell(151) <= lc(151) and 
          not ( lc( 62) or lc( 63) or lc( 64) );

  lcell(152) <= lc(152) and 
          not ( lc( 63) or lc( 64) or lc( 65) or lc(151) or lc(153) );

  lcell(153) <= lc(153) and 
          not ( lc( 64) or lc( 65) or lc( 66) );

  lcell(154) <= lc(154) and 
          not ( lc( 65) or lc( 66) or lc( 67) or lc(153) or lc(155) );

  lcell(155) <= lc(155) and 
          not ( lc( 66) or lc( 67) or lc( 68) );

  lcell(156) <= lc(156) and 
          not ( lc( 67) or lc( 68) or lc( 69) or lc(155) or lc(157) );

  lcell(157) <= lc(157) and 
          not ( lc( 68) or lc( 69) or lc( 70) );

  lcell(158) <= lc(158) and 
          not ( lc( 69) or lc( 70) or lc( 71) or lc(157) or lc(159) );

  lcell(159) <= lc(159) and 
          not ( lc( 70) or lc( 71) or lc( 72) );

  lcell(160) <= lc(160) and 
          not ( lc( 71) or lc( 72) or lc( 73) or lc(159) or lc(161) );

  lcell(161) <= lc(161) and 
          not ( lc( 72) or lc( 73) or lc( 74) );

  lcell(162) <= lc(162) and 
          not ( lc( 73) or lc( 74) or lc( 75) or lc(161) or lc(163) );

  lcell(163) <= lc(163) and 
          not ( lc( 74) or lc( 75) or lc( 76) );

  lcell(164) <= lc(164) and 
          not ( lc( 75) or lc( 76) or lc( 77) or lc(163) or lc(165) );

  lcell(165) <= lc(165) and 
          not ( lc( 76) or lc( 77) or lc( 78) );

  lcell(166) <= lc(166) and 
          not ( lc( 77) or lc( 78) or lc( 79) or lc(165) or lc(167) );

  lcell(167) <= lc(167) and 
          not ( lc( 78) or lc( 79) or lc( 80) );

  lcell(168) <= lc(168) and 
          not ( lc( 79) or lc( 80) or lc( 81) or lc(167) or lc(169) );

  lcell(169) <= lc(169) and 
          not ( lc( 80) or lc( 81) or lc( 82) );

  lcell(170) <= lc(170) and 
          not ( lc( 81) or lc( 82) or lc( 83) or lc(169) or lc(171) );

  lcell(171) <= lc(171) and 
          not ( lc( 82) or lc( 83) or lc( 84) );

  lcell(172) <= lc(172) and 
          not ( lc( 83) or lc( 84) or lc( 85) or lc(171) or lc(173) );

  lcell(173) <= lc(173) and 
          not ( lc( 84) or lc( 85) or lc( 86) );

  lcell(174) <= lc(174) and 
          not ( lc( 85) or lc( 86) or lc( 87) or lc(173) or lc(175) );

  lcell(175) <= lc(175) and 
          not ( ll( 44) or lc( 86) or lc( 87) );

  lcell(176) <= lc(176) and 
          not ( lc( 88) or lc( 89) or lr(131) or lc(177) or lr(219) );

  lcell(177) <= lc(177) and 
          not ( lc( 88) or lc( 89) or lc( 90) );

  lcell(178) <= lc(178) and 
          not ( lc( 89) or lc( 90) or lc( 91) or lc(177) or lc(179) );

  lcell(179) <= lc(179) and 
          not ( lc( 90) or lc( 91) or lc( 92) );

  lcell(180) <= lc(180) and 
          not ( lc( 91) or lc( 92) or lc( 93) or lc(179) or lc(181) );

  lcell(181) <= lc(181) and 
          not ( lc( 92) or lc( 93) or lc( 94) );

  lcell(182) <= lc(182) and 
          not ( lc( 93) or lc( 94) or lc( 95) or lc(181) or lc(183) );

  lcell(183) <= lc(183) and 
          not ( lc( 94) or lc( 95) or lc( 96) );

  lcell(184) <= lc(184) and 
          not ( lc( 95) or lc( 96) or lc( 97) or lc(183) or lc(185) );

  lcell(185) <= lc(185) and 
          not ( lc( 96) or lc( 97) or lc( 98) );

  lcell(186) <= lc(186) and 
          not ( lc( 97) or lc( 98) or lc( 99) or lc(185) or lc(187) );

  lcell(187) <= lc(187) and 
          not ( lc( 98) or lc( 99) or lc(100) );

  lcell(188) <= lc(188) and 
          not ( lc( 99) or lc(100) or lc(101) or lc(187) or lc(189) );

  lcell(189) <= lc(189) and 
          not ( lc(100) or lc(101) or lc(102) );

  lcell(190) <= lc(190) and 
          not ( lc(101) or lc(102) or lc(103) or lc(189) or lc(191) );

  lcell(191) <= lc(191) and 
          not ( lc(102) or lc(103) or lc(104) );

  lcell(192) <= lc(192) and 
          not ( lc(103) or lc(104) or lc(105) or lc(191) or lc(193) );

  lcell(193) <= lc(193) and 
          not ( lc(104) or lc(105) or lc(106) );

  lcell(194) <= lc(194) and 
          not ( lc(105) or lc(106) or lc(107) or lc(193) or lc(195) );

  lcell(195) <= lc(195) and 
          not ( lc(106) or lc(107) or lc(108) );

  lcell(196) <= lc(196) and 
          not ( lc(107) or lc(108) or lc(109) or lc(195) or lc(197) );

  lcell(197) <= lc(197) and 
          not ( lc(108) or lc(109) or lc(110) );

  lcell(198) <= lc(198) and 
          not ( lc(109) or lc(110) or lc(111) or lc(197) or lc(199) );

  lcell(199) <= lc(199) and 
          not ( lc(110) or lc(111) or lc(112) );

  lcell(200) <= lc(200) and 
          not ( lc(111) or lc(112) or lc(113) or lc(199) or lc(201) );

  lcell(201) <= lc(201) and 
          not ( lc(112) or lc(113) or lc(114) );

  lcell(202) <= lc(202) and 
          not ( lc(113) or lc(114) or lc(115) or lc(201) or lc(203) );

  lcell(203) <= lc(203) and 
          not ( lc(114) or lc(115) or lc(116) );

  lcell(204) <= lc(204) and 
          not ( lc(115) or lc(116) or lc(117) or lc(203) or lc(205) );

  lcell(205) <= lc(205) and 
          not ( lc(116) or lc(117) or lc(118) );

  lcell(206) <= lc(206) and 
          not ( lc(117) or lc(118) or lc(119) or lc(205) or lc(207) );

  lcell(207) <= lc(207) and 
          not ( lc(118) or lc(119) or lc(120) );

  lcell(208) <= lc(208) and 
          not ( lc(119) or lc(120) or lc(121) or lc(207) or lc(209) );

  lcell(209) <= lc(209) and 
          not ( lc(120) or lc(121) or lc(122) );

  lcell(210) <= lc(210) and 
          not ( lc(121) or lc(122) or lc(123) or lc(209) or lc(211) );

  lcell(211) <= lc(211) and 
          not ( lc(122) or lc(123) or lc(124) );

  lcell(212) <= lc(212) and 
          not ( lc(123) or lc(124) or lc(125) or lc(211) or lc(213) );

  lcell(213) <= lc(213) and 
          not ( lc(124) or lc(125) or lc(126) );

  lcell(214) <= lc(214) and 
          not ( lc(125) or lc(126) or lc(127) or lc(213) or lc(215) );

  lcell(215) <= lc(215) and 
          not ( lc(126) or lc(127) or lc(128) );

  lcell(216) <= lc(216) and 
          not ( lc(127) or lc(128) or lc(129) or lc(215) or lc(217) );

  lcell(217) <= lc(217) and 
          not ( lc(128) or lc(129) or lc(130) );

  lcell(218) <= lc(218) and 
          not ( lc(129) or lc(130) or lc(131) or lc(217) or lc(219) );

  lcell(219) <= lc(219) and 
          not ( ll( 88) or lc(130) or lc(131) );

  lcell(220) <= lc(220) and 
          not ( lc(132) or lc(133) or lr(175) or lc(221) or lr(263) );

  lcell(221) <= lc(221) and 
          not ( lc(132) or lc(133) or lc(134) );

  lcell(222) <= lc(222) and 
          not ( lc(133) or lc(134) or lc(135) or lc(221) or lc(223) );

  lcell(223) <= lc(223) and 
          not ( lc(134) or lc(135) or lc(136) );

  lcell(224) <= lc(224) and 
          not ( lc(135) or lc(136) or lc(137) or lc(223) or lc(225) );

  lcell(225) <= lc(225) and 
          not ( lc(136) or lc(137) or lc(138) );

  lcell(226) <= lc(226) and 
          not ( lc(137) or lc(138) or lc(139) or lc(225) or lc(227) );

  lcell(227) <= lc(227) and 
          not ( lc(138) or lc(139) or lc(140) );

  lcell(228) <= lc(228) and 
          not ( lc(139) or lc(140) or lc(141) or lc(227) or lc(229) );

  lcell(229) <= lc(229) and 
          not ( lc(140) or lc(141) or lc(142) );

  lcell(230) <= lc(230) and 
          not ( lc(141) or lc(142) or lc(143) or lc(229) or lc(231) );

  lcell(231) <= lc(231) and 
          not ( lc(142) or lc(143) or lc(144) );

  lcell(232) <= lc(232) and 
          not ( lc(143) or lc(144) or lc(145) or lc(231) or lc(233) );

  lcell(233) <= lc(233) and 
          not ( lc(144) or lc(145) or lc(146) );

  lcell(234) <= lc(234) and 
          not ( lc(145) or lc(146) or lc(147) or lc(233) or lc(235) );

  lcell(235) <= lc(235) and 
          not ( lc(146) or lc(147) or lc(148) );

  lcell(236) <= lc(236) and 
          not ( lc(147) or lc(148) or lc(149) or lc(235) or lc(237) );

  lcell(237) <= lc(237) and 
          not ( lc(148) or lc(149) or lc(150) );

  lcell(238) <= lc(238) and 
          not ( lc(149) or lc(150) or lc(151) or lc(237) or lc(239) );

  lcell(239) <= lc(239) and 
          not ( lc(150) or lc(151) or lc(152) );

  lcell(240) <= lc(240) and 
          not ( lc(151) or lc(152) or lc(153) or lc(239) or lc(241) );

  lcell(241) <= lc(241) and 
          not ( lc(152) or lc(153) or lc(154) );

  lcell(242) <= lc(242) and 
          not ( lc(153) or lc(154) or lc(155) or lc(241) or lc(243) );

  lcell(243) <= lc(243) and 
          not ( lc(154) or lc(155) or lc(156) );

  lcell(244) <= lc(244) and 
          not ( lc(155) or lc(156) or lc(157) or lc(243) or lc(245) );

  lcell(245) <= lc(245) and 
          not ( lc(156) or lc(157) or lc(158) );

  lcell(246) <= lc(246) and 
          not ( lc(157) or lc(158) or lc(159) or lc(245) or lc(247) );

  lcell(247) <= lc(247) and 
          not ( lc(158) or lc(159) or lc(160) );

  lcell(248) <= lc(248) and 
          not ( lc(159) or lc(160) or lc(161) or lc(247) or lc(249) );

  lcell(249) <= lc(249) and 
          not ( lc(160) or lc(161) or lc(162) );

  lcell(250) <= lc(250) and 
          not ( lc(161) or lc(162) or lc(163) or lc(249) or lc(251) );

  lcell(251) <= lc(251) and 
          not ( lc(162) or lc(163) or lc(164) );

  lcell(252) <= lc(252) and 
          not ( lc(163) or lc(164) or lc(165) or lc(251) or lc(253) );

  lcell(253) <= lc(253) and 
          not ( lc(164) or lc(165) or lc(166) );

  lcell(254) <= lc(254) and 
          not ( lc(165) or lc(166) or lc(167) or lc(253) or lc(255) );

  lcell(255) <= lc(255) and 
          not ( lc(166) or lc(167) or lc(168) );

  lcell(256) <= lc(256) and 
          not ( lc(167) or lc(168) or lc(169) or lc(255) or lc(257) );

  lcell(257) <= lc(257) and 
          not ( lc(168) or lc(169) or lc(170) );

  lcell(258) <= lc(258) and 
          not ( lc(169) or lc(170) or lc(171) or lc(257) or lc(259) );

  lcell(259) <= lc(259) and 
          not ( lc(170) or lc(171) or lc(172) );

  lcell(260) <= lc(260) and 
          not ( lc(171) or lc(172) or lc(173) or lc(259) or lc(261) );

  lcell(261) <= lc(261) and 
          not ( lc(172) or lc(173) or lc(174) );

  lcell(262) <= lc(262) and 
          not ( lc(173) or lc(174) or lc(175) or lc(261) or lc(263) );

  lcell(263) <= lc(263) and 
          not ( ll(132) or lc(174) or lc(175) );

  lcell(264) <= lc(264) and 
          not ( lc(176) or lc(177) or lr(219) or lc(265) or lr(307) );

  lcell(265) <= lc(265) and 
          not ( lc(176) or lc(177) or lc(178) );

  lcell(266) <= lc(266) and 
          not ( lc(177) or lc(178) or lc(179) or lc(265) or lc(267) );

  lcell(267) <= lc(267) and 
          not ( lc(178) or lc(179) or lc(180) );

  lcell(268) <= lc(268) and 
          not ( lc(179) or lc(180) or lc(181) or lc(267) or lc(269) );

  lcell(269) <= lc(269) and 
          not ( lc(180) or lc(181) or lc(182) );

  lcell(270) <= lc(270) and 
          not ( lc(181) or lc(182) or lc(183) or lc(269) or lc(271) );

  lcell(271) <= lc(271) and 
          not ( lc(182) or lc(183) or lc(184) );

  lcell(272) <= lc(272) and 
          not ( lc(183) or lc(184) or lc(185) or lc(271) or lc(273) );

  lcell(273) <= lc(273) and 
          not ( lc(184) or lc(185) or lc(186) );

  lcell(274) <= lc(274) and 
          not ( lc(185) or lc(186) or lc(187) or lc(273) or lc(275) );

  lcell(275) <= lc(275) and 
          not ( lc(186) or lc(187) or lc(188) );

  lcell(276) <= lc(276) and 
          not ( lc(187) or lc(188) or lc(189) or lc(275) or lc(277) );

  lcell(277) <= lc(277) and 
          not ( lc(188) or lc(189) or lc(190) );

  lcell(278) <= lc(278) and 
          not ( lc(189) or lc(190) or lc(191) or lc(277) or lc(279) );

  lcell(279) <= lc(279) and 
          not ( lc(190) or lc(191) or lc(192) );

  lcell(280) <= lc(280) and 
          not ( lc(191) or lc(192) or lc(193) or lc(279) or lc(281) );

  lcell(281) <= lc(281) and 
          not ( lc(192) or lc(193) or lc(194) );

  lcell(282) <= lc(282) and 
          not ( lc(193) or lc(194) or lc(195) or lc(281) or lc(283) );

  lcell(283) <= lc(283) and 
          not ( lc(194) or lc(195) or lc(196) );

  lcell(284) <= lc(284) and 
          not ( lc(195) or lc(196) or lc(197) or lc(283) or lc(285) );

  lcell(285) <= lc(285) and 
          not ( lc(196) or lc(197) or lc(198) );

  lcell(286) <= lc(286) and 
          not ( lc(197) or lc(198) or lc(199) or lc(285) or lc(287) );

  lcell(287) <= lc(287) and 
          not ( lc(198) or lc(199) or lc(200) );

  lcell(288) <= lc(288) and 
          not ( lc(199) or lc(200) or lc(201) or lc(287) or lc(289) );

  lcell(289) <= lc(289) and 
          not ( lc(200) or lc(201) or lc(202) );

  lcell(290) <= lc(290) and 
          not ( lc(201) or lc(202) or lc(203) or lc(289) or lc(291) );

  lcell(291) <= lc(291) and 
          not ( lc(202) or lc(203) or lc(204) );

  lcell(292) <= lc(292) and 
          not ( lc(203) or lc(204) or lc(205) or lc(291) or lc(293) );

  lcell(293) <= lc(293) and 
          not ( lc(204) or lc(205) or lc(206) );

  lcell(294) <= lc(294) and 
          not ( lc(205) or lc(206) or lc(207) or lc(293) or lc(295) );

  lcell(295) <= lc(295) and 
          not ( lc(206) or lc(207) or lc(208) );

  lcell(296) <= lc(296) and 
          not ( lc(207) or lc(208) or lc(209) or lc(295) or lc(297) );

  lcell(297) <= lc(297) and 
          not ( lc(208) or lc(209) or lc(210) );

  lcell(298) <= lc(298) and 
          not ( lc(209) or lc(210) or lc(211) or lc(297) or lc(299) );

  lcell(299) <= lc(299) and 
          not ( lc(210) or lc(211) or lc(212) );

  lcell(300) <= lc(300) and 
          not ( lc(211) or lc(212) or lc(213) or lc(299) or lc(301) );

  lcell(301) <= lc(301) and 
          not ( lc(212) or lc(213) or lc(214) );

  lcell(302) <= lc(302) and 
          not ( lc(213) or lc(214) or lc(215) or lc(301) or lc(303) );

  lcell(303) <= lc(303) and 
          not ( lc(214) or lc(215) or lc(216) );

  lcell(304) <= lc(304) and 
          not ( lc(215) or lc(216) or lc(217) or lc(303) or lc(305) );

  lcell(305) <= lc(305) and 
          not ( lc(216) or lc(217) or lc(218) );

  lcell(306) <= lc(306) and 
          not ( lc(217) or lc(218) or lc(219) or lc(305) or lc(307) );

  lcell(307) <= lc(307) and 
          not ( ll(176) or lc(218) or lc(219) );

  lcell(308) <= lc(308) and 
          not ( lc(220) or lc(221) or lr(263) or lc(309) or lr(351) );

  lcell(309) <= lc(309) and 
          not ( lc(220) or lc(221) or lc(222) );

  lcell(310) <= lc(310) and 
          not ( lc(221) or lc(222) or lc(223) or lc(309) or lc(311) );

  lcell(311) <= lc(311) and 
          not ( lc(222) or lc(223) or lc(224) );

  lcell(312) <= lc(312) and 
          not ( lc(223) or lc(224) or lc(225) or lc(311) or lc(313) );

  lcell(313) <= lc(313) and 
          not ( lc(224) or lc(225) or lc(226) );

  lcell(314) <= lc(314) and 
          not ( lc(225) or lc(226) or lc(227) or lc(313) or lc(315) );

  lcell(315) <= lc(315) and 
          not ( lc(226) or lc(227) or lc(228) );

  lcell(316) <= lc(316) and 
          not ( lc(227) or lc(228) or lc(229) or lc(315) or lc(317) );

  lcell(317) <= lc(317) and 
          not ( lc(228) or lc(229) or lc(230) );

  lcell(318) <= lc(318) and 
          not ( lc(229) or lc(230) or lc(231) or lc(317) or lc(319) );

  lcell(319) <= lc(319) and 
          not ( lc(230) or lc(231) or lc(232) );

  lcell(320) <= lc(320) and 
          not ( lc(231) or lc(232) or lc(233) or lc(319) or lc(321) );

  lcell(321) <= lc(321) and 
          not ( lc(232) or lc(233) or lc(234) );

  lcell(322) <= lc(322) and 
          not ( lc(233) or lc(234) or lc(235) or lc(321) or lc(323) );

  lcell(323) <= lc(323) and 
          not ( lc(234) or lc(235) or lc(236) );

  lcell(324) <= lc(324) and 
          not ( lc(235) or lc(236) or lc(237) or lc(323) or lc(325) );

  lcell(325) <= lc(325) and 
          not ( lc(236) or lc(237) or lc(238) );

  lcell(326) <= lc(326) and 
          not ( lc(237) or lc(238) or lc(239) or lc(325) or lc(327) );

  lcell(327) <= lc(327) and 
          not ( lc(238) or lc(239) or lc(240) );

  lcell(328) <= lc(328) and 
          not ( lc(239) or lc(240) or lc(241) or lc(327) or lc(329) );

  lcell(329) <= lc(329) and 
          not ( lc(240) or lc(241) or lc(242) );

  lcell(330) <= lc(330) and 
          not ( lc(241) or lc(242) or lc(243) or lc(329) or lc(331) );

  lcell(331) <= lc(331) and 
          not ( lc(242) or lc(243) or lc(244) );

  lcell(332) <= lc(332) and 
          not ( lc(243) or lc(244) or lc(245) or lc(331) or lc(333) );

  lcell(333) <= lc(333) and 
          not ( lc(244) or lc(245) or lc(246) );

  lcell(334) <= lc(334) and 
          not ( lc(245) or lc(246) or lc(247) or lc(333) or lc(335) );

  lcell(335) <= lc(335) and 
          not ( lc(246) or lc(247) or lc(248) );

  lcell(336) <= lc(336) and 
          not ( lc(247) or lc(248) or lc(249) or lc(335) or lc(337) );

  lcell(337) <= lc(337) and 
          not ( lc(248) or lc(249) or lc(250) );

  lcell(338) <= lc(338) and 
          not ( lc(249) or lc(250) or lc(251) or lc(337) or lc(339) );

  lcell(339) <= lc(339) and 
          not ( lc(250) or lc(251) or lc(252) );

  lcell(340) <= lc(340) and 
          not ( lc(251) or lc(252) or lc(253) or lc(339) or lc(341) );

  lcell(341) <= lc(341) and 
          not ( lc(252) or lc(253) or lc(254) );

  lcell(342) <= lc(342) and 
          not ( lc(253) or lc(254) or lc(255) or lc(341) or lc(343) );

  lcell(343) <= lc(343) and 
          not ( lc(254) or lc(255) or lc(256) );

  lcell(344) <= lc(344) and 
          not ( lc(255) or lc(256) or lc(257) or lc(343) or lc(345) );

  lcell(345) <= lc(345) and 
          not ( lc(256) or lc(257) or lc(258) );

  lcell(346) <= lc(346) and 
          not ( lc(257) or lc(258) or lc(259) or lc(345) or lc(347) );

  lcell(347) <= lc(347) and 
          not ( lc(258) or lc(259) or lc(260) );

  lcell(348) <= lc(348) and 
          not ( lc(259) or lc(260) or lc(261) or lc(347) or lc(349) );

  lcell(349) <= lc(349) and 
          not ( lc(260) or lc(261) or lc(262) );

  lcell(350) <= lc(350) and 
          not ( lc(261) or lc(262) or lc(263) or lc(349) or lc(351) );

  lcell(351) <= lc(351) and 
          not ( ll(220) or lc(262) or lc(263) );

  lcell(352) <= lc(352) and 
          not ( lc(264) or lc(265) or lr(307) or lc(353) or lr(395) );

  lcell(353) <= lc(353) and 
          not ( lc(264) or lc(265) or lc(266) );

  lcell(354) <= lc(354) and 
          not ( lc(265) or lc(266) or lc(267) or lc(353) or lc(355) );

  lcell(355) <= lc(355) and 
          not ( lc(266) or lc(267) or lc(268) );

  lcell(356) <= lc(356) and 
          not ( lc(267) or lc(268) or lc(269) or lc(355) or lc(357) );

  lcell(357) <= lc(357) and 
          not ( lc(268) or lc(269) or lc(270) );

  lcell(358) <= lc(358) and 
          not ( lc(269) or lc(270) or lc(271) or lc(357) or lc(359) );

  lcell(359) <= lc(359) and 
          not ( lc(270) or lc(271) or lc(272) );

  lcell(360) <= lc(360) and 
          not ( lc(271) or lc(272) or lc(273) or lc(359) or lc(361) );

  lcell(361) <= lc(361) and 
          not ( lc(272) or lc(273) or lc(274) );

  lcell(362) <= lc(362) and 
          not ( lc(273) or lc(274) or lc(275) or lc(361) or lc(363) );

  lcell(363) <= lc(363) and 
          not ( lc(274) or lc(275) or lc(276) );

  lcell(364) <= lc(364) and 
          not ( lc(275) or lc(276) or lc(277) or lc(363) or lc(365) );

  lcell(365) <= lc(365) and 
          not ( lc(276) or lc(277) or lc(278) );

  lcell(366) <= lc(366) and 
          not ( lc(277) or lc(278) or lc(279) or lc(365) or lc(367) );

  lcell(367) <= lc(367) and 
          not ( lc(278) or lc(279) or lc(280) );

  lcell(368) <= lc(368) and 
          not ( lc(279) or lc(280) or lc(281) or lc(367) or lc(369) );

  lcell(369) <= lc(369) and 
          not ( lc(280) or lc(281) or lc(282) );

  lcell(370) <= lc(370) and 
          not ( lc(281) or lc(282) or lc(283) or lc(369) or lc(371) );

  lcell(371) <= lc(371) and 
          not ( lc(282) or lc(283) or lc(284) );

  lcell(372) <= lc(372) and 
          not ( lc(283) or lc(284) or lc(285) or lc(371) or lc(373) );

  lcell(373) <= lc(373) and 
          not ( lc(284) or lc(285) or lc(286) );

  lcell(374) <= lc(374) and 
          not ( lc(285) or lc(286) or lc(287) or lc(373) or lc(375) );

  lcell(375) <= lc(375) and 
          not ( lc(286) or lc(287) or lc(288) );

  lcell(376) <= lc(376) and 
          not ( lc(287) or lc(288) or lc(289) or lc(375) or lc(377) );

  lcell(377) <= lc(377) and 
          not ( lc(288) or lc(289) or lc(290) );

  lcell(378) <= lc(378) and 
          not ( lc(289) or lc(290) or lc(291) or lc(377) or lc(379) );

  lcell(379) <= lc(379) and 
          not ( lc(290) or lc(291) or lc(292) );

  lcell(380) <= lc(380) and 
          not ( lc(291) or lc(292) or lc(293) or lc(379) or lc(381) );

  lcell(381) <= lc(381) and 
          not ( lc(292) or lc(293) or lc(294) );

  lcell(382) <= lc(382) and 
          not ( lc(293) or lc(294) or lc(295) or lc(381) or lc(383) );

  lcell(383) <= lc(383) and 
          not ( lc(294) or lc(295) or lc(296) );

  lcell(384) <= lc(384) and 
          not ( lc(295) or lc(296) or lc(297) or lc(383) or lc(385) );

  lcell(385) <= lc(385) and 
          not ( lc(296) or lc(297) or lc(298) );

  lcell(386) <= lc(386) and 
          not ( lc(297) or lc(298) or lc(299) or lc(385) or lc(387) );

  lcell(387) <= lc(387) and 
          not ( lc(298) or lc(299) or lc(300) );

  lcell(388) <= lc(388) and 
          not ( lc(299) or lc(300) or lc(301) or lc(387) or lc(389) );

  lcell(389) <= lc(389) and 
          not ( lc(300) or lc(301) or lc(302) );

  lcell(390) <= lc(390) and 
          not ( lc(301) or lc(302) or lc(303) or lc(389) or lc(391) );

  lcell(391) <= lc(391) and 
          not ( lc(302) or lc(303) or lc(304) );

  lcell(392) <= lc(392) and 
          not ( lc(303) or lc(304) or lc(305) or lc(391) or lc(393) );

  lcell(393) <= lc(393) and 
          not ( lc(304) or lc(305) or lc(306) );

  lcell(394) <= lc(394) and 
          not ( lc(305) or lc(306) or lc(307) or lc(393) or lc(395) );

  lcell(395) <= lc(395) and 
          not ( ll(264) or lc(306) or lc(307) );

  lcell(396) <= lc(396) and 
          not ( lc(308) or lc(309) or lr(351) or lc(397) or lr(439) );

  lcell(397) <= lc(397) and 
          not ( lc(308) or lc(309) or lc(310) );

  lcell(398) <= lc(398) and 
          not ( lc(309) or lc(310) or lc(311) or lc(397) or lc(399) );

  lcell(399) <= lc(399) and 
          not ( lc(310) or lc(311) or lc(312) );

  lcell(400) <= lc(400) and 
          not ( lc(311) or lc(312) or lc(313) or lc(399) or lc(401) );

  lcell(401) <= lc(401) and 
          not ( lc(312) or lc(313) or lc(314) );

  lcell(402) <= lc(402) and 
          not ( lc(313) or lc(314) or lc(315) or lc(401) or lc(403) );

  lcell(403) <= lc(403) and 
          not ( lc(314) or lc(315) or lc(316) );

  lcell(404) <= lc(404) and 
          not ( lc(315) or lc(316) or lc(317) or lc(403) or lc(405) );

  lcell(405) <= lc(405) and 
          not ( lc(316) or lc(317) or lc(318) );

  lcell(406) <= lc(406) and 
          not ( lc(317) or lc(318) or lc(319) or lc(405) or lc(407) );

  lcell(407) <= lc(407) and 
          not ( lc(318) or lc(319) or lc(320) );

  lcell(408) <= lc(408) and 
          not ( lc(319) or lc(320) or lc(321) or lc(407) or lc(409) );

  lcell(409) <= lc(409) and 
          not ( lc(320) or lc(321) or lc(322) );

  lcell(410) <= lc(410) and 
          not ( lc(321) or lc(322) or lc(323) or lc(409) or lc(411) );

  lcell(411) <= lc(411) and 
          not ( lc(322) or lc(323) or lc(324) );

  lcell(412) <= lc(412) and 
          not ( lc(323) or lc(324) or lc(325) or lc(411) or lc(413) );

  lcell(413) <= lc(413) and 
          not ( lc(324) or lc(325) or lc(326) );

  lcell(414) <= lc(414) and 
          not ( lc(325) or lc(326) or lc(327) or lc(413) or lc(415) );

  lcell(415) <= lc(415) and 
          not ( lc(326) or lc(327) or lc(328) );

  lcell(416) <= lc(416) and 
          not ( lc(327) or lc(328) or lc(329) or lc(415) or lc(417) );

  lcell(417) <= lc(417) and 
          not ( lc(328) or lc(329) or lc(330) );

  lcell(418) <= lc(418) and 
          not ( lc(329) or lc(330) or lc(331) or lc(417) or lc(419) );

  lcell(419) <= lc(419) and 
          not ( lc(330) or lc(331) or lc(332) );

  lcell(420) <= lc(420) and 
          not ( lc(331) or lc(332) or lc(333) or lc(419) or lc(421) );

  lcell(421) <= lc(421) and 
          not ( lc(332) or lc(333) or lc(334) );

  lcell(422) <= lc(422) and 
          not ( lc(333) or lc(334) or lc(335) or lc(421) or lc(423) );

  lcell(423) <= lc(423) and 
          not ( lc(334) or lc(335) or lc(336) );

  lcell(424) <= lc(424) and 
          not ( lc(335) or lc(336) or lc(337) or lc(423) or lc(425) );

  lcell(425) <= lc(425) and 
          not ( lc(336) or lc(337) or lc(338) );

  lcell(426) <= lc(426) and 
          not ( lc(337) or lc(338) or lc(339) or lc(425) or lc(427) );

  lcell(427) <= lc(427) and 
          not ( lc(338) or lc(339) or lc(340) );

  lcell(428) <= lc(428) and 
          not ( lc(339) or lc(340) or lc(341) or lc(427) or lc(429) );

  lcell(429) <= lc(429) and 
          not ( lc(340) or lc(341) or lc(342) );

  lcell(430) <= lc(430) and 
          not ( lc(341) or lc(342) or lc(343) or lc(429) or lc(431) );

  lcell(431) <= lc(431) and 
          not ( lc(342) or lc(343) or lc(344) );

  lcell(432) <= lc(432) and 
          not ( lc(343) or lc(344) or lc(345) or lc(431) or lc(433) );

  lcell(433) <= lc(433) and 
          not ( lc(344) or lc(345) or lc(346) );

  lcell(434) <= lc(434) and 
          not ( lc(345) or lc(346) or lc(347) or lc(433) or lc(435) );

  lcell(435) <= lc(435) and 
          not ( lc(346) or lc(347) or lc(348) );

  lcell(436) <= lc(436) and 
          not ( lc(347) or lc(348) or lc(349) or lc(435) or lc(437) );

  lcell(437) <= lc(437) and 
          not ( lc(348) or lc(349) or lc(350) );

  lcell(438) <= lc(438) and 
          not ( lc(349) or lc(350) or lc(351) or lc(437) or lc(439) );

  lcell(439) <= lc(439) and 
          not ( ll(308) or lc(350) or lc(351) );

  lcell(440) <= lc(440) and 
          not ( lc(352) or lc(353) or lr(395) or lc(441) or lr(483) );

  lcell(441) <= lc(441) and 
          not ( lc(352) or lc(353) or lc(354) );

  lcell(442) <= lc(442) and 
          not ( lc(353) or lc(354) or lc(355) or lc(441) or lc(443) );

  lcell(443) <= lc(443) and 
          not ( lc(354) or lc(355) or lc(356) );

  lcell(444) <= lc(444) and 
          not ( lc(355) or lc(356) or lc(357) or lc(443) or lc(445) );

  lcell(445) <= lc(445) and 
          not ( lc(356) or lc(357) or lc(358) );

  lcell(446) <= lc(446) and 
          not ( lc(357) or lc(358) or lc(359) or lc(445) or lc(447) );

  lcell(447) <= lc(447) and 
          not ( lc(358) or lc(359) or lc(360) );

  lcell(448) <= lc(448) and 
          not ( lc(359) or lc(360) or lc(361) or lc(447) or lc(449) );

  lcell(449) <= lc(449) and 
          not ( lc(360) or lc(361) or lc(362) );

  lcell(450) <= lc(450) and 
          not ( lc(361) or lc(362) or lc(363) or lc(449) or lc(451) );

  lcell(451) <= lc(451) and 
          not ( lc(362) or lc(363) or lc(364) );

  lcell(452) <= lc(452) and 
          not ( lc(363) or lc(364) or lc(365) or lc(451) or lc(453) );

  lcell(453) <= lc(453) and 
          not ( lc(364) or lc(365) or lc(366) );

  lcell(454) <= lc(454) and 
          not ( lc(365) or lc(366) or lc(367) or lc(453) or lc(455) );

  lcell(455) <= lc(455) and 
          not ( lc(366) or lc(367) or lc(368) );

  lcell(456) <= lc(456) and 
          not ( lc(367) or lc(368) or lc(369) or lc(455) or lc(457) );

  lcell(457) <= lc(457) and 
          not ( lc(368) or lc(369) or lc(370) );

  lcell(458) <= lc(458) and 
          not ( lc(369) or lc(370) or lc(371) or lc(457) or lc(459) );

  lcell(459) <= lc(459) and 
          not ( lc(370) or lc(371) or lc(372) );

  lcell(460) <= lc(460) and 
          not ( lc(371) or lc(372) or lc(373) or lc(459) or lc(461) );

  lcell(461) <= lc(461) and 
          not ( lc(372) or lc(373) or lc(374) );

  lcell(462) <= lc(462) and 
          not ( lc(373) or lc(374) or lc(375) or lc(461) or lc(463) );

  lcell(463) <= lc(463) and 
          not ( lc(374) or lc(375) or lc(376) );

  lcell(464) <= lc(464) and 
          not ( lc(375) or lc(376) or lc(377) or lc(463) or lc(465) );

  lcell(465) <= lc(465) and 
          not ( lc(376) or lc(377) or lc(378) );

  lcell(466) <= lc(466) and 
          not ( lc(377) or lc(378) or lc(379) or lc(465) or lc(467) );

  lcell(467) <= lc(467) and 
          not ( lc(378) or lc(379) or lc(380) );

  lcell(468) <= lc(468) and 
          not ( lc(379) or lc(380) or lc(381) or lc(467) or lc(469) );

  lcell(469) <= lc(469) and 
          not ( lc(380) or lc(381) or lc(382) );

  lcell(470) <= lc(470) and 
          not ( lc(381) or lc(382) or lc(383) or lc(469) or lc(471) );

  lcell(471) <= lc(471) and 
          not ( lc(382) or lc(383) or lc(384) );

  lcell(472) <= lc(472) and 
          not ( lc(383) or lc(384) or lc(385) or lc(471) or lc(473) );

  lcell(473) <= lc(473) and 
          not ( lc(384) or lc(385) or lc(386) );

  lcell(474) <= lc(474) and 
          not ( lc(385) or lc(386) or lc(387) or lc(473) or lc(475) );

  lcell(475) <= lc(475) and 
          not ( lc(386) or lc(387) or lc(388) );

  lcell(476) <= lc(476) and 
          not ( lc(387) or lc(388) or lc(389) or lc(475) or lc(477) );

  lcell(477) <= lc(477) and 
          not ( lc(388) or lc(389) or lc(390) );

  lcell(478) <= lc(478) and 
          not ( lc(389) or lc(390) or lc(391) or lc(477) or lc(479) );

  lcell(479) <= lc(479) and 
          not ( lc(390) or lc(391) or lc(392) );

  lcell(480) <= lc(480) and 
          not ( lc(391) or lc(392) or lc(393) or lc(479) or lc(481) );

  lcell(481) <= lc(481) and 
          not ( lc(392) or lc(393) or lc(394) );

  lcell(482) <= lc(482) and 
          not ( lc(393) or lc(394) or lc(395) or lc(481) or lc(483) );

  lcell(483) <= lc(483) and 
          not ( ll(352) or lc(394) or lc(395) );

  lcell(484) <= lc(484) and 
          not ( lc(396) or lc(397) or lr(439) or lc(485) or lr(527) );

  lcell(485) <= lc(485) and 
          not ( lc(396) or lc(397) or lc(398) );

  lcell(486) <= lc(486) and 
          not ( lc(397) or lc(398) or lc(399) or lc(485) or lc(487) );

  lcell(487) <= lc(487) and 
          not ( lc(398) or lc(399) or lc(400) );

  lcell(488) <= lc(488) and 
          not ( lc(399) or lc(400) or lc(401) or lc(487) or lc(489) );

  lcell(489) <= lc(489) and 
          not ( lc(400) or lc(401) or lc(402) );

  lcell(490) <= lc(490) and 
          not ( lc(401) or lc(402) or lc(403) or lc(489) or lc(491) );

  lcell(491) <= lc(491) and 
          not ( lc(402) or lc(403) or lc(404) );

  lcell(492) <= lc(492) and 
          not ( lc(403) or lc(404) or lc(405) or lc(491) or lc(493) );

  lcell(493) <= lc(493) and 
          not ( lc(404) or lc(405) or lc(406) );

  lcell(494) <= lc(494) and 
          not ( lc(405) or lc(406) or lc(407) or lc(493) or lc(495) );

  lcell(495) <= lc(495) and 
          not ( lc(406) or lc(407) or lc(408) );

  lcell(496) <= lc(496) and 
          not ( lc(407) or lc(408) or lc(409) or lc(495) or lc(497) );

  lcell(497) <= lc(497) and 
          not ( lc(408) or lc(409) or lc(410) );

  lcell(498) <= lc(498) and 
          not ( lc(409) or lc(410) or lc(411) or lc(497) or lc(499) );

  lcell(499) <= lc(499) and 
          not ( lc(410) or lc(411) or lc(412) );

  lcell(500) <= lc(500) and 
          not ( lc(411) or lc(412) or lc(413) or lc(499) or lc(501) );

  lcell(501) <= lc(501) and 
          not ( lc(412) or lc(413) or lc(414) );

  lcell(502) <= lc(502) and 
          not ( lc(413) or lc(414) or lc(415) or lc(501) or lc(503) );

  lcell(503) <= lc(503) and 
          not ( lc(414) or lc(415) or lc(416) );

  lcell(504) <= lc(504) and 
          not ( lc(415) or lc(416) or lc(417) or lc(503) or lc(505) );

  lcell(505) <= lc(505) and 
          not ( lc(416) or lc(417) or lc(418) );

  lcell(506) <= lc(506) and 
          not ( lc(417) or lc(418) or lc(419) or lc(505) or lc(507) );

  lcell(507) <= lc(507) and 
          not ( lc(418) or lc(419) or lc(420) );

  lcell(508) <= lc(508) and 
          not ( lc(419) or lc(420) or lc(421) or lc(507) or lc(509) );

  lcell(509) <= lc(509) and 
          not ( lc(420) or lc(421) or lc(422) );

  lcell(510) <= lc(510) and 
          not ( lc(421) or lc(422) or lc(423) or lc(509) or lc(511) );

  lcell(511) <= lc(511) and 
          not ( lc(422) or lc(423) or lc(424) );

  lcell(512) <= lc(512) and 
          not ( lc(423) or lc(424) or lc(425) or lc(511) or lc(513) );

  lcell(513) <= lc(513) and 
          not ( lc(424) or lc(425) or lc(426) );

  lcell(514) <= lc(514) and 
          not ( lc(425) or lc(426) or lc(427) or lc(513) or lc(515) );

  lcell(515) <= lc(515) and 
          not ( lc(426) or lc(427) or lc(428) );

  lcell(516) <= lc(516) and 
          not ( lc(427) or lc(428) or lc(429) or lc(515) or lc(517) );

  lcell(517) <= lc(517) and 
          not ( lc(428) or lc(429) or lc(430) );

  lcell(518) <= lc(518) and 
          not ( lc(429) or lc(430) or lc(431) or lc(517) or lc(519) );

  lcell(519) <= lc(519) and 
          not ( lc(430) or lc(431) or lc(432) );

  lcell(520) <= lc(520) and 
          not ( lc(431) or lc(432) or lc(433) or lc(519) or lc(521) );

  lcell(521) <= lc(521) and 
          not ( lc(432) or lc(433) or lc(434) );

  lcell(522) <= lc(522) and 
          not ( lc(433) or lc(434) or lc(435) or lc(521) or lc(523) );

  lcell(523) <= lc(523) and 
          not ( lc(434) or lc(435) or lc(436) );

  lcell(524) <= lc(524) and 
          not ( lc(435) or lc(436) or lc(437) or lc(523) or lc(525) );

  lcell(525) <= lc(525) and 
          not ( lc(436) or lc(437) or lc(438) );

  lcell(526) <= lc(526) and 
          not ( lc(437) or lc(438) or lc(439) or lc(525) or lc(527) );

  lcell(527) <= lc(527) and 
          not ( ll(396) or lc(438) or lc(439) );

  lcell(528) <= lc(528) and 
          not ( lc(440) or lc(441) or lr(483) or lc(529) or lr(571) );

  lcell(529) <= lc(529) and 
          not ( lc(440) or lc(441) or lc(442) );

  lcell(530) <= lc(530) and 
          not ( lc(441) or lc(442) or lc(443) or lc(529) or lc(531) );

  lcell(531) <= lc(531) and 
          not ( lc(442) or lc(443) or lc(444) );

  lcell(532) <= lc(532) and 
          not ( lc(443) or lc(444) or lc(445) or lc(531) or lc(533) );

  lcell(533) <= lc(533) and 
          not ( lc(444) or lc(445) or lc(446) );

  lcell(534) <= lc(534) and 
          not ( lc(445) or lc(446) or lc(447) or lc(533) or lc(535) );

  lcell(535) <= lc(535) and 
          not ( lc(446) or lc(447) or lc(448) );

  lcell(536) <= lc(536) and 
          not ( lc(447) or lc(448) or lc(449) or lc(535) or lc(537) );

  lcell(537) <= lc(537) and 
          not ( lc(448) or lc(449) or lc(450) );

  lcell(538) <= lc(538) and 
          not ( lc(449) or lc(450) or lc(451) or lc(537) or lc(539) );

  lcell(539) <= lc(539) and 
          not ( lc(450) or lc(451) or lc(452) );

  lcell(540) <= lc(540) and 
          not ( lc(451) or lc(452) or lc(453) or lc(539) or lc(541) );

  lcell(541) <= lc(541) and 
          not ( lc(452) or lc(453) or lc(454) );

  lcell(542) <= lc(542) and 
          not ( lc(453) or lc(454) or lc(455) or lc(541) or lc(543) );

  lcell(543) <= lc(543) and 
          not ( lc(454) or lc(455) or lc(456) );

  lcell(544) <= lc(544) and 
          not ( lc(455) or lc(456) or lc(457) or lc(543) or lc(545) );

  lcell(545) <= lc(545) and 
          not ( lc(456) or lc(457) or lc(458) );

  lcell(546) <= lc(546) and 
          not ( lc(457) or lc(458) or lc(459) or lc(545) or lc(547) );

  lcell(547) <= lc(547) and 
          not ( lc(458) or lc(459) or lc(460) );

  lcell(548) <= lc(548) and 
          not ( lc(459) or lc(460) or lc(461) or lc(547) or lc(549) );

  lcell(549) <= lc(549) and 
          not ( lc(460) or lc(461) or lc(462) );

  lcell(550) <= lc(550) and 
          not ( lc(461) or lc(462) or lc(463) or lc(549) or lc(551) );

  lcell(551) <= lc(551) and 
          not ( lc(462) or lc(463) or lc(464) );

  lcell(552) <= lc(552) and 
          not ( lc(463) or lc(464) or lc(465) or lc(551) or lc(553) );

  lcell(553) <= lc(553) and 
          not ( lc(464) or lc(465) or lc(466) );

  lcell(554) <= lc(554) and 
          not ( lc(465) or lc(466) or lc(467) or lc(553) or lc(555) );

  lcell(555) <= lc(555) and 
          not ( lc(466) or lc(467) or lc(468) );

  lcell(556) <= lc(556) and 
          not ( lc(467) or lc(468) or lc(469) or lc(555) or lc(557) );

  lcell(557) <= lc(557) and 
          not ( lc(468) or lc(469) or lc(470) );

  lcell(558) <= lc(558) and 
          not ( lc(469) or lc(470) or lc(471) or lc(557) or lc(559) );

  lcell(559) <= lc(559) and 
          not ( lc(470) or lc(471) or lc(472) );

  lcell(560) <= lc(560) and 
          not ( lc(471) or lc(472) or lc(473) or lc(559) or lc(561) );

  lcell(561) <= lc(561) and 
          not ( lc(472) or lc(473) or lc(474) );

  lcell(562) <= lc(562) and 
          not ( lc(473) or lc(474) or lc(475) or lc(561) or lc(563) );

  lcell(563) <= lc(563) and 
          not ( lc(474) or lc(475) or lc(476) );

  lcell(564) <= lc(564) and 
          not ( lc(475) or lc(476) or lc(477) or lc(563) or lc(565) );

  lcell(565) <= lc(565) and 
          not ( lc(476) or lc(477) or lc(478) );

  lcell(566) <= lc(566) and 
          not ( lc(477) or lc(478) or lc(479) or lc(565) or lc(567) );

  lcell(567) <= lc(567) and 
          not ( lc(478) or lc(479) or lc(480) );

  lcell(568) <= lc(568) and 
          not ( lc(479) or lc(480) or lc(481) or lc(567) or lc(569) );

  lcell(569) <= lc(569) and 
          not ( lc(480) or lc(481) or lc(482) );

  lcell(570) <= lc(570) and 
          not ( lc(481) or lc(482) or lc(483) or lc(569) or lc(571) );

  lcell(571) <= lc(571) and 
          not ( ll(440) or lc(482) or lc(483) );

  lcell(572) <= lc(572) and 
          not ( lc(484) or lc(485) or lr(527) or lc(573) or lr(615) );

  lcell(573) <= lc(573) and 
          not ( lc(484) or lc(485) or lc(486) );

  lcell(574) <= lc(574) and 
          not ( lc(485) or lc(486) or lc(487) or lc(573) or lc(575) );

  lcell(575) <= lc(575) and 
          not ( lc(486) or lc(487) or lc(488) );

  lcell(576) <= lc(576) and 
          not ( lc(487) or lc(488) or lc(489) or lc(575) or lc(577) );

  lcell(577) <= lc(577) and 
          not ( lc(488) or lc(489) or lc(490) );

  lcell(578) <= lc(578) and 
          not ( lc(489) or lc(490) or lc(491) or lc(577) or lc(579) );

  lcell(579) <= lc(579) and 
          not ( lc(490) or lc(491) or lc(492) );

  lcell(580) <= lc(580) and 
          not ( lc(491) or lc(492) or lc(493) or lc(579) or lc(581) );

  lcell(581) <= lc(581) and 
          not ( lc(492) or lc(493) or lc(494) );

  lcell(582) <= lc(582) and 
          not ( lc(493) or lc(494) or lc(495) or lc(581) or lc(583) );

  lcell(583) <= lc(583) and 
          not ( lc(494) or lc(495) or lc(496) );

  lcell(584) <= lc(584) and 
          not ( lc(495) or lc(496) or lc(497) or lc(583) or lc(585) );

  lcell(585) <= lc(585) and 
          not ( lc(496) or lc(497) or lc(498) );

  lcell(586) <= lc(586) and 
          not ( lc(497) or lc(498) or lc(499) or lc(585) or lc(587) );

  lcell(587) <= lc(587) and 
          not ( lc(498) or lc(499) or lc(500) );

  lcell(588) <= lc(588) and 
          not ( lc(499) or lc(500) or lc(501) or lc(587) or lc(589) );

  lcell(589) <= lc(589) and 
          not ( lc(500) or lc(501) or lc(502) );

  lcell(590) <= lc(590) and 
          not ( lc(501) or lc(502) or lc(503) or lc(589) or lc(591) );

  lcell(591) <= lc(591) and 
          not ( lc(502) or lc(503) or lc(504) );

  lcell(592) <= lc(592) and 
          not ( lc(503) or lc(504) or lc(505) or lc(591) or lc(593) );

  lcell(593) <= lc(593) and 
          not ( lc(504) or lc(505) or lc(506) );

  lcell(594) <= lc(594) and 
          not ( lc(505) or lc(506) or lc(507) or lc(593) or lc(595) );

  lcell(595) <= lc(595) and 
          not ( lc(506) or lc(507) or lc(508) );

  lcell(596) <= lc(596) and 
          not ( lc(507) or lc(508) or lc(509) or lc(595) or lc(597) );

  lcell(597) <= lc(597) and 
          not ( lc(508) or lc(509) or lc(510) );

  lcell(598) <= lc(598) and 
          not ( lc(509) or lc(510) or lc(511) or lc(597) or lc(599) );

  lcell(599) <= lc(599) and 
          not ( lc(510) or lc(511) or lc(512) );

  lcell(600) <= lc(600) and 
          not ( lc(511) or lc(512) or lc(513) or lc(599) or lc(601) );

  lcell(601) <= lc(601) and 
          not ( lc(512) or lc(513) or lc(514) );

  lcell(602) <= lc(602) and 
          not ( lc(513) or lc(514) or lc(515) or lc(601) or lc(603) );

  lcell(603) <= lc(603) and 
          not ( lc(514) or lc(515) or lc(516) );

  lcell(604) <= lc(604) and 
          not ( lc(515) or lc(516) or lc(517) or lc(603) or lc(605) );

  lcell(605) <= lc(605) and 
          not ( lc(516) or lc(517) or lc(518) );

  lcell(606) <= lc(606) and 
          not ( lc(517) or lc(518) or lc(519) or lc(605) or lc(607) );

  lcell(607) <= lc(607) and 
          not ( lc(518) or lc(519) or lc(520) );

  lcell(608) <= lc(608) and 
          not ( lc(519) or lc(520) or lc(521) or lc(607) or lc(609) );

  lcell(609) <= lc(609) and 
          not ( lc(520) or lc(521) or lc(522) );

  lcell(610) <= lc(610) and 
          not ( lc(521) or lc(522) or lc(523) or lc(609) or lc(611) );

  lcell(611) <= lc(611) and 
          not ( lc(522) or lc(523) or lc(524) );

  lcell(612) <= lc(612) and 
          not ( lc(523) or lc(524) or lc(525) or lc(611) or lc(613) );

  lcell(613) <= lc(613) and 
          not ( lc(524) or lc(525) or lc(526) );

  lcell(614) <= lc(614) and 
          not ( lc(525) or lc(526) or lc(527) or lc(613) or lc(615) );

  lcell(615) <= lc(615) and 
          not ( ll(484) or lc(526) or lc(527) );

  lcell(616) <= lc(616) and 
          not ( lc(528) or lc(529) or lr(571) or lc(617) or lr(659) );

  lcell(617) <= lc(617) and 
          not ( lc(528) or lc(529) or lc(530) );

  lcell(618) <= lc(618) and 
          not ( lc(529) or lc(530) or lc(531) or lc(617) or lc(619) );

  lcell(619) <= lc(619) and 
          not ( lc(530) or lc(531) or lc(532) );

  lcell(620) <= lc(620) and 
          not ( lc(531) or lc(532) or lc(533) or lc(619) or lc(621) );

  lcell(621) <= lc(621) and 
          not ( lc(532) or lc(533) or lc(534) );

  lcell(622) <= lc(622) and 
          not ( lc(533) or lc(534) or lc(535) or lc(621) or lc(623) );

  lcell(623) <= lc(623) and 
          not ( lc(534) or lc(535) or lc(536) );

  lcell(624) <= lc(624) and 
          not ( lc(535) or lc(536) or lc(537) or lc(623) or lc(625) );

  lcell(625) <= lc(625) and 
          not ( lc(536) or lc(537) or lc(538) );

  lcell(626) <= lc(626) and 
          not ( lc(537) or lc(538) or lc(539) or lc(625) or lc(627) );

  lcell(627) <= lc(627) and 
          not ( lc(538) or lc(539) or lc(540) );

  lcell(628) <= lc(628) and 
          not ( lc(539) or lc(540) or lc(541) or lc(627) or lc(629) );

  lcell(629) <= lc(629) and 
          not ( lc(540) or lc(541) or lc(542) );

  lcell(630) <= lc(630) and 
          not ( lc(541) or lc(542) or lc(543) or lc(629) or lc(631) );

  lcell(631) <= lc(631) and 
          not ( lc(542) or lc(543) or lc(544) );

  lcell(632) <= lc(632) and 
          not ( lc(543) or lc(544) or lc(545) or lc(631) or lc(633) );

  lcell(633) <= lc(633) and 
          not ( lc(544) or lc(545) or lc(546) );

  lcell(634) <= lc(634) and 
          not ( lc(545) or lc(546) or lc(547) or lc(633) or lc(635) );

  lcell(635) <= lc(635) and 
          not ( lc(546) or lc(547) or lc(548) );

  lcell(636) <= lc(636) and 
          not ( lc(547) or lc(548) or lc(549) or lc(635) or lc(637) );

  lcell(637) <= lc(637) and 
          not ( lc(548) or lc(549) or lc(550) );

  lcell(638) <= lc(638) and 
          not ( lc(549) or lc(550) or lc(551) or lc(637) or lc(639) );

  lcell(639) <= lc(639) and 
          not ( lc(550) or lc(551) or lc(552) );

  lcell(640) <= lc(640) and 
          not ( lc(551) or lc(552) or lc(553) or lc(639) or lc(641) );

  lcell(641) <= lc(641) and 
          not ( lc(552) or lc(553) or lc(554) );

  lcell(642) <= lc(642) and 
          not ( lc(553) or lc(554) or lc(555) or lc(641) or lc(643) );

  lcell(643) <= lc(643) and 
          not ( lc(554) or lc(555) or lc(556) );

  lcell(644) <= lc(644) and 
          not ( lc(555) or lc(556) or lc(557) or lc(643) or lc(645) );

  lcell(645) <= lc(645) and 
          not ( lc(556) or lc(557) or lc(558) );

  lcell(646) <= lc(646) and 
          not ( lc(557) or lc(558) or lc(559) or lc(645) or lc(647) );

  lcell(647) <= lc(647) and 
          not ( lc(558) or lc(559) or lc(560) );

  lcell(648) <= lc(648) and 
          not ( lc(559) or lc(560) or lc(561) or lc(647) or lc(649) );

  lcell(649) <= lc(649) and 
          not ( lc(560) or lc(561) or lc(562) );

  lcell(650) <= lc(650) and 
          not ( lc(561) or lc(562) or lc(563) or lc(649) or lc(651) );

  lcell(651) <= lc(651) and 
          not ( lc(562) or lc(563) or lc(564) );

  lcell(652) <= lc(652) and 
          not ( lc(563) or lc(564) or lc(565) or lc(651) or lc(653) );

  lcell(653) <= lc(653) and 
          not ( lc(564) or lc(565) or lc(566) );

  lcell(654) <= lc(654) and 
          not ( lc(565) or lc(566) or lc(567) or lc(653) or lc(655) );

  lcell(655) <= lc(655) and 
          not ( lc(566) or lc(567) or lc(568) );

  lcell(656) <= lc(656) and 
          not ( lc(567) or lc(568) or lc(569) or lc(655) or lc(657) );

  lcell(657) <= lc(657) and 
          not ( lc(568) or lc(569) or lc(570) );

  lcell(658) <= lc(658) and 
          not ( lc(569) or lc(570) or lc(571) or lc(657) or lc(659) );

  lcell(659) <= lc(659) and 
          not ( ll(528) or lc(570) or lc(571) );

  lcell(660) <= lc(660) and 
          not ( lc(572) or lc(573) or lr(615) or lc(661) or lr(703) );

  lcell(661) <= lc(661) and 
          not ( lc(572) or lc(573) or lc(574) );

  lcell(662) <= lc(662) and 
          not ( lc(573) or lc(574) or lc(575) or lc(661) or lc(663) );

  lcell(663) <= lc(663) and 
          not ( lc(574) or lc(575) or lc(576) );

  lcell(664) <= lc(664) and 
          not ( lc(575) or lc(576) or lc(577) or lc(663) or lc(665) );

  lcell(665) <= lc(665) and 
          not ( lc(576) or lc(577) or lc(578) );

  lcell(666) <= lc(666) and 
          not ( lc(577) or lc(578) or lc(579) or lc(665) or lc(667) );

  lcell(667) <= lc(667) and 
          not ( lc(578) or lc(579) or lc(580) );

  lcell(668) <= lc(668) and 
          not ( lc(579) or lc(580) or lc(581) or lc(667) or lc(669) );

  lcell(669) <= lc(669) and 
          not ( lc(580) or lc(581) or lc(582) );

  lcell(670) <= lc(670) and 
          not ( lc(581) or lc(582) or lc(583) or lc(669) or lc(671) );

  lcell(671) <= lc(671) and 
          not ( lc(582) or lc(583) or lc(584) );

  lcell(672) <= lc(672) and 
          not ( lc(583) or lc(584) or lc(585) or lc(671) or lc(673) );

  lcell(673) <= lc(673) and 
          not ( lc(584) or lc(585) or lc(586) );

  lcell(674) <= lc(674) and 
          not ( lc(585) or lc(586) or lc(587) or lc(673) or lc(675) );

  lcell(675) <= lc(675) and 
          not ( lc(586) or lc(587) or lc(588) );

  lcell(676) <= lc(676) and 
          not ( lc(587) or lc(588) or lc(589) or lc(675) or lc(677) );

  lcell(677) <= lc(677) and 
          not ( lc(588) or lc(589) or lc(590) );

  lcell(678) <= lc(678) and 
          not ( lc(589) or lc(590) or lc(591) or lc(677) or lc(679) );

  lcell(679) <= lc(679) and 
          not ( lc(590) or lc(591) or lc(592) );

  lcell(680) <= lc(680) and 
          not ( lc(591) or lc(592) or lc(593) or lc(679) or lc(681) );

  lcell(681) <= lc(681) and 
          not ( lc(592) or lc(593) or lc(594) );

  lcell(682) <= lc(682) and 
          not ( lc(593) or lc(594) or lc(595) or lc(681) or lc(683) );

  lcell(683) <= lc(683) and 
          not ( lc(594) or lc(595) or lc(596) );

  lcell(684) <= lc(684) and 
          not ( lc(595) or lc(596) or lc(597) or lc(683) or lc(685) );

  lcell(685) <= lc(685) and 
          not ( lc(596) or lc(597) or lc(598) );

  lcell(686) <= lc(686) and 
          not ( lc(597) or lc(598) or lc(599) or lc(685) or lc(687) );

  lcell(687) <= lc(687) and 
          not ( lc(598) or lc(599) or lc(600) );

  lcell(688) <= lc(688) and 
          not ( lc(599) or lc(600) or lc(601) or lc(687) or lc(689) );

  lcell(689) <= lc(689) and 
          not ( lc(600) or lc(601) or lc(602) );

  lcell(690) <= lc(690) and 
          not ( lc(601) or lc(602) or lc(603) or lc(689) or lc(691) );

  lcell(691) <= lc(691) and 
          not ( lc(602) or lc(603) or lc(604) );

  lcell(692) <= lc(692) and 
          not ( lc(603) or lc(604) or lc(605) or lc(691) or lc(693) );

  lcell(693) <= lc(693) and 
          not ( lc(604) or lc(605) or lc(606) );

  lcell(694) <= lc(694) and 
          not ( lc(605) or lc(606) or lc(607) or lc(693) or lc(695) );

  lcell(695) <= lc(695) and 
          not ( lc(606) or lc(607) or lc(608) );

  lcell(696) <= lc(696) and 
          not ( lc(607) or lc(608) or lc(609) or lc(695) or lc(697) );

  lcell(697) <= lc(697) and 
          not ( lc(608) or lc(609) or lc(610) );

  lcell(698) <= lc(698) and 
          not ( lc(609) or lc(610) or lc(611) or lc(697) or lc(699) );

  lcell(699) <= lc(699) and 
          not ( lc(610) or lc(611) or lc(612) );

  lcell(700) <= lc(700) and 
          not ( lc(611) or lc(612) or lc(613) or lc(699) or lc(701) );

  lcell(701) <= lc(701) and 
          not ( lc(612) or lc(613) or lc(614) );

  lcell(702) <= lc(702) and 
          not ( lc(613) or lc(614) or lc(615) or lc(701) or lc(703) );

  lcell(703) <= lc(703) and 
          not ( ll(572) or lc(614) or lc(615) );

end generalveto_arch; 
